@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -92,6 +92,15 @@ const InfinotedParameterInfo INFINOTED_OPTIONS[] = {
     'p',
     N_("The TCP port number to listen on."),
     N_("PORT")
+  }, {
+    "listen-address",
+    INFINOTED_PARAMETER_STRING,
+    0,
+    offsetof(InfinotedOptions, listen_address),
+    infinoted_parameter_convert_ip_address,
+    0,
+    N_("The IP address to listen on."),
+    N_("ADDRESS"),
   }, {
     "security-policy",
     INFINOTED_PARAMETER_STRING,
@@ -199,11 +208,10 @@ infinoted_options_validate(InfinotedOptions* options,
 #ifdef LIBINFINITY_HAVE_PAM
   if(options->password != NULL && options->pam_service != NULL)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       infinoted_options_error_quark(),
       INFINOTED_OPTIONS_ERROR_INVALID_AUTHENTICATION_SETTINGS,
-      "%s",
       _("Cannot use both server password and system authentication.")
     );
     return FALSE;
@@ -213,11 +221,10 @@ infinoted_options_validate(InfinotedOptions* options,
      && (options->pam_allowed_users != NULL
          || options->pam_allowed_groups != NULL))
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       infinoted_options_error_quark(),
       INFINOTED_OPTIONS_ERROR_INVALID_AUTHENTICATION_SETTINGS,
-      "%s",
       _("Need a pam service to authenticate users.")
     );
     return FALSE;
@@ -244,11 +251,10 @@ infinoted_options_validate(InfinotedOptions* options,
 
   if(options->create_key == TRUE && options->create_certificate == FALSE)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       infinoted_options_error_quark(),
       INFINOTED_OPTIONS_ERROR_INVALID_CREATE_OPTIONS,
-      "%s",
       _("Creating a new private key also requires creating a new certificate "
         "signed with it.")
     );
@@ -258,11 +264,10 @@ infinoted_options_validate(InfinotedOptions* options,
   else if(security_policy != INF_XMPP_CONNECTION_SECURITY_ONLY_UNSECURED && 
           options->key_file == NULL)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       infinoted_options_error_quark(),
       INFINOTED_OPTIONS_ERROR_EMPTY_KEY_FILE,
-      "%s",
       _("No private key file given. If you don't have a suitable key file, "
         "either create one using the --create-key command line argument, "
         "or disable TLS by setting the security policy to \"no-tls\".")
@@ -273,11 +278,10 @@ infinoted_options_validate(InfinotedOptions* options,
   else if(security_policy != INF_XMPP_CONNECTION_SECURITY_ONLY_UNSECURED &&
           options->certificate_file == NULL)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       infinoted_options_error_quark(),
       INFINOTED_OPTIONS_ERROR_EMPTY_CERTIFICATE_FILE,
-      "%s",
       _("No certificate file given. If you don't have a suitable certificate "
         "file, either create one using the --create-certificate command line "
         "agument, or disable TLS via by setting the security policy to "
@@ -967,6 +971,7 @@ infinoted_options_new(const gchar* const* config_files,
   options->create_key = FALSE;
   options->create_certificate = FALSE;
   options->port = inf_protocol_get_default_port();
+  options->listen_address = NULL;
   options->security_policy = INF_XMPP_CONNECTION_SECURITY_ONLY_TLS;
   options->root_directory =
     g_build_filename(g_get_home_dir(), ".infinote", NULL);
@@ -1008,6 +1013,8 @@ infinoted_options_free(InfinotedOptions* options)
   g_free(options->certificate_file);
   g_free(options->certificate_chain_file);
   g_free(options->root_directory);
+  if(options->listen_address != NULL)
+    inf_ip_address_free(options->listen_address);
   g_strfreev(options->plugins);
   g_free(options->password);
 #ifdef LIBINFINITY_HAVE_PAM
