@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -42,6 +42,8 @@
 
 #include <string.h>
 
+G_DEFINE_BOXED_TYPE(InfinotedParameterTypedValue, infinoted_parameter_typed_value, infinoted_parameter_typed_value_copy, infinoted_parameter_typed_value_free)
+
 static void
 infinoted_parameter_free_data(InfinotedParameterType type,
                               InfinotedParameterValue* value)
@@ -199,6 +201,44 @@ infinoted_parameter_typed_value_new(void)
   return g_slice_new(InfinotedParameterTypedValue);
 }
 
+/**
+ * infinoted_parameter_typed_value_copy:
+ * @val: The value to copy.
+ *
+ * Makes a dynamically allocated copy of @val.
+ *
+ * Returns: A new #InfinotedParameterTypedValue. Free with
+ * infinoted_parameter_typed_value_free() when no longer needed.
+ */
+InfinotedParameterTypedValue*
+infinoted_parameter_typed_value_copy(const InfinotedParameterTypedValue* val)
+{
+  InfinotedParameterTypedValue* v;
+  v = g_slice_new(InfinotedParameterTypedValue);
+  v->type = val->type;
+
+  switch(v->type)
+  {
+  case INFINOTED_PARAMETER_BOOLEAN:
+    v->value.yesno = val->value.yesno;
+    break;
+  case INFINOTED_PARAMETER_INT:
+    v->value.number = val->value.number;
+    break;
+  case INFINOTED_PARAMETER_STRING:
+    v->value.str = g_strdup(val->value.str);
+    break;
+  case INFINOTED_PARAMETER_STRING_LIST:
+    v->value.strv = g_strdupv(val->value.strv);
+    break;
+  default:
+    g_assert_not_reached();
+    break;
+  }
+
+  return v;
+}
+
 /**
  * infinoted_parameter_typed_value_free:
  * @data: The #InfinotedParameterTypedValue to free.
@@ -222,7 +262,8 @@ infinoted_parameter_typed_value_free(gpointer data)
 
 /**
  * infinoted_parameter_load_from_key_file:
- * @infos: A 0-terminated array of #InfinotedParameterInfo objects.
+ * @infos: (array zero-terminated=1): A 0-terminated array of
+ * #InfinotedParameterInfo objects.
  * @key_file: The #GKeyFile to load parameter values from.
  * @group: The keyfile group to load the values from.
  * @base: The instance into which to write the read parameters.
@@ -274,8 +315,8 @@ infinoted_parameter_load_from_key_file(const InfinotedParameterInfo* infos,
 
 /**
  * infinoted_parameter_convert_string:
- * @out: The pointer to the output string location.
- * @in: A pointer to the input string location.
+ * @out: (type gchar**) (out): The pointer to the output string location.
+ * @in: (type gchar**) (in): A pointer to the input string location.
  * @error: Location to store error information, if any, or %NULL.
  *
  * This is basically a no-op, moving the string from the @in location to the
@@ -317,8 +358,8 @@ infinoted_parameter_convert_string(gpointer out,
 
 /**
  * infinoted_parameter_convert_string_list:
- * @out: The pointer to the output string list.
- * @in: The pointer to the input string list.
+ * @out: (type gchar***) (out): The pointer to the output string list.
+ * @in: (type gchar***) (array zero-terminated=1) (in): The pointer to the input string list.
  * @error: Location to store error information, if any, or %NULL.
  *
  * This is basically a no-op, moving the string list from the @in location to
@@ -374,8 +415,8 @@ infinoted_parameter_convert_string_list(gpointer out,
 
 /**
  * infinoted_parameter_convert_filename:
- * @out: The pointer to the output string location.
- * @in: A pointer to the input string location.
+ * @out: (type gchar**) (out): The pointer to the output string location.
+ * @in: (type gchar**) (in): A pointer to the input string location.
  * @error: Location to store error information, if any, or %NULL.
  *
  * This function converts the input string from UTF-8
@@ -418,8 +459,8 @@ infinoted_parameter_convert_filename(gpointer out,
 
 /**
  * infinoted_parameter_convert_boolean:
- * @out: The pointer to the output #gboolean.
- * @in: The pointer to the input #gboolean.
+ * @out: (type gboolean*) (out): The pointer to the output #gboolean.
+ * @in: (type gboolean*) (in): The pointer to the input #gboolean.
  * @error: Location to store error information, if any, or %NULL.
  *
  * This function simply writes the boolean value from @in to @out without any
@@ -443,8 +484,8 @@ infinoted_parameter_convert_boolean(gpointer out,
 
 /**
  * infinoted_parameter_convert_port:
- * @out: The pointer to the output #guint.
- * @in: The pointer to the input #gint.
+ * @out: (type guint*) (out): The pointer to the output #guint.
+ * @in: (type gint*) (in): The pointer to the input #gint.
  * @error: Location to store error information, if any, or %NULL.
  *
  * This function validates the input number to be in the valid range for
@@ -483,10 +524,56 @@ infinoted_parameter_convert_port(gpointer out,
   return TRUE;
 }
 
+/**
+ * infinoted_parameter_convert_ip_address:
+ * @out: (type InfIpAddress**) (out): The pointer to the output #InfIpAddress
+ * location.
+ * @in: (type gchar**) (in): The pointer to the input string location.
+ * @error: Location to store error information, if any, or %NULL.
+ *
+ * Converts the string that @in points to to an #InfIpAddress* value. If the
+ * string can not be converted to an IP address, the functions fails and @error
+ * is set.
+ *
+ * This is a #InfinotedParameterConvertFunc function that can be used for
+ * fields of type #InfIpAddress*.
+ *
+ * Returns: %TRUE on success, or %FALSE otherwise.
+ */
+gboolean
+infinoted_parameter_convert_ip_address(gpointer out,
+                                       gpointer in,
+                                       GError** error)
+{
+  gchar** in_str;
+  InfIpAddress** out_val;
+
+  in_str = (gchar**) in;
+  out_val = (InfIpAddress**) out;
+
+  InfIpAddress *address = inf_ip_address_new_from_string(*in_str);
+
+  if(address == NULL)
+  {
+    g_set_error_literal(
+      error,
+      infinoted_parameter_error_quark(),
+      INFINOTED_PARAMETER_ERROR_INVALID_IP_ADDRESS,
+      _("Invalid IP address")
+    );
+
+    return FALSE;
+  }
+
+  *out_val = address;
+
+  return TRUE;
+}
+
 /**
  * infinoted_parameter_convert_nonnegative:
- * @out: The pointer to the output #guint.
- * @in: The pointer to the input #gint.
+ * @out: (type guint*) (out): The pointer to the output #guint.
+ * @in: (type gint*) (in): The pointer to the input #gint.
  * @error: Location to store error information, if any, or %NULL.
  *
  * This function validates the input number to be non-negative, and
@@ -507,11 +594,10 @@ infinoted_parameter_convert_nonnegative(gpointer out,
 
   if(number < 0)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       infinoted_parameter_error_quark(),
       INFINOTED_PARAMETER_ERROR_INVALID_NUMBER,
-      "%s",
       _("Number must not be negative")
     );
 
@@ -525,8 +611,8 @@ infinoted_parameter_convert_nonnegative(gpointer out,
 
 /**
  * infinoted_parameter_convert_positive:
- * @out: The pointer to the output #guint.
- * @in: The pointer to the input #gint.
+ * @out: (type guint*) (out): The pointer to the output #guint.
+ * @in: (type gint*) (in): The pointer to the input #gint.
  * @error: Location to store error information, if any, or %NULL.
  *
  * This function validates the input number to be positve, i.e. greater than
@@ -547,11 +633,10 @@ infinoted_parameter_convert_positive(gpointer out,
 
   if(number <= 0)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       infinoted_parameter_error_quark(),
       INFINOTED_PARAMETER_ERROR_INVALID_NUMBER,
-      "%s",
       _("Number must be positive")
     );
 
@@ -565,8 +650,9 @@ infinoted_parameter_convert_positive(gpointer out,
 
 /**
  * infinoted_parameter_convert_security_policy:
- * @out: The pointer to the output #InfXmppConnectionSecurityPolicy.
- * @in: The pointer to the input string location.
+ * @out: (type InfXmppConnectionSecurityPolicy*) (out): The pointer to the
+ * output #InfXmppConnectionSecurityPolicy.
+ * @in: (type gchar**) (in): The pointer to the input string location.
  * @error: Location to store error information, if any, or %NULL.
  *
  * Converts the string that @in points to to an
@@ -619,4 +705,81 @@ infinoted_parameter_convert_security_policy(gpointer out,
   return TRUE;
 }
 
+/**
+ * infinoted_parameter_convert_flags:
+ * @out: (type gint*) (out): The pointer to the output flags (a #gint).
+ * @in: (type gchar***) (in) (array zero-terminated=1): The pointer to the
+ * input string list.
+ * @values: Allowed flag values.
+ * @error: Location to store error information, if any, or %NULL.
+ *
+ * Converts the string list that @in points to to a bitmask. This function
+ * can not directly be used as a convert function as expected by
+ * #InfinotedParameterInfo since it has an additional argument. However, an
+ * actual convert function can make use of this function.
+ *
+ * Each string entry is interpreted as a value of a bitmask. The @values
+ * list specifies which string constant corresponds to which flag value.
+ *
+ * Returns: %TRUE on success, or %FALSE otherwise.
+ */
+gboolean
+infinoted_parameter_convert_flags(gpointer out,
+                                  gpointer in,
+                                  const GFlagsValue* values,
+                                  GError** error)
+{
+  GString* error_string;
+  const GFlagsValue* value;
+  gchar*** in_str;
+  gchar** cur;
+
+  *(gint*)out = 0;
+  in_str = (gchar***)in;
+
+  if(*in_str != NULL)
+  {
+    for(cur = *in_str; *cur != NULL; ++cur)
+    {
+      if( (*cur)[0] == '\0') continue;
+
+      for(value = values; value->value_name != NULL; ++value)
+      {
+        if(strcmp(*cur, value->value_nick) == 0 ||
+           strcmp(*cur, value->value_name) == 0)
+        {
+          break;
+        }
+      }
+
+      if(value->value_name == NULL)
+      {
+        error_string = g_string_sized_new(256);
+        for(value = values; value->value_name != NULL; ++value)
+        {
+          if(error_string->len > 0)
+            g_string_append(error_string, ", ");
+          g_string_append(error_string, value->value_nick);
+        }
+
+        g_set_error(
+          error,
+          infinoted_parameter_error_quark(),
+          INFINOTED_PARAMETER_ERROR_INVALID_FLAG,
+          _("\"%s\" is not a valid value. Allowed values are: %s."),
+          *cur,
+          error_string->str
+        );
+
+        g_string_free(error_string, TRUE);
+        return FALSE;
+      }
+
+      *(gint*)out |= value->value;
+    }
+  }
+
+  return TRUE;
+}
+
 /* vim:set et sw=2 ts=2: */
