@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -51,23 +51,6 @@ infinoted_run_load_directory(InfinotedRun* run,
 
   storage = infd_filesystem_storage_new(startup->options->root_directory);
 
-  /* Set an account storage, too. This can be replaced by something more fancy
-   * by a plugin. */
-  account_storage = infd_filesystem_account_storage_new();
-
-  result = infd_filesystem_account_storage_set_filesystem(
-    account_storage,
-    storage,
-    error
-  );
-
-  if(result == FALSE)
-  {
-    g_object_unref(account_storage);
-    g_object_unref(storage);
-    return FALSE;
-  }
-
   communication_manager = inf_communication_manager_new();
 
   run->io = inf_standalone_io_new();
@@ -78,17 +61,8 @@ infinoted_run_load_directory(InfinotedRun* run,
     communication_manager
   );
 
-  g_object_set(
-    G_OBJECT(run->directory),
-    "account-storage",
-    account_storage,
-    NULL
-  );
-
   infd_directory_enable_chat(run->directory, TRUE);
 
-  g_object_unref(storage);
-  g_object_unref(account_storage);
   g_object_unref(communication_manager);
 
   /* Load server plugins via plugin manager */
@@ -119,6 +93,8 @@ infinoted_run_load_directory(InfinotedRun* run,
 
   if(result == FALSE)
   {
+    g_object_unref(storage);
+
     g_object_unref(run->plugin_manager);
     g_object_unref(run->directory);
     g_object_unref(run->io);
@@ -128,6 +104,47 @@ infinoted_run_load_directory(InfinotedRun* run,
     return FALSE;
   }
 
+  /* Set an account storage, too. This can be replaced by something more fancy
+   * by a plugin. */
+  /* TODO: This filesystem storage should be moved into a plugin, so that
+   * users can enable it optionally. */
+  g_object_get(G_OBJECT(run->directory), "account-storage", &account_storage, NULL);
+  if(account_storage == NULL)
+  {
+    account_storage = infd_filesystem_account_storage_new();
+
+    result = infd_filesystem_account_storage_set_filesystem(
+      account_storage,
+      storage,
+      error
+    );
+
+    if(result == FALSE)
+    {
+      g_object_unref(storage);
+      g_object_unref(account_storage);
+
+      g_object_unref(run->plugin_manager);
+      g_object_unref(run->directory);
+      g_object_unref(run->io);
+      run->plugin_manager = NULL;
+      run->directory = NULL;
+      run->io = NULL;
+
+      return FALSE;
+    }
+
+    g_object_set(
+      G_OBJECT(run->directory),
+      "account-storage",
+      account_storage,
+      NULL
+    );
+  }
+
+  g_object_unref(storage);
+  g_object_unref(account_storage);
+
   return TRUE;
 }
 
@@ -150,6 +167,8 @@ infinoted_run_create_server(InfinotedRun* run,
     )
   );
 
+  infd_tcp_server_set_keepalive(tcp, &startup->keepalive);
+
   if(!infd_tcp_server_bind(tcp, error))
   {
     g_object_unref(tcp);
@@ -230,18 +249,40 @@ infinoted_run_new(InfinotedStartup* startup,
   g_object_unref(xmpp_manager);
 #endif
 
-  address = inf_ip_address_new_raw6(INFINOTED_RUN_IPV6_ANY_ADDR);
-  run->xmpp6 = infinoted_run_create_server(run, startup, address, NULL);
-
   local_error = NULL;
-  run->xmpp4 = infinoted_run_create_server(run, startup, NULL, &local_error);
+
+  if(startup->options->listen_address != NULL)
+  {
+    /* Use manually specified listen address */
+    address = inf_ip_address_copy(startup->options->listen_address);
+
+    switch(inf_ip_address_get_family(address))
+    {
+    case INF_IP_ADDRESS_IPV4:
+      run->xmpp4 = infinoted_run_create_server(run, startup, address, &local_error);
+      run->xmpp6 = NULL;
+      break;
+    case INF_IP_ADDRESS_IPV6:
+      run->xmpp4 = NULL;
+      run->xmpp6 = infinoted_run_create_server(run, startup, address, &local_error);
+      break;
+    }
+  }
+  else
+  {
+    address = inf_ip_address_new_raw6(INFINOTED_RUN_IPV6_ANY_ADDR);
+
+    run->xmpp6 = infinoted_run_create_server(run, startup, address, NULL);
+    run->xmpp4 = infinoted_run_create_server(run, startup, NULL, &local_error);
+  }
 
   if(run->xmpp4 == NULL)
   {
     /* Ignore if we have an IPv6 server running */
     if(run->xmpp6 != NULL)
     {
-      g_error_free(local_error);
+      if(local_error != NULL)
+        g_error_free(local_error);
     }
     else
     {
@@ -253,7 +294,7 @@ infinoted_run_new(InfinotedStartup* startup,
       g_object_unref(run->directory);
       g_object_unref(run->io);
       g_slice_free(InfinotedRun, run);
-      return NULL;
+      run = NULL;
     }
   }
 
