@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -166,11 +166,12 @@ static void
 infinoted_plugin_document_stream_make_system_error(int code,
                                                    GError** error)
 {
-  g_set_error(
+  g_set_error_literal(
     error,
-    g_quark_from_static_string("INFINOTED_PLUGIN_DOCUMENT_STREAM_SYSTEM_ERROR"),
+    g_quark_from_static_string(
+      "INFINOTED_PLUGIN_DOCUMENT_STREAM_SYSTEM_ERROR"
+    ),
     code,
-    "%s",
     strerror(code)
   );
 }
@@ -790,6 +791,7 @@ infinoted_plugin_document_stream_process_get_document(
       ),
       doc_name,
       doc_len,
+      FALSE,
       infinoted_plugin_document_stream_navigate_func,
       stream
     );
