@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -36,7 +36,6 @@
 #include <libinfinity/common/inf-cert-util.h>
 #include <libinfinity/common/inf-error.h>
 #include <libinfinity/inf-i18n.h>
-#include <libinfinity/inf-marshal.h>
 #include <libinfinity/inf-signals.h>
 
 #include <gnutls/x509.h>
@@ -49,7 +48,7 @@ struct _InfGtkAccountCreationDialogPrivate {
   InfBrowser* browser;
 
   GtkWidget* status_text;
-  GtkWidget* generate_button;
+  GtkWidget* create_account_button;
   GtkWidget* name_entry;
 
   InfAsyncOperation* key_generator;
@@ -80,9 +79,11 @@ enum {
 
 #define INF_GTK_ACCOUNT_CREATION_DIALOG_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_GTK_TYPE_ACCOUNT_CREATION_DIALOG, InfGtkAccountCreationDialogPrivate))
 
-static GtkDialogClass* parent_class;
 static guint account_creation_dialog_signals[LAST_SIGNAL];
 
+G_DEFINE_TYPE_WITH_CODE(InfGtkAccountCreationDialog, inf_gtk_account_creation_dialog, GTK_TYPE_DIALOG,
+  G_ADD_PRIVATE(InfGtkAccountCreationDialog))
+
 static void
 inf_gtk_account_creation_dialog_keygen_result_free(gpointer data)
 {
@@ -166,12 +167,12 @@ inf_gtk_account_creation_dialog_set_activatable(
   if(activatable)
   {
     /*gtk_widget_set_sensitive(priv->name_entry, TRUE);*/
-    gtk_widget_set_sensitive(priv->generate_button, TRUE);
+    gtk_widget_set_sensitive(priv->create_account_button, TRUE);
   }
   else
   {
     /*gtk_widget_set_sensitive(priv->name_entry, FALSE);*/
-    gtk_widget_set_sensitive(priv->generate_button, FALSE);
+    gtk_widget_set_sensitive(priv->create_account_button, FALSE);
   }
 }
 
@@ -497,124 +498,33 @@ inf_gtk_account_creation_dialog_notify_status_cb(GObject* object,
  */
 
 static void
-inf_gtk_account_creation_dialog_init(GTypeInstance* instance,
-                                     gpointer g_class)
+inf_gtk_account_creation_dialog_init(InfGtkAccountCreationDialog* dialog)
 {
-  InfGtkAccountCreationDialog* dialog;
   InfGtkAccountCreationDialogPrivate* priv;
-
-  GtkWidget* name_label;
-  GtkWidget* box;
-  GtkWidget* vbox;
-  GtkWidget* image;
-  GtkWidget* imagebox;
-  GtkWidget* dialog_vbox;
-
-  dialog = INF_GTK_ACCOUNT_CREATION_DIALOG(instance);
   priv = INF_GTK_ACCOUNT_CREATION_DIALOG_PRIVATE(dialog);
 
+  priv->io = NULL;
   priv->browser = NULL;
   priv->key_generator = NULL;
   priv->create_account_request = NULL;
   priv->key = NULL;
 
-  name_label = gtk_label_new(_("Account Name:"));
-  gtk_widget_show(name_label);
+  gtk_widget_init_template(GTK_WIDGET(dialog));
 
-  priv->name_entry = gtk_entry_new();
-  gtk_entry_set_activates_default(GTK_ENTRY(priv->name_entry), TRUE);
-  gtk_widget_show(priv->name_entry);
-
-  g_signal_connect(
-    G_OBJECT(priv->name_entry),
-    "changed",
-    G_CALLBACK(inf_gtk_account_creation_dialog_entry_changed_cb),
-    dialog
-  );
-
-  priv->generate_button = gtk_button_new_with_mnemonic(_("Create _Account"));
-  gtk_widget_set_sensitive(priv->generate_button, FALSE);
-  gtk_widget_show(priv->generate_button);
-
-  g_signal_connect(
-    G_OBJECT(priv->generate_button),
-    "clicked",
-    G_CALLBACK(inf_gtk_account_creation_dialog_generate_clicked_cb),
-    dialog
-  );
-
-  box = gtk_hbox_new(FALSE, 12);
-  gtk_box_pack_start(GTK_BOX(box), name_label, FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(box), priv->name_entry, FALSE, TRUE, 0);
-  gtk_box_pack_start(GTK_BOX(box), priv->generate_button, FALSE, FALSE, 0);
-  gtk_widget_show(box);
-
-  priv->status_text = gtk_label_new("");
-  gtk_misc_set_alignment(GTK_MISC(priv->status_text), 0.0, 0.5);
-  gtk_label_set_line_wrap(GTK_LABEL(priv->status_text), TRUE);
-  gtk_label_set_max_width_chars(GTK_LABEL(priv->status_text), 50);
-  gtk_widget_show(priv->status_text);
-  vbox = gtk_vbox_new(FALSE, 12);
-  gtk_box_pack_start(GTK_BOX(vbox), box, FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(vbox), priv->status_text, FALSE, FALSE, 0);
-  gtk_widget_show(vbox);
-
-  /* TODO: Make sure that icon name is available, otherwise hide the
-   * image in the dialog. */
-  image = gtk_image_new_from_icon_name(
-    "application-certificate",
-    GTK_ICON_SIZE_DIALOG
-  );
-
-  gtk_misc_set_alignment(GTK_MISC(image), 0.0, 0.0);
-  gtk_widget_show(image);
-
-  imagebox = gtk_hbox_new(FALSE, 12);
-  gtk_box_pack_start(GTK_BOX(imagebox), image, FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(imagebox), vbox, FALSE, TRUE, 0);
-  gtk_widget_show(imagebox);
-
-#if GTK_CHECK_VERSION(2,14,0)
-  dialog_vbox = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
-#else
-  dialog_vbox = GTK_DIALOG(dialog)->vbox;
-#endif
-
-  gtk_box_set_spacing(GTK_BOX(dialog_vbox), 12);
-  gtk_box_pack_start(GTK_BOX(dialog_vbox), imagebox, FALSE, FALSE, 0);
-
-  gtk_container_set_border_width(GTK_CONTAINER(dialog), 12);
-  gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
-
-#if GTK_CHECK_VERSION(2,18,0)
-  gtk_widget_set_can_default(priv->generate_button, TRUE);
-#else
-  GTK_WIDGET_SET_FLAGS(priv->generate_button, GTK_CAN_DEFAULT);
-#endif
-  gtk_widget_grab_default(priv->generate_button);
-
-  gtk_window_set_title(GTK_WINDOW(dialog), _("Create New Account"));
+  gtk_widget_grab_default(priv->create_account_button);
 }
 
-static GObject*
-inf_gtk_account_creation_dialog_constructor(GType type,
-                                            guint n_properties,
-                                            GObjectConstructParam* properties)
+static void
+inf_gtk_account_creation_dialog_constructed(GObject* object)
 {
-  GObject* object;
-
-  object = G_OBJECT_CLASS(parent_class)->constructor(
-    type,
-    n_properties,
-    properties
+  G_OBJECT_CLASS(inf_gtk_account_creation_dialog_parent_class)->constructed(
+    object
   );
 
   inf_gtk_account_creation_dialog_update(
     INF_GTK_ACCOUNT_CREATION_DIALOG(object),
     NULL
   );
-
-  return object;
 }
 
 static void
@@ -633,7 +543,7 @@ inf_gtk_account_creation_dialog_dispose(GObject* object)
 
   g_assert(priv->key == NULL);
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_gtk_account_creation_dialog_parent_class)->dispose(object);
 }
 
 
@@ -646,7 +556,7 @@ inf_gtk_account_creation_dialog_finalize(GObject* object)
   dialog = INF_GTK_ACCOUNT_CREATION_DIALOG(object);
   priv = INF_GTK_ACCOUNT_CREATION_DIALOG_PRIVATE(dialog);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(inf_gtk_account_creation_dialog_parent_class)->finalize(object);
 }
 
 static void
@@ -711,24 +621,13 @@ inf_gtk_account_creation_dialog_get_property(GObject* object,
  */
 
 static void
-inf_gtk_account_creation_dialog_class_init(gpointer g_class,
-                                           gpointer class_data)
+inf_gtk_account_creation_dialog_class_init(
+  InfGtkAccountCreationDialogClass* account_creation_dialog_class)
 {
   GObjectClass* object_class;
-  InfGtkAccountCreationDialogClass* account_creation_dialog_class;
+  object_class = G_OBJECT_CLASS(account_creation_dialog_class);
 
-  object_class = G_OBJECT_CLASS(g_class);
-  account_creation_dialog_class =
-    INF_GTK_ACCOUNT_CREATION_DIALOG_CLASS(g_class);
-
-  parent_class = GTK_DIALOG_CLASS(g_type_class_peek_parent(g_class));
-
-  g_type_class_add_private(
-    g_class,
-    sizeof(InfGtkAccountCreationDialogPrivate)
-  );
-
-  object_class->constructor = inf_gtk_account_creation_dialog_constructor;
+  object_class->constructed = inf_gtk_account_creation_dialog_constructed;
   object_class->dispose = inf_gtk_account_creation_dialog_dispose;
   object_class->finalize = inf_gtk_account_creation_dialog_finalize;
   object_class->set_property = inf_gtk_account_creation_dialog_set_property;
@@ -736,6 +635,39 @@ inf_gtk_account_creation_dialog_class_init(gpointer g_class,
 
   account_creation_dialog_class->account_created = NULL;
 
+  gtk_widget_class_set_template_from_resource(
+    GTK_WIDGET_CLASS(object_class),
+    "/de/0x539/libinfgtk/ui/infgtkaccountcreationdialog.ui"
+  );
+
+  gtk_widget_class_bind_template_child_private(
+    GTK_WIDGET_CLASS(object_class),
+    InfGtkAccountCreationDialog,
+    status_text
+  );
+
+  gtk_widget_class_bind_template_child_private(
+    GTK_WIDGET_CLASS(object_class),
+    InfGtkAccountCreationDialog,
+    create_account_button
+  );
+
+  gtk_widget_class_bind_template_child_private(
+    GTK_WIDGET_CLASS(object_class),
+    InfGtkAccountCreationDialog,
+    name_entry
+  );
+
+  gtk_widget_class_bind_template_callback(
+    GTK_WIDGET_CLASS(object_class),
+    inf_gtk_account_creation_dialog_entry_changed_cb
+  );
+
+  gtk_widget_class_bind_template_callback(
+    GTK_WIDGET_CLASS(object_class),
+    inf_gtk_account_creation_dialog_generate_clicked_cb
+  );
+
   g_object_class_install_property(
     object_class,
     PROP_IO,
@@ -780,7 +712,7 @@ inf_gtk_account_creation_dialog_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfGtkAccountCreationDialogClass, account_created),
     NULL, NULL,
-    inf_marshal_VOID__POINTER_BOXED_BOXED,
+    NULL,
     G_TYPE_NONE,
     3,
     G_TYPE_POINTER, /* gnutls_x509_privkey_t */
@@ -789,43 +721,12 @@ inf_gtk_account_creation_dialog_class_init(gpointer g_class,
   );
 }
 
-GType
-inf_gtk_account_creation_dialog_get_type(void)
-{
-  static GType account_creation_dialog_type = 0;
-
-  if(!account_creation_dialog_type)
-  {
-    static const GTypeInfo account_creation_dialog_type_info = {
-      sizeof(InfGtkAccountCreationDialogClass),    /* class_size */
-      NULL,                                        /* base_init */
-      NULL,                                        /* base_finalize */
-      inf_gtk_account_creation_dialog_class_init,  /* class_init */
-      NULL,                                        /* class_finalize */
-      NULL,                                        /* class_data */
-      sizeof(InfGtkAccountCreationDialog),         /* instance_size */
-      0,                                           /* n_preallocs */
-      inf_gtk_account_creation_dialog_init,        /* instance_init */
-      NULL                                         /* value_table */
-    };
-
-    account_creation_dialog_type = g_type_register_static(
-      GTK_TYPE_DIALOG,
-      "InfGtkAccountCreationDialog",
-      &account_creation_dialog_type_info,
-      0
-    );
-  }
-
-  return account_creation_dialog_type;
-}
-
 /*
  * Public API.
  */
 
 /**
- * inf_gtk_account_creation_dialog_new:
+ * inf_gtk_account_creation_dialog_new: (constructor)
  * @parent: Parent #GtkWindow of the dialog.
  * @dialog_flags: Flags for the dialog, see #GtkDialogFlags.
  * @io: A #InfIo object to schedule asynchronous operations.
@@ -834,8 +735,8 @@ inf_gtk_account_creation_dialog_get_type(void)
  * Creates a new #InfGtkAccountCreationDialog, which can be used to generate
  * a new account on the infinote directory represented by the given browser.
  *
- * Returns: A new #InfGtkAccountCreationDialog. Free with gtk_widget_destroy()
- * when no longer needed.
+ * Returns: (transfer full): A new #InfGtkAccountCreationDialog. Free with
+ * gtk_widget_destroy() when no longer needed.
  */
 InfGtkAccountCreationDialog*
 inf_gtk_account_creation_dialog_new(GtkWindow* parent,
@@ -862,11 +763,6 @@ inf_gtk_account_creation_dialog_new(GtkWindow* parent,
   if(dialog_flags & GTK_DIALOG_DESTROY_WITH_PARENT)
     gtk_window_set_destroy_with_parent(GTK_WINDOW(object), TRUE);
 
-#if !GTK_CHECK_VERSION(2,90,7)
-  if(dialog_flags & GTK_DIALOG_NO_SEPARATOR)
-    gtk_dialog_set_has_separator(GTK_DIALOG(object), FALSE);
-#endif
-
   gtk_window_set_transient_for(GTK_WINDOW(object), parent);
   return INF_GTK_ACCOUNT_CREATION_DIALOG(object);
 }
