@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,7 +19,6 @@
 
 #include <libinfgtk/inf-gtk-browser-view.h>
 #include <libinfinity/common/inf-discovery.h>
-#include <libinfinity/inf-marshal.h>
 #include <libinfinity/inf-i18n.h>
 #include <libinfinity/inf-signals.h>
 
@@ -66,7 +65,6 @@ struct _InfGtkBrowserViewPrivate {
 
   /* Note that progress and status_text are never visible at the same time */
   GtkCellRenderer* renderer_icon;
-  GtkCellRenderer* renderer_status_icon; /* toplevel only */
   GtkCellRenderer* renderer_name;
   GtkCellRenderer* renderer_progress;
   GtkCellRenderer* renderer_status;
@@ -106,9 +104,11 @@ enum {
  * browsers from our array. The same holds for explore requests and
  * session synchronizations. */
 
-static GObjectClass* parent_class;
 static guint view_signals[LAST_SIGNAL];
 
+G_DEFINE_TYPE_WITH_CODE(InfGtkBrowserView, inf_gtk_browser_view, GTK_TYPE_TREE_VIEW,
+  G_ADD_PRIVATE(InfGtkBrowserView))
+
 /*
  * Utility functions
  */
@@ -186,11 +186,7 @@ inf_gtk_browser_view_redraw_row(InfGtkBrowserView* view,
    * in the GTK+ treeview and friends. */
   priv = INF_GTK_BROWSER_VIEW_PRIVATE(view);
 
-#if GTK_CHECK_VERSION(2,20,0)
   if(gtk_widget_get_realized(GTK_WIDGET(view)))
-#else
-  if(GTK_WIDGET_REALIZED(GTK_WIDGET(view)))
-#endif
   {
     gtk_tree_view_get_cell_area(
       GTK_TREE_VIEW(view),
@@ -1563,6 +1559,7 @@ inf_gtk_browser_view_row_expanded(GtkTreeView* tree_view,
   const InfAclAccount* account;
   InfAclAccountId acc_id;
   InfAclMask mask;
+  GtkTreeViewClass* parent_class;
 
   model = gtk_tree_view_get_model(tree_view);
 
@@ -1606,8 +1603,9 @@ inf_gtk_browser_view_row_expanded(GtkTreeView* tree_view,
   inf_browser_iter_free(browser_iter);
   g_object_unref(browser);
 
-  if(GTK_TREE_VIEW_CLASS(parent_class)->row_expanded != NULL)
-    GTK_TREE_VIEW_CLASS(parent_class)->row_expanded(tree_view, iter, path);
+  parent_class = GTK_TREE_VIEW_CLASS(inf_gtk_browser_view_parent_class);
+  if(parent_class->row_expanded != NULL)
+    parent_class->row_expanded(tree_view, iter, path);
 }
 
 static void
@@ -1630,6 +1628,7 @@ inf_gtk_browser_view_row_activated(GtkTreeView* tree_view,
   InfXmlConnectionStatus xml_status;
   GError* error;
   InfGtkBrowserViewBrowser* view_browser;
+  GtkTreeViewClass* parent_class;
 
   view = INF_GTK_BROWSER_VIEW(tree_view);
   priv = INF_GTK_BROWSER_VIEW_PRIVATE(view);
@@ -1736,8 +1735,9 @@ inf_gtk_browser_view_row_activated(GtkTreeView* tree_view,
     g_object_unref(G_OBJECT(browser));
   }
 
-  if(GTK_TREE_VIEW_CLASS(parent_class)->row_activated != NULL)
-    GTK_TREE_VIEW_CLASS(parent_class)->row_activated(tree_view, path, column);
+  parent_class = GTK_TREE_VIEW_CLASS(inf_gtk_browser_view_parent_class);
+  if(parent_class->row_activated != NULL)
+    parent_class->row_activated(tree_view, path, column);
 }
 
 static void
@@ -1802,13 +1802,9 @@ inf_gtk_browser_view_popup_menu_position_func(GtkMenu* menu,
   gtk_menu_set_monitor(menu, monitor_num);
 
   gdk_screen_get_monitor_geometry(screen, monitor_num, &monitor);
-  gtk_widget_size_request(GTK_WIDGET(menu), &menu_req);
+  gtk_widget_get_preferred_size(GTK_WIDGET(menu), NULL, &menu_req);
 
-#if GTK_CHECK_VERSION(2, 91, 0)
-      height = gdk_window_get_height(bin_window);
-#else
-      gdk_drawable_get_size(GDK_DRAWABLE(bin_window), NULL, &height);
-#endif
+  height = gdk_window_get_height(bin_window);
 
   selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
   gtk_tree_selection_get_selected(selection, &model, &selected_iter);
@@ -1905,6 +1901,7 @@ inf_gtk_browser_view_button_press_event(GtkWidget* treeview,
 {
   GtkTreePath* path;
   gboolean has_path;
+  GtkWidgetClass* parent_class;
 
   if(event->button == 3 &&
      event->window == gtk_tree_view_get_bin_window(GTK_TREE_VIEW(treeview)))
@@ -1936,7 +1933,8 @@ inf_gtk_browser_view_button_press_event(GtkWidget* treeview,
     }
   }
 
-  return GTK_WIDGET_CLASS(parent_class)->button_press_event(treeview, event);
+  parent_class = GTK_WIDGET_CLASS(inf_gtk_browser_view_parent_class);
+  return parent_class->button_press_event(treeview, event);
 }
 
 static gboolean
@@ -1945,12 +1943,9 @@ inf_gtk_browser_view_key_press_event(GtkWidget* treeview,
 {
   GtkTreeSelection* selection;
   GtkTreeIter iter;
+  GtkWidgetClass* parent_class;
 
-#if GTK_CHECK_VERSION(2,90,7)
   if(event->keyval == GDK_KEY_Menu)
-#else
-  if(event->keyval == GDK_Menu)
-#endif
   {
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
     if(gtk_tree_selection_get_selected(selection, NULL, &iter))
@@ -1963,7 +1958,8 @@ inf_gtk_browser_view_key_press_event(GtkWidget* treeview,
     }
   }
 
-  return GTK_WIDGET_CLASS(parent_class)->key_press_event(treeview, event);
+  parent_class = GTK_WIDGET_CLASS(inf_gtk_browser_view_parent_class);
+  return parent_class->key_press_event(treeview, event);
 }
 
 /*
@@ -1978,13 +1974,14 @@ inf_gtk_browser_view_icon_data_func(GtkTreeViewColumn* column,
                                     gpointer user_data)
 {
   GtkTreeIter iter_parent;
+  InfGtkBrowserModelStatus status;
   InfDiscovery* discovery;
   InfBrowser* browser;
   InfBrowserIter* browser_iter;
   InfAclMask mask;
   const InfAclAccount* account;
   InfAclAccountId acc_id;
-  const gchar* stock_id;
+  const gchar* icon_name;
 
   if(gtk_tree_model_iter_parent(model, &iter_parent, iter))
   {
@@ -2009,19 +2006,21 @@ inf_gtk_browser_view_icon_data_func(GtkTreeViewColumn* column,
     {
       inf_acl_mask_set1(&mask, INF_ACL_CAN_EXPLORE_NODE);
       if(inf_browser_check_acl(browser, browser_iter, acc_id, &mask, NULL))
-        stock_id = GTK_STOCK_DIRECTORY;
+        icon_name = "folder";
       else
-        stock_id = GTK_STOCK_DIALOG_AUTHENTICATION;
-      g_object_set(G_OBJECT(renderer), "stock-id", stock_id, NULL);
+        /* Would be nice to have a more appropriate icon for this */
+        icon_name = "dialog-password";
+      g_object_set(G_OBJECT(renderer), "icon-name", icon_name, NULL);
     }
     else
     {
       inf_acl_mask_set1(&mask, INF_ACL_CAN_SUBSCRIBE_SESSION);
       if(inf_browser_check_acl(browser, browser_iter, acc_id, &mask, NULL))
-        stock_id = GTK_STOCK_FILE;
+        icon_name = "text-x-generic"; /* appropriate? */
       else
-        stock_id = GTK_STOCK_DIALOG_AUTHENTICATION;
-      g_object_set(G_OBJECT(renderer), "stock-id", stock_id, NULL);
+        /* Would be nice to have a more appropriate icon for this */
+        icon_name = "dialog-password";
+      g_object_set(G_OBJECT(renderer), "icon-name", icon_name, NULL);
     }
 
     inf_browser_iter_free(browser_iter);
@@ -2029,41 +2028,17 @@ inf_gtk_browser_view_icon_data_func(GtkTreeViewColumn* column,
   }
   else
   {
-    gtk_tree_model_get(
+    /* toplevel */
+
+    /* TODO: Set icon depending on discovery type (LAN, jabber, direct) */
+    /*gtk_tree_model_get(
       model,
       iter,
       INF_GTK_BROWSER_MODEL_COL_DISCOVERY, &discovery,
       INF_GTK_BROWSER_MODEL_COL_BROWSER, &browser,
       -1
-    );
+    );*/
 
-    /* TODO: Set icon depending on discovery type (LAN, jabber, direct) */
-    g_object_set(G_OBJECT(renderer), "stock-id", GTK_STOCK_NETWORK, NULL);
-
-    if(discovery != NULL) g_object_unref(G_OBJECT(discovery));
-    if(browser != NULL) g_object_unref(G_OBJECT(browser));
-  }
-}
-
-static void
-inf_gtk_browser_view_status_icon_data_func(GtkTreeViewColumn* column,
-                                           GtkCellRenderer* renderer,
-                                           GtkTreeModel* model,
-                                           GtkTreeIter* iter,
-                                           gpointer user_data)
-{
-  GtkTreeIter iter_parent;
-  InfGtkBrowserModelStatus status;
-  const gchar* stock_id;
-
-  if(gtk_tree_model_iter_parent(model, &iter_parent, iter))
-  {
-    /* inner node, ignore */
-    g_object_set(G_OBJECT(renderer), "visible", FALSE, NULL);
-  }
-  else
-  {
-    /* toplevel */
     gtk_tree_model_get(
       model,
       iter,
@@ -2073,29 +2048,35 @@ inf_gtk_browser_view_status_icon_data_func(GtkTreeViewColumn* column,
 
     switch(status)
     {
+    case INF_GTK_BROWSER_MODEL_INVALID:
+      /* This can happen when the cell data func is called between the
+       * resetting of an item and its removal from the tree. But in any case
+       * it will be removed soon, so no need to do anything here. */
+      icon_name = NULL;
+      break;
     case INF_GTK_BROWSER_MODEL_DISCONNECTED:
     case INF_GTK_BROWSER_MODEL_DISCOVERED:
     case INF_GTK_BROWSER_MODEL_RESOLVING:
     case INF_GTK_BROWSER_MODEL_CONNECTING:
-      stock_id = GTK_STOCK_DISCONNECT;
+      icon_name = "network-offline";
       break;
     case INF_GTK_BROWSER_MODEL_CONNECTED:
-      stock_id = GTK_STOCK_CONNECT;
+      /* TODO: Could choose to show network-transmit and/or
+       * network-transmit-received when there is activitiy */
+      icon_name = "network-idle";
       break;
     case INF_GTK_BROWSER_MODEL_ERROR:
-      stock_id = GTK_STOCK_DIALOG_ERROR;
+      icon_name = "network-error";
       break;
     default:
       g_assert_not_reached();
       break;
     }
     
-    g_object_set(
-      G_OBJECT(renderer),
-      "visible", TRUE,
-      "stock-id", stock_id,
-      NULL
-    );
+    g_object_set(G_OBJECT(renderer), "icon-name", icon_name, NULL);
+
+    /*if(discovery != NULL) g_object_unref(G_OBJECT(discovery));
+    if(browser != NULL) g_object_unref(G_OBJECT(browser));*/
   }
 }
 
@@ -2326,6 +2307,11 @@ inf_gtk_browser_view_status_data_func(GtkTreeViewColumn* column,
 
     switch(status)
     {
+    case INF_GTK_BROWSER_MODEL_INVALID:
+      /* This can happen when the cell data func is called between the
+       * resetting of an item and its removal from the tree. But in any case
+       * it will be removed soon, so no need to do anything here. */
+      break;
     case INF_GTK_BROWSER_MODEL_DISCOVERED:
     case INF_GTK_BROWSER_MODEL_DISCONNECTED:
       g_object_set(
@@ -2382,21 +2368,17 @@ inf_gtk_browser_view_status_data_func(GtkTreeViewColumn* column,
  */
 
 static void
-inf_gtk_browser_view_init(GTypeInstance* instance,
-                          gpointer g_class)
+inf_gtk_browser_view_init(InfGtkBrowserView* view)
 {
-  InfGtkBrowserView* view;
   InfGtkBrowserViewPrivate* priv;
   GtkTreeSelection* selection;
 
-  view = INF_GTK_BROWSER_VIEW(instance);
   priv = INF_GTK_BROWSER_VIEW_PRIVATE(view);
 
   priv->column = gtk_tree_view_column_new();
   selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
   
   priv->renderer_icon = gtk_cell_renderer_pixbuf_new();
-  priv->renderer_status_icon = gtk_cell_renderer_pixbuf_new();
   priv->renderer_name = gtk_cell_renderer_text_new();
   priv->renderer_progress = gtk_cell_renderer_progress_new();
   priv->renderer_status = gtk_cell_renderer_text_new();
@@ -2405,16 +2387,9 @@ inf_gtk_browser_view_init(GTypeInstance* instance,
   priv->info_resolvs = NULL;
 
   g_object_set(G_OBJECT(priv->renderer_status), "xpad", 10, NULL);
-  g_object_set(G_OBJECT(priv->renderer_status_icon), "xpad", 5, NULL);
 
   gtk_tree_view_column_pack_start(priv->column, priv->renderer_icon, FALSE);
 
-  gtk_tree_view_column_pack_start(
-    priv->column,
-    priv->renderer_status_icon,
-    FALSE
-  );
-
   gtk_tree_view_column_pack_start(priv->column, priv->renderer_name, FALSE);
   gtk_tree_view_column_pack_start(
     priv->column,
@@ -2432,14 +2407,6 @@ inf_gtk_browser_view_init(GTypeInstance* instance,
     NULL
   );
 
-  gtk_tree_view_column_set_cell_data_func(
-    priv->column,
-    priv->renderer_status_icon,
-    inf_gtk_browser_view_status_icon_data_func,
-    NULL,
-    NULL
-  );
-
   gtk_tree_view_column_set_cell_data_func(
     priv->column,
     priv->renderer_name,
@@ -2486,7 +2453,7 @@ inf_gtk_browser_view_dispose(GObject* object)
 
   inf_gtk_browser_view_set_model(view, NULL);
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_gtk_browser_view_parent_class)->dispose(object);
 }
 
 static void
@@ -2500,7 +2467,7 @@ inf_gtk_browser_view_finalize(GObject* object)
 
   g_assert(priv->browsers == NULL);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(inf_gtk_browser_view_parent_class)->finalize(object);
 }
 
 static void
@@ -2516,7 +2483,6 @@ inf_gtk_browser_view_set_property(GObject* object,
   switch(prop_id)
   {
   case PROP_MODEL:
-
     if (g_value_get_object(value) != NULL)
       g_assert(INF_GTK_IS_BROWSER_MODEL(g_value_get_object(value)));
 
@@ -2560,11 +2526,7 @@ inf_gtk_browser_view_get_property(GObject* object,
  * GtkObject / GtkWidget overrides */
 
 static void
-#if GTK_CHECK_VERSION(2, 91, 0)
 inf_gtk_browser_view_destroy(GtkWidget* object)
-#else
-inf_gtk_browser_view_destroy(GtkObject* object)
-#endif
 {
   InfGtkBrowserView* view;
   InfGtkBrowserViewPrivate* priv;
@@ -2574,13 +2536,8 @@ inf_gtk_browser_view_destroy(GtkObject* object)
 
   inf_gtk_browser_view_set_model(view, NULL);
 
-#if GTK_CHECK_VERSION(2, 91, 0)
-  if(GTK_WIDGET_CLASS(parent_class)->destroy)
-    GTK_WIDGET_CLASS(parent_class)->destroy(object);
-#else
-  if(GTK_OBJECT_CLASS(parent_class)->destroy)
-    GTK_OBJECT_CLASS(parent_class)->destroy(object);
-#endif
+  if(GTK_WIDGET_CLASS(inf_gtk_browser_view_parent_class)->destroy)
+    GTK_WIDGET_CLASS(inf_gtk_browser_view_parent_class)->destroy(object);
 }
 
 /*
@@ -2588,43 +2545,28 @@ inf_gtk_browser_view_destroy(GtkObject* object)
  */
 
 static void
-inf_gtk_browser_view_class_init(gpointer g_class,
-                                gpointer class_data)
+inf_gtk_browser_view_class_init(InfGtkBrowserViewClass* view_class)
 {
   GObjectClass* object_class;
   GtkWidgetClass* widget_class;
-  InfGtkBrowserViewClass* view_class;
   GtkTreeViewClass* tree_class;
-#if ! GTK_CHECK_VERSION(2, 91, 0)
-  GtkObjectClass *gtk_object_class;
-#endif
 
-  object_class = G_OBJECT_CLASS(g_class);
-  widget_class = GTK_WIDGET_CLASS(g_class);
-  view_class = INF_GTK_BROWSER_VIEW_CLASS(g_class);
-  tree_class = GTK_TREE_VIEW_CLASS(g_class);
-
-#if ! GTK_CHECK_VERSION(2, 91, 0)
-  gtk_object_class = GTK_OBJECT_CLASS(g_class);
-  gtk_object_class->destroy = inf_gtk_browser_view_destroy;
-#else
-  widget_class->destroy = inf_gtk_browser_view_destroy;
-#endif
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfGtkBrowserViewPrivate));
+  object_class = G_OBJECT_CLASS(view_class);
+  widget_class = GTK_WIDGET_CLASS(view_class);
+  tree_class = GTK_TREE_VIEW_CLASS(view_class);
 
   object_class->dispose = inf_gtk_browser_view_dispose;
   object_class->finalize = inf_gtk_browser_view_finalize;
   object_class->set_property = inf_gtk_browser_view_set_property;
   object_class->get_property = inf_gtk_browser_view_get_property;
 
-  tree_class->row_expanded = inf_gtk_browser_view_row_expanded;
-  tree_class->row_activated = inf_gtk_browser_view_row_activated;
-
+  widget_class->destroy = inf_gtk_browser_view_destroy;
   widget_class->button_press_event = inf_gtk_browser_view_button_press_event;
   widget_class->key_press_event = inf_gtk_browser_view_key_press_event;
 
+  tree_class->row_expanded = inf_gtk_browser_view_row_expanded;
+  tree_class->row_activated = inf_gtk_browser_view_row_activated;
+
   view_class->activate = NULL;
   view_class->selection_changed = NULL;
   view_class->populate_popup = NULL;
@@ -2637,7 +2579,7 @@ inf_gtk_browser_view_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfGtkBrowserViewClass, activate),
     NULL, NULL,
-    inf_marshal_VOID__BOXED,
+    g_cclosure_marshal_VOID__BOXED,
     G_TYPE_NONE,
     1,
     GTK_TYPE_TREE_ITER | G_SIGNAL_TYPE_STATIC_SCOPE
@@ -2649,7 +2591,7 @@ inf_gtk_browser_view_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfGtkBrowserViewClass, selection_changed),
     NULL, NULL,
-    inf_marshal_VOID__BOXED,
+    g_cclosure_marshal_VOID__BOXED,
     G_TYPE_NONE,
     1,
     GTK_TYPE_TREE_ITER | G_SIGNAL_TYPE_STATIC_SCOPE
@@ -2661,54 +2603,23 @@ inf_gtk_browser_view_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfGtkBrowserViewClass, populate_popup),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT,
+    g_cclosure_marshal_VOID__OBJECT,
     G_TYPE_NONE,
     1,
     GTK_TYPE_MENU
   );
 }
 
-GType
-inf_gtk_browser_view_get_type(void)
-{
-  static GType browser_view_type = 0;
-
-  if(!browser_view_type)
-  {
-    static const GTypeInfo browser_view_type_info = {
-      sizeof(InfGtkBrowserViewClass),    /* class_size */
-      NULL,                              /* base_init */
-      NULL,                              /* base_finalize */
-      inf_gtk_browser_view_class_init,   /* class_init */
-      NULL,                              /* class_finalize */
-      NULL,                              /* class_data */
-      sizeof(InfGtkBrowserView),         /* instance_size */
-      0,                                 /* n_preallocs */
-      inf_gtk_browser_view_init,         /* instance_init */
-      NULL                               /* value_table */
-    };
-
-    browser_view_type = g_type_register_static(
-      GTK_TYPE_TREE_VIEW,
-      "InfGtkBrowserView",
-      &browser_view_type_info,
-      0
-    );
-  }
-
-  return browser_view_type;
-}
-
 /*
  * Public API.
  */
 
 /**
- * inf_gtk_browser_view_new:
+ * inf_gtk_browser_view_new: (constructor)
  *
  * Creates a new #InfGtkBrowserView.
  *
- * Return Value: A new #InfGtkBrowserView.
+ * Returns: (transfer floating): A new #InfGtkBrowserView.
  **/
 GtkWidget*
 inf_gtk_browser_view_new(void)
@@ -2719,12 +2630,12 @@ inf_gtk_browser_view_new(void)
 }
 
 /**
- * inf_gtk_browser_view_new_with_model:
+ * inf_gtk_browser_view_new_with_model: (constructor)
  * @model: A #InfGtkBrowserModel.
  *
  * Creates a new #InfGtkBrowserView showing @model.
  *
- * Return Value: A new #InfGtkBrowserView.
+ * Returns: (transfer floating): A new #InfGtkBrowserView.
  **/
 GtkWidget*
 inf_gtk_browser_view_new_with_model(InfGtkBrowserModel* model)
@@ -2737,7 +2648,7 @@ inf_gtk_browser_view_new_with_model(InfGtkBrowserModel* model)
 /**
  * inf_gtk_browser_view_get_selected:
  * @view: A #InfGtkBrowserView.
- * @iter: An uninitialized #GtkTreeIter.
+ * @iter: (out): An uninitialized #GtkTreeIter.
  *
  * Sets @iter to point to the currently selected row in the browser view. If
  * no row is selected, @iter is left untouched and %FALSE is returned.
@@ -2759,7 +2670,7 @@ inf_gtk_browser_view_get_selected(InfGtkBrowserView* view,
 /**
  * inf_gtk_browser_view_set_selected:
  * @view: A #InfGtkBrowserView.
- * @iter: A #GtkTreeIter pointing to a row in @view.
+ * @iter: (transfer none): A #GtkTreeIter pointing to a row in @view.
  *
  * Sets the currently selected row to be @iter. If necessary, rows will be
  * expanded so that @iter is visible.
