@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -90,14 +90,14 @@ struct _InfGtkPermissionsDialogPrivate {
   GtkMenu* popup_menu;
   InfAclAccountId popup_account;
 
-  GtkCellRenderer* renderer;
+  GtkWidget* status_text;
   GtkWidget* tree_view;
   GtkWidget* sheet_view;
-  GtkWidget* status_image;
-  GtkWidget* status_text;
 
   GtkWidget* add_button;
   GtkWidget* remove_button;
+
+  GtkCellRenderer* renderer;
 };
 
 enum {
@@ -109,7 +109,8 @@ enum {
 
 #define INF_GTK_PERMISSIONS_DIALOG_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_GTK_TYPE_PERMISSIONS_DIALOG, InfGtkPermissionsDialogPrivate))
 
-static GtkDialogClass* parent_class;
+G_DEFINE_TYPE_WITH_CODE(InfGtkPermissionsDialog, inf_gtk_permissions_dialog, GTK_TYPE_DIALOG,
+  G_ADD_PRIVATE(InfGtkPermissionsDialog))
 
 /*
  * Private functionality
@@ -260,42 +261,44 @@ inf_gtk_permissions_dialog_sheet_changed_cb(InfGtkAclSheetView* sheet_view,
     INF_GTK_ACL_SHEET_VIEW(priv->sheet_view)
   );
 
-  g_assert(sheet != NULL);
-
-  /* If the sheet does not have an ID set, the lookup is still in progress.
-   * In that case, we run the ACL setting once we have looked up the ID. */
-  if(sheet->account != 0)
+  /* This can be NULL during destruction, when the sheet view is reset. */
+  if(sheet != NULL)
   {
-    sheet_set.own_sheets = NULL;
-    sheet_set.sheets = sheet;
-    sheet_set.n_sheets = 1;
-
-    request = inf_browser_set_acl(
-      priv->browser,
-      &priv->browser_iter,
-      &sheet_set,
-      inf_gtk_permissions_dialog_set_acl_finished_cb,
-      dialog
-    );
-
-    if(request != NULL)
+    /* If the sheet does not have an ID set, the lookup is still in progress.
+     * In that case, we run the ACL setting once we have looked up the ID. */
+    if(sheet->account != 0)
     {
-      priv->set_acl_requests =
-        g_slist_prepend(priv->set_acl_requests, request);
-      g_object_ref(request);
+      sheet_set.own_sheets = NULL;
+      sheet_set.sheets = sheet;
+      sheet_set.n_sheets = 1;
+
+      request = inf_browser_set_acl(
+        priv->browser,
+        &priv->browser_iter,
+        &sheet_set,
+        inf_gtk_permissions_dialog_set_acl_finished_cb,
+        dialog
+      );
+
+      if(request != NULL)
+      {
+        priv->set_acl_requests =
+          g_slist_prepend(priv->set_acl_requests, request);
+        g_object_ref(request);
+      }
     }
-  }
-  else
-  {
-    /* Must be a pending sheet */
-    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(priv->tree_view));
-    has_selection = gtk_tree_selection_get_selected(selection, NULL, &iter);
-    g_assert(has_selection);
+    else
+    {
+      /* Must be a pending sheet */
+      selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(priv->tree_view));
+      has_selection = gtk_tree_selection_get_selected(selection, NULL, &iter);
+      g_assert(has_selection);
 
-    pending = inf_gtk_permissions_dialog_find_pending_sheet(dialog, &iter);
-    g_assert(pending != NULL);
+      pending = inf_gtk_permissions_dialog_find_pending_sheet(dialog, &iter);
+      g_assert(pending != NULL);
 
-    pending->sheet = *sheet;
+      pending->sheet = *sheet;
+    }
   }
 }
 
@@ -1628,9 +1631,6 @@ inf_gtk_permissions_dialog_remove_clicked_cb(GtkButton* button,
   guint i;
   InfRequest* request;
   GtkTreeIter move_iter;
-#if !GTK_CHECK_VERSION(3,0,0)
-  GtkTreePath* move_path;
-#endif
   gboolean could_move;
 
   dialog = INF_GTK_PERMISSIONS_DIALOG(user_data);
@@ -1713,29 +1713,11 @@ inf_gtk_permissions_dialog_remove_clicked_cb(GtkButton* button,
     if(!could_move)
     {
       move_iter = selected_iter;
-#if GTK_CHECK_VERSION(3,0,0)
+
       could_move = gtk_tree_model_iter_previous(
         GTK_TREE_MODEL(priv->account_store),
         &move_iter
       );
-#else
-      move_path = gtk_tree_model_get_path(
-        GTK_TREE_MODEL(priv->account_store),
-        &move_iter
-      );
-
-      could_move = gtk_tree_path_prev(move_path);
-      if(could_move == TRUE)
-      {
-        gtk_tree_model_get_iter(
-          GTK_TREE_MODEL(priv->account_store),
-          &move_iter,
-          move_path
-        );
-      }
-
-      gtk_tree_path_free(move_path);
-#endif
     }
 
     g_assert(could_move);
@@ -1894,12 +1876,7 @@ inf_gtk_permissions_dialog_populate_popup(InfGtkPermissionsDialog* dialog,
     return FALSE;
  
   /* Then, show a menu item to remove an account. */
-  item = gtk_image_menu_item_new_with_mnemonic(_("_Delete Account"));
-
-  gtk_image_menu_item_set_image(
-    GTK_IMAGE_MENU_ITEM(item),
-    gtk_image_new_from_stock(GTK_STOCK_DELETE, GTK_ICON_SIZE_MENU)
-  );
+  item = gtk_menu_item_new_with_mnemonic(_("_Delete Account"));
 
   g_signal_connect(
     G_OBJECT(item),
@@ -1963,13 +1940,9 @@ inf_gtk_permissions_dialog_popup_menu_position_func(GtkMenu* menu,
   gtk_menu_set_monitor(menu, monitor_num);
 
   gdk_screen_get_monitor_geometry(screen, monitor_num, &monitor);
-  gtk_widget_size_request(GTK_WIDGET(menu), &menu_req);
+  gtk_widget_get_preferred_size(GTK_WIDGET(menu), NULL, &menu_req);
 
-#if GTK_CHECK_VERSION(2, 91, 0)
   height = gdk_window_get_height(bin_window);
-#else
-  gdk_drawable_get_size(GDK_DRAWABLE(bin_window), NULL, &height);
-#endif
 
   selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(priv->tree_view));
   gtk_tree_selection_get_selected(selection, &model, &selected_iter);
@@ -2140,11 +2113,7 @@ inf_gtk_permissions_dialog_key_press_event_cb(GtkWidget* treeview,
 
   dialog = INF_GTK_PERMISSIONS_DIALOG(user_data);
 
-#if GTK_CHECK_VERSION(2,90,7)
   if(event->keyval == GDK_KEY_Menu)
-#else
-  if(event->keyval == GDK_Menu)
-#endif
   {
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
     if(gtk_tree_selection_get_selected(selection, NULL, &iter))
@@ -2341,13 +2310,6 @@ inf_gtk_permissions_dialog_update(InfGtkPermissionsDialog* dialog,
   {
     gtk_list_store_clear(priv->account_store);
     gtk_label_set_text(GTK_LABEL(priv->status_text), _("No node selected"));
-
-    gtk_image_set_from_stock(
-      GTK_IMAGE(priv->status_image),
-      GTK_STOCK_DISCONNECT,
-      GTK_ICON_SIZE_BUTTON
-    );
-
     return;
   }
 
@@ -2531,12 +2493,6 @@ inf_gtk_permissions_dialog_update(InfGtkPermissionsDialog* dialog,
     gtk_widget_set_sensitive(priv->add_button, FALSE);
     gtk_widget_set_sensitive(priv->remove_button, FALSE);
 
-    gtk_image_set_from_stock(
-      GTK_IMAGE(priv->status_image),
-      GTK_STOCK_NO,
-      GTK_ICON_SIZE_BUTTON
-    );
-
     set_acl_str = _("Permission is <b>not granted</b> to modify the permission list. It is read-only.");
   }
   else
@@ -2561,12 +2517,6 @@ inf_gtk_permissions_dialog_update(InfGtkPermissionsDialog* dialog,
     else
       gtk_widget_set_sensitive(priv->remove_button, FALSE);
 
-    gtk_image_set_from_stock(
-      GTK_IMAGE(priv->status_image),
-      GTK_STOCK_YES,
-      GTK_ICON_SIZE_BUTTON
-    );
-
     set_acl_str = _("Permission is <b>granted</b> to modify the permission list.");
   }
 
@@ -2685,45 +2635,11 @@ inf_gtk_permissions_dialog_unregister(InfGtkPermissionsDialog* dialog)
  */
 
 static void
-inf_gtk_permissions_dialog_init(GTypeInstance* instance,
-                                gpointer g_class)
+inf_gtk_permissions_dialog_init(InfGtkPermissionsDialog* dialog)
 {
-  InfGtkPermissionsDialog* dialog;
   InfGtkPermissionsDialogPrivate* priv;
-  GtkTreeViewColumn* column;
-  GtkTreeSelection* selection;
-  GtkWidget* scroll;
-  GtkWidget* hbox;
-  GtkWidget* buttons_hbox;
-  GtkWidget* status_hbox;
-  GtkWidget* vbox;
-  GtkWidget* account_list_vbox;
-
-  GtkWidget* image;
-  GtkWidget* image_hbox;
-
-  GtkWidget* dialog_vbox;
-
-  dialog = INF_GTK_PERMISSIONS_DIALOG(instance);
   priv = INF_GTK_PERMISSIONS_DIALOG_PRIVATE(dialog);
 
-  /* The pointer is the account ID with INF_ACL_ACCOUNT_ID_TO_POINTER */
-  priv->account_store = gtk_list_store_new(2, G_TYPE_POINTER, G_TYPE_STRING);
-
-  gtk_tree_sortable_set_sort_column_id(
-    GTK_TREE_SORTABLE(priv->account_store),
-    INF_GTK_PERMISSIONS_DIALOG_COLUMN_NAME,
-    GTK_SORT_ASCENDING
-  );
-
-  gtk_tree_sortable_set_sort_func(
-    GTK_TREE_SORTABLE(priv->account_store),
-    INF_GTK_PERMISSIONS_DIALOG_COLUMN_NAME,
-    inf_gtk_permissions_dialog_account_sort_func,
-    dialog,
-    NULL
-  );
-
   priv->query_acl_account_list_request = NULL;
   priv->account_list_queried = FALSE;
   priv->accounts = NULL;
@@ -2739,255 +2655,38 @@ inf_gtk_permissions_dialog_init(GTypeInstance* instance,
   priv->popup_menu = NULL;
   priv->popup_account = 0;
 
-  column = gtk_tree_view_column_new();
-  gtk_tree_view_column_set_title(column, _("Accounts"));
-  gtk_tree_view_column_set_spacing(column, 6);
+  gtk_widget_init_template(GTK_WIDGET(dialog));
 
-  priv->renderer = gtk_cell_renderer_combo_new();
-  gtk_tree_view_column_pack_start(column, priv->renderer, FALSE);
+  gtk_tree_sortable_set_sort_column_id(
+    GTK_TREE_SORTABLE(priv->account_store),
+    INF_GTK_PERMISSIONS_DIALOG_COLUMN_NAME,
+    GTK_SORT_ASCENDING
+  );
+
+  gtk_tree_sortable_set_sort_func(
+    GTK_TREE_SORTABLE(priv->account_store),
+    INF_GTK_PERMISSIONS_DIALOG_COLUMN_NAME,
+    inf_gtk_permissions_dialog_account_sort_func,
+    dialog,
+    NULL
+  );
 
   gtk_tree_view_column_set_cell_data_func(
-    column,
+    gtk_tree_view_get_column(GTK_TREE_VIEW(priv->tree_view), 0),
     priv->renderer,
     inf_gtk_permissions_dialog_name_data_func,
     NULL,
     NULL
   );
-
-  g_signal_connect(
-    G_OBJECT(priv->renderer),
-    "editing-started",
-    G_CALLBACK(inf_gtk_permissions_dialog_renderer_editing_started_cb),
-    dialog
-  );
-
-  g_signal_connect(
-    G_OBJECT(priv->renderer),
-    "editing-canceled",
-    G_CALLBACK(inf_gtk_permissions_dialog_renderer_editing_canceled_cb),
-    dialog
-  );
-
-  g_signal_connect(
-    G_OBJECT(priv->renderer),
-    "edited",
-    G_CALLBACK(inf_gtk_permissions_dialog_renderer_edited_cb),
-    dialog
-  );
-
-  g_signal_connect(
-    G_OBJECT(priv->renderer),
-    "changed",
-    G_CALLBACK(inf_gtk_permissions_dialog_renderer_changed_cb),
-    dialog
-  );
-
-  priv->tree_view =
-    gtk_tree_view_new_with_model(GTK_TREE_MODEL(priv->account_store));
-  gtk_tree_view_append_column(GTK_TREE_VIEW(priv->tree_view), column);
-
-  g_signal_connect(
-    G_OBJECT(priv->tree_view),
-    "key-press-event",
-    G_CALLBACK(inf_gtk_permissions_dialog_key_press_event_cb),
-    dialog
-  );
-
-  g_signal_connect(
-    G_OBJECT(priv->tree_view),
-    "button-press-event",
-    G_CALLBACK(inf_gtk_permissions_dialog_button_press_event_cb),
-    dialog
-  );
-
-  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(priv->tree_view));
-  gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
-
-  g_signal_connect(
-    G_OBJECT(selection),
-    "changed",
-    G_CALLBACK(inf_gtk_permissions_dialog_selection_changed_cb),
-    dialog
-  );
-
-  gtk_widget_show(priv->tree_view);
-
-  scroll = gtk_scrolled_window_new(NULL, NULL);
-  gtk_widget_set_size_request(scroll, 200, 350);
-
-  gtk_scrolled_window_set_shadow_type(
-    GTK_SCROLLED_WINDOW(scroll),
-    GTK_SHADOW_IN
-  );
-
-  gtk_scrolled_window_set_policy(
-    GTK_SCROLLED_WINDOW(scroll),
-    GTK_POLICY_AUTOMATIC,
-    GTK_POLICY_AUTOMATIC
-  );
-
-  gtk_container_add(GTK_CONTAINER(scroll), priv->tree_view);
-  gtk_widget_show(scroll);
-
-  image = gtk_image_new_from_stock(GTK_STOCK_ADD, GTK_ICON_SIZE_BUTTON);
-  gtk_widget_show(image);
-
-  priv->add_button = gtk_button_new();
-  gtk_container_add(GTK_CONTAINER(priv->add_button), image);
-
-  g_signal_connect(
-    G_OBJECT(priv->add_button),
-    "clicked",
-    G_CALLBACK(inf_gtk_permissions_dialog_add_clicked_cb),
-    dialog
-  );
-
-  gtk_widget_show(priv->add_button);
-
-  image = gtk_image_new_from_stock(GTK_STOCK_REMOVE, GTK_ICON_SIZE_BUTTON);
-  gtk_widget_show(image);
-
-  priv->remove_button = gtk_button_new();
-  gtk_container_add(GTK_CONTAINER(priv->remove_button), image);
-
-  g_signal_connect(
-    G_OBJECT(priv->remove_button),
-    "clicked",
-    G_CALLBACK(inf_gtk_permissions_dialog_remove_clicked_cb),
-    dialog
-  );
-
-  gtk_widget_show(priv->remove_button);
-
-  buttons_hbox = gtk_hbox_new(FALSE, 12);
-
-  gtk_box_pack_start(
-    GTK_BOX(buttons_hbox),
-    priv->add_button,
-    FALSE,
-    FALSE,
-    0
-  );
-
-  gtk_box_pack_start(
-    GTK_BOX(buttons_hbox),
-    priv->remove_button,
-    FALSE,
-    FALSE,
-    0
-  );
-
-  gtk_widget_show(buttons_hbox);
-  account_list_vbox = gtk_vbox_new(FALSE, 12);
-
-  gtk_box_pack_start(
-    GTK_BOX(account_list_vbox),
-    scroll,
-    TRUE,
-    TRUE,
-    0
-  );
-
-  gtk_box_pack_start(
-    GTK_BOX(account_list_vbox),
-    buttons_hbox,
-    FALSE,
-    FALSE,
-    0
-  );
-
-  gtk_widget_show(account_list_vbox);
-
-  priv->sheet_view = inf_gtk_acl_sheet_view_new();
-
-  g_signal_connect(
-    G_OBJECT(priv->sheet_view),
-    "sheet-changed",
-    G_CALLBACK(inf_gtk_permissions_dialog_sheet_changed_cb),
-    dialog
-  );
-
-  gtk_widget_show(priv->sheet_view);
-
-  hbox = gtk_hbox_new(FALSE, 12);
-  gtk_box_pack_start(GTK_BOX(hbox), account_list_vbox, FALSE, TRUE, 0);
-  gtk_box_pack_start(GTK_BOX(hbox), priv->sheet_view, TRUE, TRUE, 0);
-  gtk_widget_show(hbox);
-
-  priv->status_image = gtk_image_new();
-  gtk_widget_show(priv->status_image);
-
-  priv->status_text = gtk_label_new(NULL);
-  gtk_label_set_max_width_chars(GTK_LABEL(priv->status_text), 50);
-  gtk_label_set_width_chars(GTK_LABEL(priv->status_text), 50);
-  gtk_label_set_line_wrap(GTK_LABEL(priv->status_text), TRUE);
-  gtk_misc_set_alignment(GTK_MISC(priv->status_text), 0.0, 0.5);
-  gtk_widget_show(priv->status_text);
-
-  status_hbox = gtk_hbox_new(FALSE, 12);
-
-  gtk_box_pack_start(
-    GTK_BOX(status_hbox),
-    priv->status_image,
-    FALSE,
-    FALSE,
-    0
-  );
-
-  gtk_box_pack_start(
-    GTK_BOX(status_hbox),
-    priv->status_text,
-    TRUE,
-    TRUE,
-    0
-  );
-
-  gtk_widget_show(status_hbox);
-
-  vbox = gtk_vbox_new(FALSE, 12);
-  gtk_box_pack_start(GTK_BOX(vbox), status_hbox, FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-  gtk_widget_show(vbox);
-
-  image = gtk_image_new_from_stock(
-    GTK_STOCK_DIALOG_AUTHENTICATION,
-    GTK_ICON_SIZE_DIALOG
-  );
-
-  gtk_misc_set_alignment(GTK_MISC(image), 0.0, 0.0);
-  gtk_widget_show(image);
-
-  image_hbox = gtk_hbox_new(FALSE, 12);
-  gtk_box_pack_start(GTK_BOX(image_hbox), image, FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(image_hbox), vbox, TRUE, TRUE, 0);
-  gtk_widget_show(image_hbox);
-
-#if GTK_CHECK_VERSION(2,14,0)
-  dialog_vbox = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
-#else
-  dialog_vbox = GTK_DIALOG(dialog)->vbox;
-#endif
-
-  gtk_box_set_spacing(GTK_BOX(dialog_vbox), 12);
-  gtk_box_pack_start(GTK_BOX(dialog_vbox), image_hbox, FALSE, FALSE, 0);
-
-  gtk_container_set_border_width(GTK_CONTAINER(dialog), 12);
-  gtk_window_set_resizable(GTK_WINDOW(dialog), TRUE);
-  gtk_window_set_default_size(GTK_WINDOW(dialog), 640, 480);
 }
 
-static GObject*
-inf_gtk_permissions_dialog_constructor(GType type,
-                                       guint n_properties,
-                                       GObjectConstructParam* properties)
+static void
+inf_gtk_permissions_dialog_constructed(GObject* object)
 {
-  GObject* object;
   InfGtkPermissionsDialogPrivate* priv;
 
-  object = G_OBJECT_CLASS(parent_class)->constructor(
-    type,
-    n_properties,
-    properties
+  G_OBJECT_CLASS(inf_gtk_permissions_dialog_parent_class)->constructed(
+    object
   );
 
   priv = INF_GTK_PERMISSIONS_DIALOG_PRIVATE(object);
@@ -2996,8 +2695,6 @@ inf_gtk_permissions_dialog_constructor(GType type,
     INF_GTK_PERMISSIONS_DIALOG(object),
     NULL
   );
-
-  return object;
 }
 
 static void
@@ -3009,37 +2706,6 @@ inf_gtk_permissions_dialog_dispose(GObject* object)
   dialog = INF_GTK_PERMISSIONS_DIALOG(object);
   priv = INF_GTK_PERMISSIONS_DIALOG_PRIVATE(dialog);
 
-  if(priv->renderer != NULL)
-  {
-    gtk_cell_renderer_stop_editing(GTK_CELL_RENDERER(priv->renderer), TRUE);
-
-    inf_signal_handlers_disconnect_by_func(
-      G_OBJECT(priv->renderer),
-      G_CALLBACK(inf_gtk_permissions_dialog_renderer_editing_started_cb),
-      dialog
-    );
-
-    inf_signal_handlers_disconnect_by_func(
-      G_OBJECT(priv->renderer),
-      G_CALLBACK(inf_gtk_permissions_dialog_renderer_editing_canceled_cb),
-      dialog
-    );
-
-    inf_signal_handlers_disconnect_by_func(
-      G_OBJECT(priv->renderer),
-      G_CALLBACK(inf_gtk_permissions_dialog_renderer_edited_cb),
-      dialog
-    );
-
-    inf_signal_handlers_disconnect_by_func(
-      G_OBJECT(priv->renderer),
-      G_CALLBACK(inf_gtk_permissions_dialog_renderer_changed_cb),
-      dialog
-    );
-
-    priv->renderer = NULL;
-  }
-
   while(priv->remove_acl_account_requests != NULL)
   {
     inf_gtk_permissions_dialog_remove_remove_acl_account_request(
@@ -3064,27 +2730,7 @@ inf_gtk_permissions_dialog_dispose(GObject* object)
   g_assert(priv->set_acl_requests == NULL);
   g_assert(priv->pending_sheets == NULL);
 
-  if(priv->account_store != NULL)
-  {
-    g_object_unref(priv->account_store);
-    priv->account_store = NULL;
-  }
-
-  /* During parent disposure, this callback might be called, leading to a
-   * crash since we have already disposed of all our resources, therefore
-   * explicitly disconnect here. */
-  if(priv->sheet_view != NULL)
-  {
-    inf_signal_handlers_disconnect_by_func(
-      G_OBJECT(priv->sheet_view),
-      G_CALLBACK(inf_gtk_permissions_dialog_sheet_changed_cb),
-      dialog
-    );
-
-    priv->sheet_view = NULL;
-  }
-
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_gtk_permissions_dialog_parent_class)->dispose(object);
 }
 
 
@@ -3102,7 +2748,7 @@ inf_gtk_permissions_dialog_finalize(GObject* object)
     g_free(priv->accounts[i].name);
   g_free(priv->accounts);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(inf_gtk_permissions_dialog_parent_class)->finalize(object);
 }
 
 static void
@@ -3167,21 +2813,115 @@ inf_gtk_permissions_dialog_get_property(GObject* object,
  */
 
 static void
-inf_gtk_permissions_dialog_class_init(gpointer g_class,
-                                       gpointer class_data)
+inf_gtk_permissions_dialog_class_init(
+  InfGtkPermissionsDialogClass* permissions_dialog_class)
 {
   GObjectClass* object_class;
-  object_class = G_OBJECT_CLASS(g_class);
+  object_class = G_OBJECT_CLASS(permissions_dialog_class);
 
-  parent_class = GTK_DIALOG_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfGtkPermissionsDialogPrivate));
-
-  object_class->constructor = inf_gtk_permissions_dialog_constructor;
+  object_class->constructed = inf_gtk_permissions_dialog_constructed;
   object_class->dispose = inf_gtk_permissions_dialog_dispose;
   object_class->finalize = inf_gtk_permissions_dialog_finalize;
   object_class->set_property = inf_gtk_permissions_dialog_set_property;
   object_class->get_property = inf_gtk_permissions_dialog_get_property;
 
+  gtk_widget_class_set_template_from_resource(
+    GTK_WIDGET_CLASS(object_class),
+    "/de/0x539/libinfgtk/ui/infgtkpermissionsdialog.ui"
+  );
+
+  gtk_widget_class_bind_template_child_private(
+    GTK_WIDGET_CLASS(object_class),
+    InfGtkPermissionsDialog,
+    account_store
+  );
+
+  gtk_widget_class_bind_template_child_private(
+    GTK_WIDGET_CLASS(object_class),
+    InfGtkPermissionsDialog,
+    status_text
+  );
+
+  gtk_widget_class_bind_template_child_private(
+    GTK_WIDGET_CLASS(object_class),
+    InfGtkPermissionsDialog,
+    tree_view
+  );
+
+  gtk_widget_class_bind_template_child_private(
+    GTK_WIDGET_CLASS(object_class),
+    InfGtkPermissionsDialog,
+    sheet_view
+  );
+
+  gtk_widget_class_bind_template_child_private(
+    GTK_WIDGET_CLASS(object_class),
+    InfGtkPermissionsDialog,
+    add_button
+  );
+
+  gtk_widget_class_bind_template_child_private(
+    GTK_WIDGET_CLASS(object_class),
+    InfGtkPermissionsDialog,
+    remove_button
+  );
+
+  gtk_widget_class_bind_template_child_private(
+    GTK_WIDGET_CLASS(object_class),
+    InfGtkPermissionsDialog,
+    renderer
+  );
+
+  gtk_widget_class_bind_template_callback(
+    GTK_WIDGET_CLASS(object_class),
+    inf_gtk_permissions_dialog_key_press_event_cb
+  );
+
+  gtk_widget_class_bind_template_callback(
+    GTK_WIDGET_CLASS(object_class),
+    inf_gtk_permissions_dialog_button_press_event_cb
+  );
+
+  gtk_widget_class_bind_template_callback(
+    GTK_WIDGET_CLASS(object_class),
+    inf_gtk_permissions_dialog_selection_changed_cb
+  );
+
+  gtk_widget_class_bind_template_callback(
+    GTK_WIDGET_CLASS(object_class),
+    inf_gtk_permissions_dialog_renderer_editing_started_cb
+  );
+
+  gtk_widget_class_bind_template_callback(
+    GTK_WIDGET_CLASS(object_class),
+    inf_gtk_permissions_dialog_renderer_editing_canceled_cb
+  );
+
+  gtk_widget_class_bind_template_callback(
+    GTK_WIDGET_CLASS(object_class),
+    inf_gtk_permissions_dialog_renderer_edited_cb
+  );
+
+  gtk_widget_class_bind_template_callback(
+    GTK_WIDGET_CLASS(object_class),
+    inf_gtk_permissions_dialog_renderer_changed_cb
+  );
+
+  gtk_widget_class_bind_template_callback(
+    GTK_WIDGET_CLASS(object_class),
+    inf_gtk_permissions_dialog_add_clicked_cb
+  );
+
+  gtk_widget_class_bind_template_callback(
+    GTK_WIDGET_CLASS(object_class),
+    inf_gtk_permissions_dialog_remove_clicked_cb
+  );
+
+  gtk_widget_class_bind_template_callback(
+    GTK_WIDGET_CLASS(object_class),
+    inf_gtk_permissions_dialog_sheet_changed_cb
+  );
+
   g_object_class_install_property(
     object_class,
     PROP_BROWSER,
@@ -3208,56 +2948,26 @@ inf_gtk_permissions_dialog_class_init(gpointer g_class,
   );
 }
 
-GType
-inf_gtk_permissions_dialog_get_type(void)
-{
-  static GType permissions_dialog_type = 0;
-
-  if(!permissions_dialog_type)
-  {
-    static const GTypeInfo permissions_dialog_type_info = {
-      sizeof(InfGtkPermissionsDialogClass),    /* class_size */
-      NULL,                                    /* base_init */
-      NULL,                                    /* base_finalize */
-      inf_gtk_permissions_dialog_class_init,   /* class_init */
-      NULL,                                    /* class_finalize */
-      NULL,                                    /* class_data */
-      sizeof(InfGtkPermissionsDialog),         /* instance_size */
-      0,                                       /* n_preallocs */
-      inf_gtk_permissions_dialog_init,         /* instance_init */
-      NULL                                     /* value_table */
-    };
-
-    permissions_dialog_type = g_type_register_static(
-      GTK_TYPE_DIALOG,
-      "InfGtkPermissionsDialog",
-      &permissions_dialog_type_info,
-      0
-    );
-  }
-
-  return permissions_dialog_type;
-}
-
 /*
  * Public API.
  */
 
 /**
- * inf_gtk_permissions_dialog_new:
+ * inf_gtk_permissions_dialog_new: (constructor)
  * @parent: Parent #GtkWindow of the dialog.
  * @dialog_flags: Flags for the dialog, see #GtkDialogFlags.
- * @browser: The #InfBrowser containing the node to show permissions for, or
- * %NULL.
- * @iter: An iterator pointing to the node to show permissions for, or %NULL.
+ * @browser: (allow-none): The #InfBrowser containing the node to show
+ * permissions for, or %NULL.
+ * @iter: (allow-none): An iterator pointing to the node to show permissions
+ * for, or %NULL.
  *
  * Creates a new #InfGtkPermissionsDialog, showing the ACL for the node
  * @iter points to inside @browser. If @browser is %NULL, @iter must be %NULL,
  * too. In that case no permissions are shown, and the node to be shown can
  * be set later with inf_gtk_permissions_dialog_set_node().
  *
- * Returns: A new #InfGtkPermissionsDialog. Free with gtk_widget_destroy()
- * when no longer needed.
+ * Returns: (transfer full): A new #InfGtkPermissionsDialog. Free with
+ * gtk_widget_destroy() when no longer needed.
  */
 InfGtkPermissionsDialog*
 inf_gtk_permissions_dialog_new(GtkWindow* parent,
@@ -3284,11 +2994,6 @@ inf_gtk_permissions_dialog_new(GtkWindow* parent,
   if(dialog_flags & GTK_DIALOG_DESTROY_WITH_PARENT)
     gtk_window_set_destroy_with_parent(GTK_WINDOW(object), TRUE);
 
-#if !GTK_CHECK_VERSION(2,90,7)
-  if(dialog_flags & GTK_DIALOG_NO_SEPARATOR)
-    gtk_dialog_set_has_separator(GTK_DIALOG(object), FALSE);
-#endif
-
   gtk_window_set_transient_for(GTK_WINDOW(object), parent);
   return INF_GTK_PERMISSIONS_DIALOG(object);
 }
@@ -3296,9 +3001,10 @@ inf_gtk_permissions_dialog_new(GtkWindow* parent,
 /**
  * inf_gtk_permissions_dialog_set_node:
  * @dialog: A #InfGtkPermissionsDialog.
- * @browser: The #InfBrowser containing the node to show permissions for, or
- * %NULL.
- * @iter: An iterator pointing to the node to show permissions for, or %NULL.
+ * @browser: (allow-none): The #InfBrowser containing the node to show
+ * permissions for, or %NULL.
+ * @iter: (allow-none): An iterator pointing to the node to show permissions
+ * for, or %NULL.
  *
  * Changes the node the dialog shows permissions for. To unset the node, both
  * @browser and @iter should be %NULL.
