@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -33,12 +33,10 @@
  *
  * It is based on requests, represented by the #InfAdoptedRequest class. If
  * there is at least one local #InfUser in the algorithm's user table, then
- * you can create own reequests by the functions
- * inf_adopted_algorithm_generate_request(),
- * inf_adopted_algorithm_generate_request_noexec(),
- * inf_adopted_algorithm_generate_undo() and
- * inf_adopted_algorithm_generate_redo(). Remote requests can be applied via
- * inf_adopted_algorithm_receive_request(). This class does not take care of
+ * you can create own requests with the
+ * inf_adopted_algorithm_generate_request() function.
+ * Remote requests can be applied via
+ * inf_adopted_algorithm_execute_request(). This class does not take care of
  * transfering the generated requests to other users which is the scope of
  * #InfAdoptedSession.
  *
@@ -69,7 +67,6 @@
  * dynamically as O(active users^2). */
 
 #include <libinfinity/adopted/inf-adopted-algorithm.h>
-#include <libinfinity/inf-marshal.h>
 #include <libinfinity/inf-signals.h>
 #include <libinfinity/inf-i18n.h>
 
@@ -127,17 +124,10 @@ enum {
 #define INF_ADOPTED_ALGORITHM_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_ADOPTED_TYPE_ALGORITHM, InfAdoptedAlgorithmPrivate))
 #define INF_ADOPTED_ALGORITHM_PRIVATE(obj)     ((InfAdoptedAlgorithmPrivate*)(obj)->priv)
 
-static GObjectClass* parent_class;
 static guint algorithm_signals[LAST_SIGNAL];
 
-static gint64
-inf_adopted_algorithm_get_real_time()
-{
-  /* TODO: Replace by g_get_real_time() once we depend on glib >=2.28 */
-  GTimeVal timeval;
-  g_get_current_time(&timeval);
-  return (gint64)timeval.tv_sec * 1000000 + timeval.tv_usec;
-}
+G_DEFINE_TYPE_WITH_CODE(InfAdoptedAlgorithm, inf_adopted_algorithm, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfAdoptedAlgorithm))
 
 /* Returns a new state vector v so that both first and second are causally
  * before v and so that there is no other state vector with the same property
@@ -1036,13 +1026,10 @@ inf_adopted_algorithm_apply_request(InfAdoptedAlgorithm* algorithm,
 }
 
 static void
-inf_adopted_algorithm_init(GTypeInstance* instance,
-                           gpointer g_class)
+inf_adopted_algorithm_init(InfAdoptedAlgorithm* algorithm)
 {
-  InfAdoptedAlgorithm* algorithm;
   InfAdoptedAlgorithmPrivate* priv;
 
-  algorithm = INF_ADOPTED_ALGORITHM(instance);
   algorithm->priv = INF_ADOPTED_ALGORITHM_GET_PRIVATE(algorithm);
   priv = INF_ADOPTED_ALGORITHM_PRIVATE(algorithm);
 
@@ -1063,7 +1050,7 @@ inf_adopted_algorithm_init(GTypeInstance* instance,
 }
 
 static void
-inf_adopted_algorithm_constructor_foreach_user_func(InfUser* user,
+inf_adopted_algorithm_constructed_foreach_user_func(InfUser* user,
                                                     gpointer user_data)
 {
   InfAdoptedAlgorithm* algorithm;
@@ -1074,7 +1061,7 @@ inf_adopted_algorithm_constructor_foreach_user_func(InfUser* user,
 }
 
 static void
-inf_adopted_algorithm_constructor_foreach_local_user_func(InfUser* user,
+inf_adopted_algorithm_constructed_foreach_local_user_func(InfUser* user,
                                                           gpointer user_data)
 {
   InfAdoptedAlgorithm* algorithm;
@@ -1084,21 +1071,14 @@ inf_adopted_algorithm_constructor_foreach_local_user_func(InfUser* user,
   inf_adopted_algorithm_add_local_user(algorithm, INF_ADOPTED_USER(user));
 }
 
-static GObject*
-inf_adopted_algorithm_constructor(GType type,
-                                  guint n_construct_properties,
-                                  GObjectConstructParam* construct_properties)
+static void
+inf_adopted_algorithm_constructed(GObject* object)
 {
-  GObject* object;
   InfAdoptedAlgorithm* algorithm;
   InfAdoptedAlgorithmPrivate* priv;
   gboolean modified;
 
-  object = G_OBJECT_CLASS(parent_class)->constructor(
-    type,
-    n_construct_properties,
-    construct_properties
-  );
+  G_OBJECT_CLASS(inf_adopted_algorithm_parent_class)->constructed(object);
 
   algorithm = INF_ADOPTED_ALGORITHM(object);
   priv = INF_ADOPTED_ALGORITHM_PRIVATE(algorithm);
@@ -1106,23 +1086,19 @@ inf_adopted_algorithm_constructor(GType type,
   /* Add initial users */
   inf_user_table_foreach_user(
     priv->user_table,
-    inf_adopted_algorithm_constructor_foreach_user_func,
+    inf_adopted_algorithm_constructed_foreach_user_func,
     algorithm
   );
   
   inf_user_table_foreach_local_user(
     priv->user_table,
-    inf_adopted_algorithm_constructor_foreach_local_user_func,
+    inf_adopted_algorithm_constructed_foreach_local_user_func,
     algorithm
   );
 
   g_object_get(G_OBJECT(priv->buffer), "modified", &modified, NULL);
   if(modified == FALSE)
-  {
     priv->buffer_modified_time = inf_adopted_state_vector_copy(priv->current);
-  }
-
-  return object;
 }
 
 static void
@@ -1179,7 +1155,7 @@ inf_adopted_algorithm_dispose(GObject* object)
     priv->user_table = NULL;
   }
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_adopted_algorithm_parent_class)->dispose(object);
 }
 
 static void
@@ -1193,7 +1169,7 @@ inf_adopted_algorithm_finalize(GObject* object)
 
   inf_adopted_state_vector_free(priv->current);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(inf_adopted_algorithm_parent_class)->finalize(object);
 }
 
 static void
@@ -1326,19 +1302,12 @@ inf_adopted_algorithm_can_redo_changed(InfAdoptedAlgorithm* algorithm,
 }
 
 static void
-inf_adopted_algorithm_class_init(gpointer g_class,
-                                 gpointer class_data)
+inf_adopted_algorithm_class_init(InfAdoptedAlgorithmClass* algorithm_class)
 {
   GObjectClass* object_class;
-  InfAdoptedAlgorithmClass* algorithm_class;
+  object_class = G_OBJECT_CLASS(algorithm_class);
 
-  object_class = G_OBJECT_CLASS(g_class);
-  algorithm_class = INF_ADOPTED_ALGORITHM_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfAdoptedAlgorithmPrivate));
-
-  object_class->constructor = inf_adopted_algorithm_constructor;
+  object_class->constructed = inf_adopted_algorithm_constructed;
   object_class->dispose = inf_adopted_algorithm_dispose;
   object_class->finalize = inf_adopted_algorithm_finalize;
   object_class->set_property = inf_adopted_algorithm_set_property;
@@ -1422,7 +1391,7 @@ inf_adopted_algorithm_class_init(gpointer g_class,
    * This signal is emitted every time the can-undo state of a local user
    * in @algorithm's user table changed. The can-undo state defines whether
    * @user can generate an undo request
-   * (via inf_adopted_algorithm_generate_undo()) in the current situation, see
+   * (via inf_adopted_algorithm_generate_request()) in the current situation, see
    * also inf_adopted_algorithm_can_undo().
    */
   algorithm_signals[CAN_UNDO_CHANGED] = g_signal_new(
@@ -1431,7 +1400,7 @@ inf_adopted_algorithm_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfAdoptedAlgorithmClass, can_undo_changed),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT_BOOLEAN,
+    NULL,
     G_TYPE_NONE,
     2,
     INF_ADOPTED_TYPE_USER,
@@ -1449,7 +1418,7 @@ inf_adopted_algorithm_class_init(gpointer g_class,
    * This signal is emitted every time the can-redo state of a local user
    * in @algorithm's user table changed. The can-redo state defines whether
    * @user can generate a redo request
-   * (via inf_adopted_algorithm_generate_redo()) in the current situation, see
+   * (via inf_adopted_algorithm_generate_request()) in the current situation, see
    * also inf_adopted_algorithm_can_redo().
    */
   algorithm_signals[CAN_REDO_CHANGED] = g_signal_new(
@@ -1458,7 +1427,7 @@ inf_adopted_algorithm_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfAdoptedAlgorithmClass, can_redo_changed),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT_BOOLEAN,
+    NULL,
     G_TYPE_NONE,
     2,
     INF_ADOPTED_TYPE_USER,
@@ -1481,7 +1450,7 @@ inf_adopted_algorithm_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfAdoptedAlgorithmClass, begin_execute_request),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT_OBJECT,
+    NULL,
     G_TYPE_NONE,
     2,
     INF_ADOPTED_TYPE_USER,
@@ -1529,55 +1498,24 @@ inf_adopted_algorithm_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfAdoptedAlgorithmClass, end_execute_request),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT_OBJECT_OBJECT_POINTER,
+    NULL,
     G_TYPE_NONE,
     4,
     INF_ADOPTED_TYPE_USER,
     INF_ADOPTED_TYPE_REQUEST,
     INF_ADOPTED_TYPE_REQUEST,
-    G_TYPE_POINTER /* GError* */
+    G_TYPE_ERROR
   );
 }
 
-GType
-inf_adopted_algorithm_get_type(void)
-{
-  static GType algorithm_type = 0;
-
-  if(!algorithm_type)
-  {
-    static const GTypeInfo algorithm_type_info = {
-      sizeof(InfAdoptedAlgorithmClass),   /* class_size */
-      NULL,                               /* base_init */
-      NULL,                               /* base_finalize */
-      inf_adopted_algorithm_class_init,   /* class_init */
-      NULL,                               /* class_finalize */
-      NULL,                               /* class_data */
-      sizeof(InfAdoptedAlgorithm),        /* instance_size */
-      0,                                  /* n_preallocs */
-      inf_adopted_algorithm_init,         /* instance_init */
-      NULL                                /* value_table */
-    };
-
-    algorithm_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfAdoptedAlgorithm",
-      &algorithm_type_info,
-      0
-    );
-  }
-
-  return algorithm_type;
-}
-
 /**
- * inf_adopted_algorithm_new:
+ * inf_adopted_algorithm_new: (constructor)
  * @user_table: The table of participating users.
  * @buffer: The buffer to apply operations to.
  *
  * Creates a #InfAdoptedAlgorithm.
  *
- * Return Value: A new #InfAdoptedAlgorithm.
+ * Returns: (transfer full): A new #InfAdoptedAlgorithm.
  **/
 InfAdoptedAlgorithm*
 inf_adopted_algorithm_new(InfUserTable* user_table,
@@ -1598,7 +1536,7 @@ inf_adopted_algorithm_new(InfUserTable* user_table,
 }
 
 /**
- * inf_adopted_algorithm_new_full:
+ * inf_adopted_algorithm_new_full: (constructor)
  * @user_table: The table of participating users.
  * @buffer: The buffer to apply operations to.
  * @max_total_log_size: The maxmimum number of operations to keep in all
@@ -1617,7 +1555,7 @@ inf_adopted_algorithm_new(InfUserTable* user_table,
  *
  * The default value is 2048.
  *
- * Return Value: A new #InfAdoptedAlgorithm.
+ * Returns: (transfer full): A new #InfAdoptedAlgorithm.
  **/
 InfAdoptedAlgorithm*
 inf_adopted_algorithm_new_full(InfUserTable* user_table,
@@ -1645,7 +1583,7 @@ inf_adopted_algorithm_new_full(InfUserTable* user_table,
  *
  * Returns the current vector time of @algorithm.
  *
- * Return Value: A #InfAdoptedStateVector owned by @algorithm.
+ * Returns: (transfer none): A #InfAdoptedStateVector owned by @algorithm.
  **/
 InfAdoptedStateVector*
 inf_adopted_algorithm_get_current(InfAdoptedAlgorithm* algorithm)
@@ -1665,8 +1603,9 @@ inf_adopted_algorithm_get_current(InfAdoptedAlgorithm* algorithm)
  * execution is not re-entrant, i.e. two requests cannot be executed
  * concurrently at the same time, or recursively.
  *
- * Returns: The request that @algorithm is currently processing, or %NULL.
- * The return value must not be freed by the caller.
+ * Returns: (transfer none) (allow-none): The request that @algorithm is
+ * currently processing, or %NULL. The return value must not be freed by
+ * the caller.
  */
 InfAdoptedRequest*
 inf_adopted_algorithm_get_execute_request(InfAdoptedAlgorithm* algorithm)
@@ -1693,8 +1632,8 @@ inf_adopted_algorithm_get_execute_request(InfAdoptedAlgorithm* algorithm)
  * with the @apply parameter set to %FALSE, so that the algorithm knows that
  * the request has been applied.
  *
- * Returns: A new #InfAdoptedRequest. Free with g_object_unref() when no
- * longer needed.
+ * Returns: (transfer full): A new #InfAdoptedRequest. Free with
+ * g_object_unref() when no longer needed.
  */
 InfAdoptedRequest*
 inf_adopted_algorithm_generate_request(InfAdoptedAlgorithm* algorithm,
@@ -1721,19 +1660,19 @@ inf_adopted_algorithm_generate_request(InfAdoptedAlgorithm* algorithm,
       priv->current,
       inf_user_get_id(INF_USER(user)),
       operation,
-      inf_adopted_algorithm_get_real_time()
+      g_get_real_time()
     );
   case INF_ADOPTED_REQUEST_UNDO:
     return inf_adopted_request_new_undo(
       priv->current,
       inf_user_get_id(INF_USER(user)),
-      inf_adopted_algorithm_get_real_time()
+      g_get_real_time()
     );
   case INF_ADOPTED_REQUEST_REDO:
     return inf_adopted_request_new_redo(
       priv->current,
       inf_user_get_id(INF_USER(user)),
-      inf_adopted_algorithm_get_real_time()
+      g_get_real_time()
     );
   default:
     g_return_val_if_reached(NULL);
@@ -1745,7 +1684,7 @@ inf_adopted_algorithm_generate_request(InfAdoptedAlgorithm* algorithm,
  * inf_adopted_algorithm_translate_request:
  * @algorithm: A #InfAdoptedAlgorithm.
  * @request: A #InfAdoptedRequest.
- * @to: The state vector to translate @request to.
+ * @to: (transfer none): The state vector to translate @request to.
  *
  * Translates @request so that it can be applied to the document at state @to.
  * @request will not be modified but a new, translated request is returned
@@ -1756,8 +1695,8 @@ inf_adopted_algorithm_generate_request(InfAdoptedAlgorithm* algorithm,
  * translated in forward direction, so @request's vector time must be
  * causally before (see inf_adopted_state_vector_causally_before()) @to.
  *
- * Returns: A new or cached #InfAdoptedRequest. Free with g_object_unref()
- * when no longer needed.
+ * Returns: (transfer full): A new or cached #InfAdoptedRequest. Free with
+ * g_object_unref() when no longer needed.
  */
 InfAdoptedRequest*
 inf_adopted_algorithm_translate_request(InfAdoptedAlgorithm* algorithm,
@@ -1930,10 +1869,7 @@ inf_adopted_algorithm_execute_request(InfAdoptedAlgorithm* algorithm,
   g_return_val_if_fail(priv->execute_request == NULL, FALSE);
   priv->execute_request = request;
 
-  inf_adopted_request_set_execute_time(
-    request,
-    inf_adopted_algorithm_get_real_time()
-  );
+  inf_adopted_request_set_execute_time(request, g_get_real_time());
 
   g_signal_emit(
     G_OBJECT(algorithm),
@@ -2251,7 +2187,7 @@ inf_adopted_algorithm_cleanup(InfAdoptedAlgorithm* algorithm)
  * Undo request even if the max-total-log-size is already exceeded if @user
  * does not know yet that it is exceeded.
  *
- * Return Value: %TRUE if Undo is possible, %FALSE otherwise.
+ * Returns: %TRUE if Undo is possible, %FALSE otherwise.
  **/
 gboolean
 inf_adopted_algorithm_can_undo(InfAdoptedAlgorithm* algorithm,
@@ -2292,7 +2228,7 @@ inf_adopted_algorithm_can_undo(InfAdoptedAlgorithm* algorithm,
  * Redo request even if the max-total-log-size is already exceeded if @user
  * does not know yet that it is exceeded.
  *
- * Return Value: %TRUE if Redo is possible, %FALSE otherwise.
+ * Returns: %TRUE if Redo is possible, %FALSE otherwise.
  **/
 gboolean
 inf_adopted_algorithm_can_redo(InfAdoptedAlgorithm* algorithm,
