@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2010 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -49,7 +49,7 @@ struct _InfAdoptedRequestLogClass {
   GObjectClass parent_class;
 
   /*< public >*/
-  void(*add_request)(InfAdoptedRequestLog* request_log,
+  void(*add_request)(InfAdoptedRequestLog* log,
                      InfAdoptedRequest* request);
 };
 
@@ -125,6 +125,14 @@ InfAdoptedRequest*
 inf_adopted_request_log_lower_related(InfAdoptedRequestLog* log,
                                       guint n);
 
+void
+inf_adopted_request_log_add_cached_request(InfAdoptedRequestLog* log,
+                                           InfAdoptedRequest* request);
+
+InfAdoptedRequest*
+inf_adopted_request_log_lookup_cached_request(InfAdoptedRequestLog* log,
+                                              InfAdoptedStateVector* vec);
+
 G_END_DECLS
 
 #endif /* __INF_ADOPTED_REQUEST_LOG_H__ */
