@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2010 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -86,15 +86,18 @@ inf_adopted_request_get_type(void) G_GNUC_CONST;
 InfAdoptedRequest*
 inf_adopted_request_new_do(InfAdoptedStateVector* vector,
                            guint user_id,
-                           InfAdoptedOperation* operation);
+                           InfAdoptedOperation* operation,
+                           gint64 received);
 
 InfAdoptedRequest*
 inf_adopted_request_new_undo(InfAdoptedStateVector* vector,
-                             guint user_id);
+                             guint user_id,
+                             gint64 received);
 
 InfAdoptedRequest*
 inf_adopted_request_new_redo(InfAdoptedStateVector* vector,
-                             guint user_id);
+                             guint user_id,
+                             gint64 received);
 
 InfAdoptedRequest*
 inf_adopted_request_copy(InfAdoptedRequest* request);
@@ -114,18 +117,25 @@ inf_adopted_request_get_operation(InfAdoptedRequest* request);
 guint
 inf_adopted_request_get_index(InfAdoptedRequest* request);
 
+gint64
+inf_adopted_request_get_receive_time(InfAdoptedRequest* request);
+
+gint64
+inf_adopted_request_get_execute_time(InfAdoptedRequest* request);
+
+void
+inf_adopted_request_set_execute_time(InfAdoptedRequest* request,
+                                     gint64 time);
+
 gboolean
 inf_adopted_request_need_concurrency_id(InfAdoptedRequest* request,
                                         InfAdoptedRequest* against);
 
-InfAdoptedConcurrencyId
-inf_adopted_request_get_concurrency_id(InfAdoptedRequest* request,
-                                       InfAdoptedRequest* against);
-
 InfAdoptedRequest*
 inf_adopted_request_transform(InfAdoptedRequest* request,
                               InfAdoptedRequest* against,
-                              InfAdoptedConcurrencyId concurrency_id);
+                              InfAdoptedRequest* request_lcs,
+                              InfAdoptedRequest* against_lcs);
 
 InfAdoptedRequest*
 inf_adopted_request_mirror(InfAdoptedRequest* request,
