@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,19 +17,6 @@
  * MA 02110-1301, USA.
  */
 
-/* TODO: warning if no update from a particular non-local user for some time */
-
-#include <libinfinity/adopted/inf-adopted-session.h>
-#include <libinfinity/adopted/inf-adopted-no-operation.h>
-#include <libinfinity/common/inf-xml-util.h>
-#include <libinfinity/common/inf-error.h>
-#include <libinfinity/inf-i18n.h>
-#include <libinfinity/inf-marshal.h>
-#include <libinfinity/inf-signals.h>
-
-#include <string.h>
-#include <time.h>
-
 /**
  * SECTION:inf-adopted-session
  * @title: InfAdoptedSession
@@ -48,6 +35,18 @@
  * to cleanup their request logs and request caches).
  */
 
+/* TODO: warning if no update from a particular non-local user for some time */
+
+#include <libinfinity/adopted/inf-adopted-session.h>
+#include <libinfinity/adopted/inf-adopted-no-operation.h>
+#include <libinfinity/common/inf-xml-util.h>
+#include <libinfinity/common/inf-error.h>
+#include <libinfinity/inf-i18n.h>
+#include <libinfinity/inf-signals.h>
+
+#include <string.h>
+#include <time.h>
+
 typedef struct _InfAdoptedSessionToXmlSyncForeachData
   InfAdoptedSessionToXmlSyncForeachData;
 struct _InfAdoptedSessionToXmlSyncForeachData {
@@ -59,7 +58,7 @@ typedef struct _InfAdoptedSessionLocalUser InfAdoptedSessionLocalUser;
 struct _InfAdoptedSessionLocalUser {
   InfAdoptedUser* user;
   InfAdoptedStateVector* last_send_vector;
-  time_t noop_time;
+  time_t noop_time; /* TODO: should be monotonic time */
 };
 
 typedef struct _InfAdoptedSessionPrivate InfAdoptedSessionPrivate;
@@ -97,13 +96,15 @@ enum {
 
 #define INF_ADOPTED_SESSION_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_ADOPTED_TYPE_SESSION, InfAdoptedSessionPrivate))
 
-static InfSessionClass* parent_class;
 static guint session_signals[LAST_SIGNAL];
 
 static GQuark inf_adopted_session_error_quark;
 /* TODO: This should perhaps be a property: */
 static const int INF_ADOPTED_SESSION_NOOP_INTERVAL = 30;
 
+G_DEFINE_TYPE_WITH_CODE(InfAdoptedSession, inf_adopted_session, INF_TYPE_SESSION,
+  G_ADD_PRIVATE(InfAdoptedSession))
+
 /*
  * Utility functions.
  */
@@ -173,11 +174,10 @@ inf_adopted_session_validate_request(InfAdoptedRequestLog* log,
     case INF_ADOPTED_REQUEST_UNDO:
       if(inf_adopted_request_log_next_undo(log) == NULL)
       {
-        g_set_error(
+        g_set_error_literal(
           error,
           inf_adopted_session_error_quark,
           INF_ADOPTED_SESSION_ERROR_INVALID_REQUEST,
-          "%s",
           _("Undo received, but no previous request found")
         );
 
@@ -190,11 +190,10 @@ inf_adopted_session_validate_request(InfAdoptedRequestLog* log,
     case INF_ADOPTED_REQUEST_REDO:
       if(inf_adopted_request_log_next_redo(log) == NULL)
       {
-        g_set_error(
+        g_set_error_literal(
           error,
           inf_adopted_session_error_quark,
           INF_ADOPTED_SESSION_ERROR_INVALID_REQUEST,
-          "%s",
           _("Redo received, but no previous request found")
         );
 
@@ -475,11 +474,10 @@ inf_adopted_session_process_request(InfAdoptedSession* session,
     
     if(reject_request)
     {
-      g_set_error(
+      g_set_error_literal(
         &local_error,
         inf_adopted_session_error_quark,
         INF_ADOPTED_SESSION_ERROR_INVALID_REQUEST,
-        "%s",
         _("The request was rejected via the API")
       );
 
@@ -694,7 +692,7 @@ inf_adopted_session_add_local_user_cb(InfUserTable* user_table,
 }
 
 static void
-inf_adopted_session_constructor_foreach_local_user_func(InfUser* user,
+inf_adopted_session_constructed_foreach_local_user_func(InfUser* user,
                                                         gpointer user_data)
 {
   g_assert(INF_ADOPTED_IS_USER(user));
@@ -794,13 +792,9 @@ inf_adopted_session_create_algorithm(InfAdoptedSession* session)
  */
 
 static void
-inf_adopted_session_init(GTypeInstance* instance,
-                         gpointer g_class)
+inf_adopted_session_init(InfAdoptedSession* session)
 {
-  InfAdoptedSession* session;
   InfAdoptedSessionPrivate* priv;
-
-  session = INF_ADOPTED_SESSION(instance);
   priv = INF_ADOPTED_SESSION_PRIVATE(session);
 
   priv->io = NULL;
@@ -812,22 +806,15 @@ inf_adopted_session_init(GTypeInstance* instance,
   priv->request_buffer = NULL;
 }
 
-static GObject*
-inf_adopted_session_constructor(GType type,
-                                guint n_construct_properties,
-                                GObjectConstructParam* construct_properties)
+static void
+inf_adopted_session_constructed(GObject* object)
 {
-  GObject* object;
   InfAdoptedSession* session;
   InfAdoptedSessionPrivate* priv;
   InfSessionStatus status;
   InfUserTable* user_table;
 
-  object = G_OBJECT_CLASS(parent_class)->constructor(
-    type,
-    n_construct_properties,
-    construct_properties
-  );
+  G_OBJECT_CLASS(inf_adopted_session_parent_class)->constructed(object);
 
   session = INF_ADOPTED_SESSION(object);
   priv = INF_ADOPTED_SESSION_PRIVATE(session);
@@ -874,11 +861,9 @@ inf_adopted_session_constructor(GType type,
    * though in synchronizing state no local users can exist. */
   inf_user_table_foreach_local_user(
     user_table,
-    inf_adopted_session_constructor_foreach_local_user_func,
+    inf_adopted_session_constructed_foreach_local_user_func,
     session
   );
-
-  return object;
 }
 
 static void
@@ -914,7 +899,7 @@ inf_adopted_session_dispose(GObject* object)
 
   /* This calls the close vfunc if the session is running, in which we
    * free the local users. */
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_adopted_session_parent_class)->dispose(object);
 
   g_assert(priv->local_users == NULL);
 
@@ -957,7 +942,7 @@ inf_adopted_session_finalize(GObject* object)
   /* Should have been freed in close, called by dispose */
   g_assert(priv->local_users == NULL);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(inf_adopted_session_parent_class)->finalize(object);
 }
 
 static void
@@ -1069,7 +1054,10 @@ inf_adopted_session_to_xml_sync(InfSession* session,
   priv = INF_ADOPTED_SESSION_PRIVATE(session);
   g_assert(priv->algorithm != NULL);
 
-  INF_SESSION_CLASS(parent_class)->to_xml_sync(session, parent);
+  INF_SESSION_CLASS(inf_adopted_session_parent_class)->to_xml_sync(
+    session,
+    parent
+  );
 
   foreach_data.session = INF_ADOPTED_SESSION(session);
   foreach_data.parent_xml = parent;
@@ -1091,6 +1079,7 @@ inf_adopted_session_process_xml_sync(InfSession* session,
   InfAdoptedRequest* request;
   InfAdoptedUser* user;
   InfAdoptedRequestLog* log;
+  InfSessionClass* parent_class;
 
   if(strcmp((const char*)xml->name, "sync-request") == 0)
   {
@@ -1127,12 +1116,8 @@ inf_adopted_session_process_xml_sync(InfSession* session,
     return TRUE;
   }
 
-  return INF_SESSION_CLASS(parent_class)->process_xml_sync(
-    session,
-    connection, 
-    xml,
-    error
-  );
+  parent_class = INF_SESSION_CLASS(inf_adopted_session_parent_class);
+  return parent_class->process_xml_sync(session, connection, xml, error);
 }
 
 static InfCommunicationScope
@@ -1160,6 +1145,8 @@ inf_adopted_session_process_xml_run(InfSession* session,
   gchar* request_str;
   gchar* user_str;
 
+  InfSessionClass* parent_class;
+
   priv = INF_ADOPTED_SESSION_PRIVATE(session);
 
   if(strcmp((const char*)xml->name, "request") == 0)
@@ -1179,11 +1166,10 @@ inf_adopted_session_process_xml_run(InfSession* session,
     if(inf_user_get_status(INF_USER(user)) == INF_USER_UNAVAILABLE ||
        inf_user_get_connection(INF_USER(user)) != connection)
     {
-      g_set_error(
+      g_set_error_literal(
         error,
         inf_user_error_quark(),
         INF_USER_ERROR_NOT_JOINED,
-        "%s",
         _("User did not join from this connection")
       );
 
@@ -1325,12 +1311,8 @@ inf_adopted_session_process_xml_run(InfSession* session,
     return INF_COMMUNICATION_SCOPE_GROUP;
   }
 
-  return INF_SESSION_CLASS(parent_class)->process_xml_run(
-    session,
-    connection,
-    xml,
-    error
-  );
+  parent_class = INF_SESSION_CLASS(inf_adopted_session_parent_class);
+  return parent_class->process_xml_run(session, connection, xml, error);
 }
 
 static GArray*
@@ -1338,16 +1320,14 @@ inf_adopted_session_get_xml_user_props(InfSession* session,
                                        InfXmlConnection* conn,
                                        const xmlNodePtr xml)
 {
+  InfSessionClass* parent_class;
   GArray* array;
   GParameter* parameter;
   InfAdoptedStateVector* vector;
   xmlChar* time;
 
-  array = INF_SESSION_CLASS(parent_class)->get_xml_user_props(
-    session,
-    conn,
-    xml
-  );
+  parent_class = INF_SESSION_CLASS(inf_adopted_session_parent_class);
+  array = parent_class->get_xml_user_props(session, conn, xml);
 
   /* Vector time */
   time = inf_xml_util_get_attribute(xml, "time");
@@ -1365,29 +1345,6 @@ inf_adopted_session_get_xml_user_props(InfSession* session,
     }
   }
 
-  /* log-begin is not in the  spec */
-#if 0
-  /* Initial request log, only if ID is also given */
-  id_param = inf_session_lookup_user_property(
-    (const GParameter*)array->data,
-    array->len,
-    "id"
-  );
-
-  if(id_param != NULL &&
-     inf_xml_util_get_attribute_uint(xml, "log-begin", &log_begin, NULL))
-  {
-    log = inf_adopted_request_log_new(
-      g_value_get_uint(&id_param->value),
-      log_begin
-    );
-
-    parameter = inf_session_get_user_property(array, "request-log");
-    g_value_init(&parameter->value, INF_ADOPTED_TYPE_REQUEST_LOG);
-    g_value_take_object(&parameter->value, log);
-  }
-#endif
-
   return array;
 }
 
@@ -1397,6 +1354,7 @@ inf_adopted_session_set_xml_user_props(InfSession* session,
                                        guint n_params,
                                        xmlNodePtr xml)
 {
+  InfSessionClass* parent_class;
   InfAdoptedSessionPrivate* priv;
   const GParameter* time;
   InfAdoptedStateVector* vector;
@@ -1407,12 +1365,8 @@ inf_adopted_session_set_xml_user_props(InfSession* session,
   GSList* item;
   InfAdoptedSessionLocalUser* local_user;
 
-  INF_SESSION_CLASS(parent_class)->set_xml_user_props(
-    session,
-    params,
-    n_params,
-    xml
-  );
+  parent_class = INF_SESSION_CLASS(inf_adopted_session_parent_class);
+  parent_class->set_xml_user_props(session, params, n_params, xml);
 
   priv = INF_ADOPTED_SESSION_PRIVATE(INF_ADOPTED_SESSION(session));
 
@@ -1448,19 +1402,6 @@ inf_adopted_session_set_xml_user_props(InfSession* session,
     inf_xml_util_set_attribute(xml, "time", time_string);
     g_free(time_string);
   }
-
-  /* log-begin is not in the spec */
-#if 0
-  log = inf_session_lookup_user_property(params, n_params, "request-log");
-  if(log != NULL)
-  {
-    log_begin = inf_adopted_request_log_get_begin(
-      INF_ADOPTED_REQUEST_LOG(g_value_get_object(&log->value))
-    );
-
-    inf_xml_util_set_attribute_uint(xml, "log-begin", log_begin);
-  }
-#endif
 }
 
 static gboolean
@@ -1470,10 +1411,12 @@ inf_adopted_session_validate_user_props(InfSession* session,
                                         InfUser* exclude,
                                         GError** error)
 {
+  InfSessionClass* parent_class;
   const GParameter* time;
   gboolean result;
 
-  result = INF_SESSION_CLASS(parent_class)->validate_user_props(
+  parent_class = INF_SESSION_CLASS(inf_adopted_session_parent_class);
+  result = parent_class->validate_user_props(
     session,
     params,
     n_params,
@@ -1486,12 +1429,11 @@ inf_adopted_session_validate_user_props(InfSession* session,
   time = inf_session_lookup_user_property(params, n_params, "vector");
   if(time == NULL)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       inf_adopted_session_error_quark,
       INF_ADOPTED_SESSION_ERROR_MISSING_STATE_VECTOR,
-      "%s",
-      _("'time' attribute in user message is missing")
+      _("\"time\" attribute in user message is missing")
     );
 
     return FALSE;
@@ -1509,6 +1451,7 @@ inf_adopted_session_close(InfSession* session)
 
   priv = INF_ADOPTED_SESSION_PRIVATE(session);
 
+  /* Local user info is no longer required */
   for(item = priv->local_users; item != NULL; item = g_slist_next(item))
   {
     local = (InfAdoptedSessionLocalUser*)item->data;
@@ -1519,8 +1462,7 @@ inf_adopted_session_close(InfSession* session)
   g_slist_free(priv->local_users);
   priv->local_users = NULL;
 
-  /* Local user info is no longer required */
-  INF_SESSION_CLASS(parent_class)->close(session);
+  INF_SESSION_CLASS(inf_adopted_session_parent_class)->close(session);
 }
 
 static void
@@ -1549,18 +1491,17 @@ static void
 inf_adopted_session_synchronization_complete(InfSession* session,
                                              InfXmlConnection* connection)
 {
+  InfSessionClass* parent_class;
   InfAdoptedSessionPrivate* priv;
   InfSessionStatus status;
 
   priv = INF_ADOPTED_SESSION_PRIVATE(session);
-  g_object_get(G_OBJECT(session), "status", &status, NULL);
+  status = inf_session_get_status(session);
 
   g_object_freeze_notify(G_OBJECT(session));
 
-  INF_SESSION_CLASS(parent_class)->synchronization_complete(
-    session,
-    connection
-  );
+  parent_class = INF_SESSION_CLASS(inf_adopted_session_parent_class);
+  parent_class->synchronization_complete(session, connection);
 
   if(status == INF_SESSION_SYNCHRONIZING)
   {
@@ -1592,21 +1533,15 @@ inf_adopted_session_check_request(InfAdoptedSession* session,
  */
 
 static void
-inf_adopted_session_class_init(gpointer g_class,
-                               gpointer class_data)
+inf_adopted_session_class_init(InfAdoptedSessionClass* adopted_session_class)
 {
   GObjectClass* object_class;
   InfSessionClass* session_class;
-  InfAdoptedSessionClass* adopted_session_class;
 
-  object_class = G_OBJECT_CLASS(g_class);
-  session_class = INF_SESSION_CLASS(g_class);
-  adopted_session_class = INF_ADOPTED_SESSION_CLASS(g_class);
+  object_class = G_OBJECT_CLASS(adopted_session_class);
+  session_class = INF_SESSION_CLASS(adopted_session_class);
 
-  parent_class = INF_SESSION_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfAdoptedSessionPrivate));
-
-  object_class->constructor = inf_adopted_session_constructor;
+  object_class->constructed = inf_adopted_session_constructed;
   object_class->dispose = inf_adopted_session_dispose;
   object_class->finalize = inf_adopted_session_finalize;
   object_class->set_property = inf_adopted_session_set_property;
@@ -1655,7 +1590,7 @@ inf_adopted_session_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfAdoptedSessionClass, check_request),
     g_signal_accumulator_true_handled, NULL,
-    inf_marshal_BOOLEAN__OBJECT_OBJECT,
+    NULL,
     G_TYPE_BOOLEAN,
     2,
     INF_ADOPTED_TYPE_REQUEST,
@@ -1701,37 +1636,6 @@ inf_adopted_session_class_init(gpointer g_class,
   );
 }
 
-GType
-inf_adopted_session_get_type(void)
-{
-  static GType session_type = 0;
-
-  if(!session_type)
-  {
-    static const GTypeInfo session_type_info = {
-      sizeof(InfAdoptedSessionClass),   /* class_size */
-      NULL,                             /* base_init */
-      NULL,                             /* base_finalize */
-      inf_adopted_session_class_init,   /* class_init */
-      NULL,                             /* class_finalize */
-      NULL,                             /* class_data */
-      sizeof(InfAdoptedSession),        /* instance_size */
-      0,                                /* n_preallocs */
-      inf_adopted_session_init,         /* instance_init */
-      NULL                              /* value_table */
-    };
-
-    session_type = g_type_register_static(
-      INF_TYPE_SESSION,
-      "InfAdoptedSession",
-      &session_type_info,
-      0
-    );
-  }
-
-  return session_type;
-}
-
 /*
  * Public API.
  */
@@ -1742,7 +1646,7 @@ inf_adopted_session_get_type(void)
  *
  * Returns the #InfIo object of @session.
  *
- * Return Value: A #InfIo.
+ * Returns: (transfer none): A #InfIo.
  **/
 InfIo*
 inf_adopted_session_get_io(InfAdoptedSession* session)
@@ -1760,7 +1664,7 @@ inf_adopted_session_get_io(InfAdoptedSession* session)
  * because there the algorithm object is not yet created before successful
  * synchronization.
  *
- * Return Value: A #InfAdoptedAlgorithm, or %NULL.
+ * Returns: (transfer none): A #InfAdoptedAlgorithm, or %NULL.
  **/
 InfAdoptedAlgorithm*
 inf_adopted_session_get_algorithm(InfAdoptedSession* session)
@@ -1775,10 +1679,8 @@ inf_adopted_session_get_algorithm(InfAdoptedSession* session)
  * @request: A #InfAdoptedRequest obtained from @session's algorithm.
  *
  * Sends a request to all subscribed connections. The request should originate
- * from a call to inf_adopted_algorithm_generate_request_noexec(),
- * inf_adopted_algorithm_generate_request(),
- * inf_adopted_algorithm_generate_undo() or
- * inf_adopted_algorithm_generate_redo() with @session's #InfAdoptedAlgorithm.
+ * from a call to inf_adopted_algorithm_generate_request(), with
+ * @session's #InfAdoptedAlgorithm.
  **/
 void
 inf_adopted_session_broadcast_request(InfAdoptedSession* session,
@@ -1910,11 +1812,13 @@ inf_adopted_session_redo(InfAdoptedSession* session,
  * inf_adopted_session_read_request_info:
  * @session: A #InfAdoptedSession.
  * @xml: The XML to read the data from.
- * @diff_vec: The reference vector of the time vector of the request, or
- * %NULL.
- * @user: Location to store the user of the request, or %NULL.
- * @time: Location to store the state the request was made, or %NULL.
- * @operation: Location to store the operation of the request, or %NULL.
+ * @diff_vec: The reference vector of the time vector of the request, or %NULL.
+ * @user: (out) (transfer none) (allow-none): Location to store the user of
+ * the request, or %NULL.
+ * @time: (out) (transfer full) (allow-none): Location to store the state the
+ * request was made, or %NULL.
+ * @operation: (out) (transfer none) (allow-none): Location to store the
+ * operation of the request, or %NULL.
  * @error: Location to place an error, if any.
  *
  * This function reads common information such as the state vector the request
@@ -1973,11 +1877,10 @@ inf_adopted_session_read_request_info(InfAdoptedSession* session,
 
     if(child == NULL)
     {
-      g_set_error(
+      g_set_error_literal(
         error,
         inf_adopted_session_error_quark,
         INF_ADOPTED_SESSION_ERROR_MISSING_OPERATION,
-        "%s",
         _("Operation for request missing")
       );
 
