@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -37,6 +37,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+G_DEFINE_BOXED_TYPE(InfAdoptedStateVector, inf_adopted_state_vector, inf_adopted_state_vector_copy, inf_adopted_state_vector_free)
+
 /* NOTE: What the state vector actually counts is the amount of operations
  * performed by each user. This number is called a timestamp, although it has
  * nothing to do with actual time. */
@@ -49,13 +51,11 @@ struct _InfAdoptedStateVectorComponent {
 
 typedef struct _InfAdoptedStateVectorForeachData
   InfAdoptedStateVectorForeachData;
-
 struct _InfAdoptedStateVectorForeachData {
   InfAdoptedStateVectorForeachFunc func;
   gpointer user_data;
 };
 
-
 struct _InfAdoptedStateVector {
   gsize size;
   gsize max_size;
@@ -144,23 +144,6 @@ inf_adopted_state_vector_insert(InfAdoptedStateVector* vec,
   return comp;
 }
 
-GType
-inf_adopted_state_vector_get_type(void)
-{
-  static GType state_vector_type = 0;
-
-  if(!state_vector_type)
-  {
-    state_vector_type = g_boxed_type_register_static(
-      "InfAdoptedStateVector",
-      (GBoxedCopyFunc)inf_adopted_state_vector_copy,
-      (GBoxedFreeFunc)inf_adopted_state_vector_free
-    );
-  }
-
-  return state_vector_type;
-}
-
 /**
  * inf_adopted_state_vector_error_quark:
  *
@@ -175,11 +158,11 @@ inf_adopted_state_vector_error_quark(void)
 }
 
 /**
- * inf_adopted_state_vector_new:
+ * inf_adopted_state_vector_new: (constructor)
  *
  * Returns a new state vector with all components set to zero.
  *
- * Return Value: A new #InfAdoptedStateVector.
+ * Returns: (transfer full): A new #InfAdoptedStateVector.
  **/
 InfAdoptedStateVector*
 inf_adopted_state_vector_new(void)
@@ -200,7 +183,7 @@ inf_adopted_state_vector_new(void)
  *
  * Returns a copy of @vec.
  *
- * Return Value: A copy of @vec.
+ * Returns: (transfer full): A copy of @vec.
  **/
 InfAdoptedStateVector*
 inf_adopted_state_vector_copy(InfAdoptedStateVector* vec)
@@ -252,7 +235,7 @@ inf_adopted_state_vector_free(InfAdoptedStateVector* vec)
  * Returns the timestamp for the given component. Implicitely, all IDs
  * that the vector does not contain are assigned the timestamp 0.
  *
- * Return Value: The @component'th entry in the vector.
+ * Returns: The @component'th entry in the vector.
  */
 guint
 inf_adopted_state_vector_get(const InfAdoptedStateVector* vec,
@@ -332,7 +315,7 @@ inf_adopted_state_vector_add(InfAdoptedStateVector* vec,
 /**
  * inf_adopted_state_vector_foreach:
  * @vec: A #InfAdoptedStateVector.
- * @func: The function to call.
+ * @func: (scope call): The function to call.
  * @user_data: Additional data to pass to @func.
  *
  * Calls @func for each component in @vec. Note that there may be users for
@@ -363,7 +346,7 @@ inf_adopted_state_vector_foreach(const InfAdoptedStateVector* vec,
  * can be sorted. This function returns -1 if @first compares before @second,
  * 0 if they compare equal and 1 if @first compares after @second.
  *
- * Return Value: -1, 0 or 1.
+ * Returns: -1, 0 or 1.
  **/
 int
 inf_adopted_state_vector_compare(const InfAdoptedStateVector* first,
@@ -458,7 +441,7 @@ inf_adopted_state_vector_compare(const InfAdoptedStateVector* first,
  * components of @first are less or equal to the corresponding component in
  * @second.
  *
- * Return Value: Whether @second depends on @first.
+ * Returns: Whether @second depends on @first.
  **/
 gboolean
 inf_adopted_state_vector_causally_before(const InfAdoptedStateVector* first,
@@ -533,7 +516,7 @@ inf_adopted_state_vector_causally_before(const InfAdoptedStateVector* first,
  *
  * But it is more efficient.
  *
- * Return Value: Whether @second depends on @first with the
+ * Returns: Whether @second depends on @first with the
  * @inc_component<!-- -->th component increased by one.
  **/
 gboolean
@@ -679,7 +662,8 @@ inf_adopted_state_vector_vdiff(const InfAdoptedStateVector* first,
  *
  * Returns a string representation of @vec.
  *
- * Return Value: A newly-allocated string to be freed by the caller.
+ * Returns: (transfer full): A newly-allocated string to be freed by the
+ * caller.
  **/
 gchar*
 inf_adopted_state_vector_to_string(const InfAdoptedStateVector* vec)
@@ -716,7 +700,7 @@ inf_adopted_state_vector_to_string(const InfAdoptedStateVector* vec)
  * Recreates the #InfAdoptedStateVector from its string representation. If
  * an error occurs, the function returns %NULL and @error is set.
  *
- * Return Value: A new #InfAdoptedStateVector, or %NULL.
+ * Returns: (transfer full): A new #InfAdoptedStateVector, or %NULL.
  **/
 InfAdoptedStateVector*
 inf_adopted_state_vector_from_string(const gchar* str,
@@ -739,12 +723,11 @@ inf_adopted_state_vector_from_string(const gchar* str,
     id = strtoul(strpos, &endpos, 10);
     if(*endpos != ':')
     {
-      g_set_error(
+      g_set_error_literal(
         error,
         inf_adopted_state_vector_error_quark(),
         INF_ADOPTED_STATE_VECTOR_BAD_FORMAT,
-        "%s",
-        _("Expected ':' after ID")
+        _("Expected \":\" after ID")
       );
 
       inf_adopted_state_vector_free(vec);
@@ -802,7 +785,8 @@ inf_adopted_state_vector_from_string(const gchar* str,
  * to recreate @vec from the string representation. Additionally,
  * inf_adopted_state_vector_causally_before(@orig, @vec) must hold.
  *
- * Return Value: A newly allocated string to be freed by the caller.
+ * Returns: (transfer full): A newly allocated string to be freed by the
+ * caller.
  **/
 gchar*
 inf_adopted_state_vector_to_string_diff(const InfAdoptedStateVector* vec,
@@ -893,8 +877,8 @@ inf_adopted_state_vector_to_string_diff(const InfAdoptedStateVector* vec,
  * Recreates a vector from its string representation diff and the original
  * vector. If an error returns, the function returns %NULL and @error is set.
  *
- * Return Value: The created state vector, or %NULL on error. Free with
- * inf_adopted_state_vector_free() when no longer needed.
+ * Returns: (transfer full): The created state vector, or %NULL on error.
+ * Free with inf_adopted_state_vector_free() when no longer needed.
  **/
 InfAdoptedStateVector*
 inf_adopted_state_vector_from_string_diff(const gchar* str,
