@@ -1,5 +1,5 @@
 /* infcinote - Collaborative notetaking application
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,10 +17,30 @@
  * MA 02110-1301, USA.
  */
 
+/**
+ * SECTION:infc-request-manager
+ * @short_description: Keeps track of pending asynchronous requests
+ * @see_also: #InfRequest, #InfcRequest, #InfcBrowser, #InfcSessionProxy
+ * @include: libinfinity/client/infc-request-manager.h
+ * @stability: Unstable
+ *
+ * This class keeps track of requests sent by #InfcBrowser and
+ * #InfcSessionProxy (typically via the #InfBrowser and #InfSessionProxy
+ * APIs). It is used internally by #InfcBrowser and #InfcSessionProxy, and
+ * usually end-users of the libinfinity API do not need to use this class
+ * directly.
+ *
+ * Requests can be added with the infc_request_manager_add_request() and
+ * infc_request_manager_add_request_valist() methods and retrieved back via
+ * their unique seq number with infc_request_manager_get_request_by_seq(). In
+ * addition to this basic API, there are various convenience functions
+ * available as well.
+ **/
+
 #include <libinfinity/client/infc-request-manager.h>
+#include <libinfinity/common/inf-request.h>
 #include <libinfinity/common/inf-xml-util.h>
 #include <libinfinity/common/inf-error.h>
-#include <libinfinity/inf-marshal.h>
 #include <libinfinity/inf-i18n.h>
 
 #include <gobject/gvaluecollector.h>
@@ -57,9 +77,11 @@ enum {
 
 #define INFC_REQUEST_MANAGER_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INFC_TYPE_REQUEST_MANAGER, InfcRequestManagerPrivate))
 
-static GObjectClass* parent_class;
 static guint request_manager_signals[LAST_SIGNAL];
 
+G_DEFINE_TYPE_WITH_CODE(InfcRequestManager, infc_request_manager, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfcRequestManager))
+
 static void
 infc_request_manager_foreach_request_func(gpointer key,
                                           gpointer value,
@@ -67,15 +89,18 @@ infc_request_manager_foreach_request_func(gpointer key,
 {
   InfcRequest* request;
   InfcRequestManagerForeachData* foreach_data;
+  gchar* type;
 
   request = INFC_REQUEST(value);
   foreach_data = (InfcRequestManagerForeachData*)user_data;
+  g_object_get(G_OBJECT(request), "type", &type, NULL);
 
-  if(foreach_data->name == NULL ||
-     strcmp(foreach_data->name, infc_request_get_name(request)) == 0)
+  if(foreach_data->name == NULL || strcmp(foreach_data->name, type) == 0)
   {
     foreach_data->func(request, foreach_data->user_data);
   }
+
+  g_free(type);
 }
 
 /* TODO: inf_protocol_version_parse() uses a very similar routine. We should
@@ -94,11 +119,10 @@ infc_request_manager_parse_seq(const gchar* seq,
   sid = strtoul(seq, &endptr, 10);
   if(errno == ERANGE || sid > (unsigned long)G_MAXUINT)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       inf_request_error_quark(),
       INF_REQUEST_ERROR_INVALID_SEQ,
-      "%s",
       _("Sequence identifier causes overflow")
     );
 
@@ -107,11 +131,10 @@ infc_request_manager_parse_seq(const gchar* seq,
 
   if(*endptr != '/')
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       inf_request_error_quark(),
       INF_REQUEST_ERROR_INVALID_SEQ,
-      "%s",
       _("Sequence components are not separated by '/'")
     );
 
@@ -122,11 +145,10 @@ infc_request_manager_parse_seq(const gchar* seq,
   snum = strtoul(endptr + 1, &endptr, 10);
   if(errno == ERANGE || snum > (unsigned long)G_MAXUINT)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       inf_request_error_quark(),
       INF_REQUEST_ERROR_INVALID_SEQ,
-      "%s",
       _("Sequence number causes overflow")
     );
 
@@ -135,11 +157,10 @@ infc_request_manager_parse_seq(const gchar* seq,
 
   if(*endptr != '\0')
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       inf_request_error_quark(),
       INF_REQUEST_ERROR_INVALID_SEQ,
-      "%s",
       _("Trailing characters after sequence number")
     );
 
@@ -152,14 +173,10 @@ infc_request_manager_parse_seq(const gchar* seq,
 }
 
 static void
-infc_request_manager_init(GTypeInstance* instance,
-                          gpointer g_class)
+infc_request_manager_init(InfcRequestManager* manager)
 {
-  InfcRequestManager* request_manager;
   InfcRequestManagerPrivate* priv;
-
-  request_manager = INFC_REQUEST_MANAGER(instance);
-  priv = INFC_REQUEST_MANAGER_PRIVATE(request_manager);
+  priv = INFC_REQUEST_MANAGER_PRIVATE(manager);
 
   priv->requests = g_hash_table_new_full(
     NULL,
@@ -184,8 +201,7 @@ infc_request_manager_dispose(GObject* object)
   g_hash_table_destroy(priv->requests);
   priv->requests = NULL;
 
-  if(parent_class->dispose != NULL)
-    parent_class->dispose(object);
+  G_OBJECT_CLASS(infc_request_manager_parent_class)->dispose(object);
 }
 
 static void
@@ -239,13 +255,15 @@ infc_request_manager_request_add(InfcRequestManager* manager,
                                  InfcRequest* request)
 {
   InfcRequestManagerPrivate* priv;
-  gpointer seq;
+  guint seq;
+  gpointer seq_ptr;
 
   priv = INFC_REQUEST_MANAGER_PRIVATE(manager);
-  seq = GUINT_TO_POINTER(infc_request_get_seq(request));
+  g_object_get(G_OBJECT(request), "seq", &seq, NULL);
+  seq_ptr = GUINT_TO_POINTER(seq);
 
-  g_assert(g_hash_table_lookup(priv->requests, seq) == NULL);
-  g_hash_table_insert(priv->requests, seq, request);
+  g_assert(g_hash_table_lookup(priv->requests, seq_ptr) == NULL);
+  g_hash_table_insert(priv->requests, seq_ptr, request);
 
   g_object_ref(G_OBJECT(request));
 }
@@ -255,27 +273,23 @@ infc_request_manager_request_remove(InfcRequestManager* manager,
                                     InfcRequest* request)
 {
   InfcRequestManagerPrivate* priv;
-  gpointer seq;
+  guint seq;
+  gpointer seq_ptr;
 
   priv = INFC_REQUEST_MANAGER_PRIVATE(manager);
-  seq = GUINT_TO_POINTER(infc_request_get_seq(request));
+  g_object_get(G_OBJECT(request), "seq", &seq, NULL);
+  seq_ptr = GUINT_TO_POINTER(seq);
 
-  g_assert(g_hash_table_lookup(priv->requests, seq) != NULL);
-  g_hash_table_remove(priv->requests, seq);
+  g_assert(g_hash_table_lookup(priv->requests, seq_ptr) != NULL);
+  g_hash_table_remove(priv->requests, seq_ptr);
 }
 
 static void
-infc_request_manager_class_init(gpointer g_class,
-                                gpointer class_data)
+infc_request_manager_class_init(
+  InfcRequestManagerClass* request_manager_class)
 {
   GObjectClass* object_class;
-  InfcRequestManagerClass* request_manager_class;
-
-  object_class = G_OBJECT_CLASS(g_class);
-  request_manager_class = INFC_REQUEST_MANAGER_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfcRequestManagerPrivate));
+  object_class = G_OBJECT_CLASS(request_manager_class);
 
   object_class->dispose = infc_request_manager_dispose;
   object_class->set_property = infc_request_manager_set_property;
@@ -298,69 +312,56 @@ infc_request_manager_class_init(gpointer g_class,
     )
   );
 
+  /**
+   * InfcRequestManager::request-add:
+   * @manager: The #InfcRequestManager emitting the signal.
+   * @request: The #InfcRequest that was added.
+   *
+   * This signal is emitted whenever a request was added to the request
+   * manager.
+   */
   request_manager_signals[REQUEST_ADD] = g_signal_new(
     "request-add",
     G_OBJECT_CLASS_TYPE(object_class),
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfcRequestManagerClass, request_add),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT,
+    g_cclosure_marshal_VOID__OBJECT,
     G_TYPE_NONE,
     1,
     INFC_TYPE_REQUEST
   );
 
+  /**
+   * InfcRequestManager::request-remove:
+   * @manager: The #InfcRequestManager emitting the signal.
+   * @request: The #InfcRequest that was removed.
+   *
+   * This signal is emitted whenever a request was removed from the request
+   * manager. Typically this happens when the request has finished (either
+   * successfully or unsuccessfully). It can also happen when the client is
+   * disconnected from the server while a request was still pending.
+   */
   request_manager_signals[REQUEST_REMOVE] = g_signal_new(
     "request-remove",
     G_OBJECT_CLASS_TYPE(object_class),
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfcRequestManagerClass, request_remove),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT,
+    g_cclosure_marshal_VOID__OBJECT,
     G_TYPE_NONE,
     1,
     INFC_TYPE_REQUEST
   );
 }
 
-GType
-infc_request_manager_get_type(void)
-{
-  static GType request_manager_type = 0;
-
-  if(!request_manager_type)
-  {
-    static const GTypeInfo request_manager_type_info = {
-      sizeof(InfcRequestManagerClass),  /* class_size */
-      NULL,                             /* base_init */
-      NULL,                             /* base_finalize */
-      infc_request_manager_class_init,  /* class_init */
-      NULL,                             /* class_finalize */
-      NULL,                             /* class_data */
-      sizeof(InfcRequestManager),       /* instance_size */
-      0,                                /* n_preallocs */
-      infc_request_manager_init,        /* instance_init */
-      NULL                              /* value_table */
-    };
-
-    request_manager_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfcRequestManager",
-      &request_manager_type_info,
-      0
-    );
-  }
-
-  return request_manager_type;
-}
-
 /**
- * infc_request_manager_new:
+ * infc_request_manager_new: (constructor)
  * @seq_id: The local sequence ID.
  *
  * Creates a new #InfcRequestManager.
  *
- * Return Value: A newly allocated #InfcRequestManager.
+ * Returns: (transfer full): A newly allocated #InfcRequestManager.
  **/
 InfcRequestManager*
 infc_request_manager_new(guint seq_id)
@@ -374,22 +375,30 @@ infc_request_manager_new(guint seq_id)
 /**
  * infc_request_manager_add_request:
  * @manager: A #InfcRequestManager.
- * @request_type: The type of request to add, such as %INFC_TYPE_NODE_REQUEST.
+ * @request_type: The type of request to add, such as #InfcProgressRequest.
  * @request_name: The name of the request, such as "explore-node" or
  * "subscribe-session".
+ * @callback: (scope async): A #GCallback that will be called when the request
+ * has completed, or %NULL.
+ * @user_data: Additional data passed to the callback.
  * @first_property_name: The first property name apart from name and seq to
  * set for the new request.
  * @...: The value of the first property, followed optionally by more
  * name/value pairs, followed by %NULL.
  *
- * Adds a request to the request manager.
+ * Adds a request to the request manager. A new signal handler for the
+ * request's "finished" signal will be installed and made to invoke the given
+ * callback.
  *
- * Return Value: The generated #InfcRequest (actually of type @request_type).
+ * Returns: (transfer none): The generated #InfcRequest (actually of type
+ * @request_type).
  **/
 InfcRequest*
 infc_request_manager_add_request(InfcRequestManager* manager,
                                  GType request_type,
                                  const gchar* request_name,
+                                 GCallback callback,
+                                 gpointer user_data,
                                  const gchar* first_property_name,
                                  ...)
 {
@@ -402,6 +411,8 @@ infc_request_manager_add_request(InfcRequestManager* manager,
     manager,
     request_type,
     request_name,
+    callback,
+    user_data,
     first_property_name,
     arglist
   );
@@ -413,22 +424,30 @@ infc_request_manager_add_request(InfcRequestManager* manager,
 /**
  * infc_request_manager_add_request_valist:
  * @manager: A #InfcRequestManager.
- * @request_type: The type of request to add, such as %INFC_TYPE_NODE_REQUEST.
+ * @request_type: The type of request to add, such as #InfcProgressRequest.
  * @request_name: The name of the request, such as &quot;explore-node&quot; or
  * &quot;subscribe-session&quot;
+ * @callback: (scope async): A #GCallback that will be called when the request
+ * has completed, or %NULL.
+ * @user_data: Additional data passed to the callback.
  * @first_property_name: The first property name apart from name and seq to
  * set for the new request.
  * @arglist: The value of the first property, followed optionally by more
  * name/value pairs, followed by %NULL.
  *
- * Adds a request to the request manager.
+ * Adds a request to the request manager. A new signal handler for the
+ * request's "finished" signal will be installed and made to invoke the given
+ * callback.
  *
- * Return Value: The generated #InfcRequest (actually of type @request_type).
+ * Returns: (transfer none): The generated #InfcRequest (actually of type
+ * @request_type).
  */
 InfcRequest*
 infc_request_manager_add_request_valist(InfcRequestManager* manager,
                                         GType request_type,
                                         const gchar* request_name,
+                                        GCallback callback,
+                                        gpointer user_data,
                                         const gchar* first_property_name,
                                         va_list arglist)
 {
@@ -461,7 +480,7 @@ infc_request_manager_add_request_valist(InfcRequestManager* manager,
   param_alloc = 16;
   params = g_malloc0(param_alloc * sizeof(GParameter));
 
-  params[param_size].name = "name";
+  params[param_size].name = "type";
   g_value_init(&params[param_size].value, G_TYPE_STRING);
   g_value_set_static_string(&params[param_size].value, request_name);
   ++ param_size;
@@ -529,6 +548,16 @@ infc_request_manager_add_request_valist(InfcRequestManager* manager,
   g_free(params);
   g_type_class_unref(request_class);
 
+  if(callback != NULL)
+  {
+    g_signal_connect_after(
+      G_OBJECT(request),
+      "finished",
+      callback,
+      user_data
+    );
+  }
+
   return request;
 }
 
@@ -558,26 +587,52 @@ infc_request_manager_remove_request(InfcRequestManager* manager,
   g_object_unref(G_OBJECT(request));
 }
 
+/**
+ * infc_request_manager_finish_request:
+ * @manager: A #InfcRequestManager.
+ * @request: A #InfcRequest that has previously been added to @manager.
+ * @result: (transfer full): The request result.
+ *
+ * Marks @request as finished using inf_request_finish() and then removes the
+ * request from the manager. The function takes ownership of @result.
+ **/
+void
+infc_request_manager_finish_request(InfcRequestManager* manager,
+                                    InfcRequest* request,
+                                    InfRequestResult* result)
+{
+  g_return_if_fail(INFC_IS_REQUEST_MANAGER(manager));
+  g_return_if_fail(INFC_IS_REQUEST(request));
+  g_return_if_fail(result != NULL);
+
+  g_object_ref(request);
+  infc_request_manager_remove_request(manager, request);
+  inf_request_finish(INF_REQUEST(request), result);
+  g_object_unref(request);
+}
+
 /**
  * infc_request_manager_fail_request:
  * @manager: A #InfcRequestManager.
  * @request: A #InfcRequest that has previously been added to @manager.
  * @error: Error information why the request failed.
  *
- * Emits the "failed" signal on @request and then removes the request from
- * the manager.
+ * Marks @request as failed using inf_request_fail() and then removes the
+ * request from the manager.
  **/
 void
 infc_request_manager_fail_request(InfcRequestManager* manager,
                                   InfcRequest* request,
-                                  GError* error)
+                                  const GError* error)
 {
   g_return_if_fail(INFC_IS_REQUEST_MANAGER(manager));
   g_return_if_fail(INFC_IS_REQUEST(request));
   g_return_if_fail(error != NULL);
 
-  infc_request_failed(request, error);
+  g_object_ref(request);
   infc_request_manager_remove_request(manager, request);
+  inf_request_fail(INF_REQUEST(request), error);
+  g_object_unref(request);
 }
 
 /**
@@ -600,8 +655,8 @@ infc_request_manager_clear(InfcRequestManager* manager)
  *
  * Returns a previously-added request that has the given seq.
  *
- * Return Value: The request with the given seq, or %NULL if there is no such
- * request.
+ * Returns: (transfer none) (allow-none): The request with the given seq,
+ * or %NULL if there is no such request.
  **/
 InfcRequest*
 infc_request_manager_get_request_by_seq(InfcRequestManager* manager,
@@ -631,8 +686,8 @@ infc_request_manager_get_request_by_seq(InfcRequestManager* manager,
  * attribute is set but the actual request is not present (or has another
  * name), the function returns %NULL and @error is set.
  *
- * Return Value: The resulting request, or %NULL if the "seq" attribute was
- * not present or an error occured.
+ * Returns: (transfer none) (allow-none): The resulting request, or %NULL
+ * if the "seq" attribute was not present or an error occured.
  **/
 InfcRequest*
 infc_request_manager_get_request_by_xml(InfcRequestManager* manager,
@@ -646,6 +701,7 @@ infc_request_manager_get_request_by_xml(InfcRequestManager* manager,
   gboolean has_seq;
   guint seq_id;
   guint seq;
+  gchar* request_type;
 
   g_return_val_if_fail(INFC_IS_REQUEST_MANAGER(manager), NULL);
   g_return_val_if_fail(xml != NULL, NULL);
@@ -663,6 +719,7 @@ infc_request_manager_get_request_by_xml(InfcRequestManager* manager,
     error
   );
 
+  xmlFree(seq_attr);
   if(!has_seq) return NULL;
 
   /* Not our seq ID */
@@ -675,14 +732,14 @@ infc_request_manager_get_request_by_xml(InfcRequestManager* manager,
       error,
       inf_request_error_quark(),
       INF_REQUEST_ERROR_INVALID_SEQ,
-      "%s",
-      _("The request contains an unknown sequence number")
+      _("No such sequence number %u"),
+      seq
     );
   }
   else
   {
-    if(name != NULL &&
-       strcmp(name, infc_request_get_name(request)) != 0)
+    g_object_get(G_OBJECT(request), "type", &request_type, NULL);
+    if(name != NULL && strcmp(name, request_type) != 0)
     {
       g_set_error(
         error,
@@ -690,12 +747,14 @@ infc_request_manager_get_request_by_xml(InfcRequestManager* manager,
         INF_REQUEST_ERROR_INVALID_SEQ,
         _("The request contains a sequence number refering to a request of "
           "type '%s', but a request of type '%s' was expected"),
-        infc_request_get_name(request),
+        request_type,
         name
       );
 
       request = NULL;
     }
+
+    g_free(request_type);
   }
 
   return request;
@@ -713,7 +772,8 @@ infc_request_manager_get_request_by_xml(InfcRequestManager* manager,
  * "seq" attribute is not set or the actual request is not present (or has
  * another name), the function returns %NULL and @error is set.
  *
- * Return Value: The resulting request, or %NULL if an error occured.
+ * Returns: (transfer none): The resulting request, or %NULL if an error
+ * occured.
  **/
 InfcRequest*
 infc_request_manager_get_request_by_xml_required(InfcRequestManager* manager,
@@ -736,11 +796,10 @@ infc_request_manager_get_request_by_xml_required(InfcRequestManager* manager,
   {
     if(own_error == NULL)
     {
-      g_set_error(
+      g_set_error_literal(
         error,
         inf_request_error_quark(),
         INF_REQUEST_ERROR_INVALID_SEQ,
-        "%s",
         _("The request does not contain a sequence number, but one "
           "is required")
       );
@@ -757,7 +816,7 @@ infc_request_manager_get_request_by_xml_required(InfcRequestManager* manager,
 /**
  * infc_request_manager_foreach_request:
  * @manager: A #InfcRequestManager.
- * @func: The function to be called.
+ * @func: (scope call): The function to be called.
  * @user_data: Arbitrary data to be passed to @func.
  *
  * Calls the given function for each request that has been added to the
@@ -791,7 +850,7 @@ infc_request_manager_foreach_request(InfcRequestManager* manager,
  * infc_request_manager_foreach_named_request:
  * @manager: A #InfcRequestManager.
  * @name: The name of the request to look for.
- * @func: The function to be called.
+ * @func: (scope call): The function to be called.
  * @user_data: Arbitrary data to be passed to @func.
  *
  * Calls the given function for each request that has been added to the
