@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -34,7 +34,6 @@
 #include <libinfinity/common/inf-error.h>
 
 #include <libinfinity/inf-i18n.h>
-#include <libinfinity/inf-marshal.h>
 #include <libinfinity/inf-signals.h>
 
 #include <errno.h>
@@ -69,10 +68,12 @@ enum {
 
 #define INF_CHAT_SESSION_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_TYPE_CHAT_SESSION, InfChatSessionPrivate))
 
-static InfSessionClass* parent_class;
 static guint chat_session_signals[LAST_SIGNAL];
 static GQuark inf_chat_session_error_quark;
 
+G_DEFINE_TYPE_WITH_CODE(InfChatSession, inf_chat_session, INF_TYPE_SESSION,
+  G_ADD_PRIVATE(InfChatSession))
+
 /*
  * Error functions
  */
@@ -460,11 +461,10 @@ inf_chat_session_receive_message(InfChatSession* session,
      (inf_user_get_status(message.user) == INF_USER_UNAVAILABLE ||
       inf_user_get_connection(message.user) != connection))
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       inf_user_error_quark(),
       INF_USER_ERROR_NOT_JOINED,
-      "%s",
       _("User did not join from this connection")
     );
 
@@ -599,13 +599,9 @@ inf_chat_session_add_message_cb(InfChatBuffer* buffer,
  */
 
 static void
-inf_chat_session_init(GTypeInstance* instance,
-                      gpointer g_class)
+inf_chat_session_init(InfChatSession* session)
 {
-  InfChatSession* session;
   InfChatSessionPrivate* priv;
-
-  session = INF_CHAT_SESSION(instance);
   priv = INF_CHAT_SESSION_PRIVATE(session);
 
   priv->log_filename = NULL;
@@ -613,7 +609,7 @@ inf_chat_session_init(GTypeInstance* instance,
 }
 
 static void
-inf_chat_session_constructor_foreach_user_func(InfUser* user,
+inf_chat_session_constructed_foreach_user_func(InfUser* user,
                                                gpointer user_data)
 {
   g_signal_connect(
@@ -624,21 +620,13 @@ inf_chat_session_constructor_foreach_user_func(InfUser* user,
   );
 }
 
-static GObject*
-inf_chat_session_constructor(GType type,
-                             guint n_construct_properties,
-                             GObjectConstructParam* construct_properties)
+static void
+inf_chat_session_constructed(GObject* object)
 {
-  GObject* object;
   InfUserTable* user_table;
   InfChatBuffer* buffer;
 
-  object = G_OBJECT_CLASS(parent_class)->constructor(
-    type,
-    n_construct_properties,
-    construct_properties
-  );
-
+  G_OBJECT_CLASS(inf_chat_session_parent_class)->constructed(object);
   user_table = inf_session_get_user_table(INF_SESSION(object));
 
   g_signal_connect_after(
@@ -657,7 +645,7 @@ inf_chat_session_constructor(GType type,
 
   inf_user_table_foreach_user(
     INF_USER_TABLE(user_table),
-    inf_chat_session_constructor_foreach_user_func,
+    inf_chat_session_constructed_foreach_user_func,
     object
   );
 
@@ -670,8 +658,6 @@ inf_chat_session_constructor(GType type,
     G_CALLBACK(inf_chat_session_add_message_cb),
     object
   );
-
-  return object;
 }
 
 static void
@@ -720,7 +706,7 @@ inf_chat_session_dispose(GObject* object)
     session
   );
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_chat_session_parent_class)->dispose(object);
 }
 
 static void
@@ -734,7 +720,7 @@ inf_chat_session_finalize(GObject* object)
 
   inf_chat_session_set_log_file(session, NULL, NULL);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(inf_chat_session_parent_class)->finalize(object);
 }
 
 static void
@@ -802,11 +788,13 @@ inf_chat_session_to_xml_sync(InfSession* session,
                              xmlNodePtr parent)
 {
   InfChatBuffer* buffer;
+  InfSessionClass* parent_class;
   const InfChatBufferMessage* message;
   xmlNodePtr child;
   guint i;
 
   buffer = INF_CHAT_BUFFER(inf_session_get_buffer(session));
+  parent_class = INF_SESSION_CLASS(inf_chat_session_parent_class);
 
   g_assert(parent_class->to_xml_sync != NULL);
   parent_class->to_xml_sync(session, parent);
@@ -831,6 +819,8 @@ inf_chat_session_process_xml_sync(InfSession* session,
                                   xmlNodePtr xml,
                                   GError** error)
 {
+  InfSessionClass* parent_class;
+
   if(strcmp((const char*)xml->name, "message") == 0)
   {
     return inf_chat_session_receive_message(
@@ -842,6 +832,7 @@ inf_chat_session_process_xml_sync(InfSession* session,
   }
   else
   {
+    parent_class = INF_SESSION_CLASS(inf_chat_session_parent_class);
     g_assert(parent_class->process_xml_sync != NULL);
     return parent_class->process_xml_sync(session, connection, xml, error);
   }
@@ -853,6 +844,7 @@ inf_chat_session_process_xml_run(InfSession* session,
                                  xmlNodePtr xml,
                                  GError** error)
 {
+  InfSessionClass* parent_class;
   gboolean result;
 
   if(strcmp((const char*)xml->name, "message") == 0)
@@ -871,6 +863,7 @@ inf_chat_session_process_xml_run(InfSession* session,
   }
   else
   {
+    parent_class = INF_SESSION_CLASS(inf_chat_session_parent_class);
     g_assert(parent_class->process_xml_run != NULL);
     return parent_class->process_xml_run(session, connection, xml, error);
   }
@@ -880,9 +873,12 @@ static void
 inf_chat_session_synchronization_complete(InfSession* session,
                                           InfXmlConnection* connection)
 {
+  InfSessionClass* parent_class;
+
   if(inf_session_get_status(session) == INF_SESSION_SYNCHRONIZING)
     inf_chat_session_log_userlist(INF_CHAT_SESSION(session));
 
+  parent_class = INF_SESSION_CLASS(inf_chat_session_parent_class);
   g_assert(parent_class->synchronization_complete != NULL);
   parent_class->synchronization_complete(session, connection);
 }
@@ -892,6 +888,7 @@ inf_chat_session_synchronization_failed(InfSession* session,
                                         InfXmlConnection* connection,
                                         const GError* error)
 {
+  InfSessionClass* parent_class;
   InfChatSessionPrivate* priv;
   time_t cur_time;
   struct tm* tm;
@@ -917,6 +914,7 @@ inf_chat_session_synchronization_failed(InfSession* session,
     }
   }
 
+  parent_class = INF_SESSION_CLASS(inf_chat_session_parent_class);
   g_assert(parent_class->synchronization_failed != NULL);
   parent_class->synchronization_failed(session, connection, error);
 }
@@ -1018,21 +1016,15 @@ inf_chat_session_send_message_handler(InfChatSession* session,
  */
 
 static void
-inf_chat_session_class_init(gpointer g_class,
-                            gpointer class_data)
+inf_chat_session_class_init(InfChatSessionClass* chat_session_class)
 {
   GObjectClass* object_class;
   InfSessionClass* session_class;
-  InfChatSessionClass* chat_session_class;
 
-  object_class = G_OBJECT_CLASS(g_class);
-  session_class = INF_SESSION_CLASS(g_class);
-  chat_session_class = INF_CHAT_SESSION_CLASS(g_class);
+  object_class = G_OBJECT_CLASS(chat_session_class);
+  session_class = INF_SESSION_CLASS(chat_session_class);
 
-  parent_class = INF_SESSION_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfChatSessionPrivate));
-
-  object_class->constructor = inf_chat_session_constructor;
+  object_class->constructed = inf_chat_session_constructed;
   object_class->dispose = inf_chat_session_dispose;
   object_class->finalize = inf_chat_session_finalize;
   object_class->set_property = inf_chat_session_set_property;
@@ -1083,7 +1075,7 @@ inf_chat_session_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfChatSessionClass, receive_message),
     NULL, NULL,
-    inf_marshal_VOID__BOXED,
+    g_cclosure_marshal_VOID__BOXED,
     G_TYPE_NONE,
     1,
     INF_TYPE_CHAT_BUFFER_MESSAGE | G_SIGNAL_TYPE_STATIC_SCOPE
@@ -1107,50 +1099,19 @@ inf_chat_session_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfChatSessionClass, send_message),
     NULL, NULL,
-    inf_marshal_VOID__BOXED,
+    g_cclosure_marshal_VOID__BOXED,
     G_TYPE_NONE,
     1,
     INF_TYPE_CHAT_BUFFER_MESSAGE | G_SIGNAL_TYPE_STATIC_SCOPE
   );
 }
 
-GType
-inf_chat_session_get_type(void)
-{
-  static GType chat_session_type = 0;
-
-  if(!chat_session_type)
-  {
-    static const GTypeInfo chat_session_type_info = {
-      sizeof(InfChatSessionClass),  /* class_size */
-      NULL,                         /* base_init */
-      NULL,                         /* base_finalize */
-      inf_chat_session_class_init,  /* class_init */
-      NULL,                         /* class_finalize */
-      NULL,                         /* class_data */
-      sizeof(InfChatSession),       /* instance_size */
-      0,                            /* n_preallocs */
-      inf_chat_session_init,        /* instance_init */
-      NULL                          /* value_table */
-    };
-
-    chat_session_type = g_type_register_static(
-      INF_TYPE_SESSION,
-      "InfChatSession",
-      &chat_session_type_info,
-      0
-    );
-  }
-
-  return chat_session_type;
-}
-
 /*
  * Public API
  */
 
 /**
- * inf_chat_session_new:
+ * inf_chat_session_new: (constructor)
  * @manager: A #InfCommunicationManager.
  * @buffer: The #InfChatBuffer to use for the session.
  * @status: Initial status of the session. If this is
@@ -1171,7 +1132,7 @@ inf_chat_session_get_type(void)
  * you are subscribed to the session, set the subscription group via
  * inf_session_set_subscription_group().
  *
- * Returns: A new #InfChatSession.
+ * Returns: (transfer full): A new #InfChatSession.
  */
 InfChatSession*
 inf_chat_session_new(InfCommunicationManager* manager,
@@ -1212,7 +1173,7 @@ inf_chat_session_new(InfCommunicationManager* manager,
 /**
  * inf_chat_session_set_log_file:
  * @session: A #InfChatSession.
- * @log_file: A filename to store all received messages into.
+ * @log_file: (type filename): A filename to store all received messages into.
  * @error: Location to store error information, if any.
  *
  * Sets a file onto which all received messages are appended. The file is
@@ -1263,11 +1224,10 @@ inf_chat_session_set_log_file(InfChatSession* session,
 
     if(new_file == NULL)
     {
-      g_set_error(
+      g_set_error_literal(
         error,
         G_FILE_ERROR,
         g_file_error_from_errno(save_errno),
-        "%s",
         strerror(save_errno)
       );
 
