@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -182,6 +182,8 @@ inf_directory_strerror(InfDirectoryError code)
     return _("The node type is not supported by the server");
   case INF_DIRECTORY_ERROR_ROOT_NODE_REMOVE_ATTEMPT:
     return _("The root node cannot be removed");
+  case INF_DIRECTORY_ERROR_NOT_EXPLORED:
+    return _("Subdirectory is not explored");
   case INF_DIRECTORY_ERROR_ALREADY_EXPLORED:
     return _("Subdirectory has already been explored");
   case INF_DIRECTORY_ERROR_TYPE_UNKNOWN:
