@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -41,74 +41,51 @@
  **/
 
 #include <libinfinity/common/inf-io.h>
+#include <libinfinity/inf-define-enum.h>
 
-GType
-inf_io_event_get_type(void)
-{
-  static GType io_event_type = 0;
+typedef struct _InfIoWatchUnix InfIoWatchUnix;
+struct _InfIoWatchUnix {
+  InfNativeSocket socket;
+  gpointer user_data;
+  GDestroyNotify notify;
+};
 
-  if(!io_event_type)
+static const GFlagsValue inf_io_event_values[] = {
   {
-    static const GFlagsValue io_event_values[] = {
-      {
-        INF_IO_INCOMING,
-        "INF_IO_INCOMING",
-        "incoming"
-      }, {
-        INF_IO_OUTGOING,
-        "INF_IO_OUTGOING",
-        "outgoing"
-      }, {
-        INF_IO_ERROR,
-        "INF_IO_ERROR",
-        "error"
-      }, {
-        0,
-        NULL,
-        NULL
-      }
-    };
-
-    io_event_type = g_flags_register_static(
-      "InfIoEvent",
-      io_event_values
-    );
+    INF_IO_INCOMING,
+    "INF_IO_INCOMING",
+    "incoming"
+  }, {
+    INF_IO_OUTGOING,
+    "INF_IO_OUTGOING",
+    "outgoing"
+  }, {
+    INF_IO_ERROR,
+    "INF_IO_ERROR",
+    "error"
+  }, {
+    0,
+    NULL,
+    NULL
   }
+};
 
-  return io_event_type;
+INF_DEFINE_FLAGS_TYPE(InfIoEvent, inf_io_event, inf_io_event_values)
+G_DEFINE_INTERFACE(InfIo, inf_io, G_TYPE_OBJECT)
+
+static void
+inf_io_watch_unix_free(gpointer data)
+{
+  InfIoWatchUnix* unixwatch;
+  unixwatch = (InfIoWatchUnix*)data;
+
+  unixwatch->notify(unixwatch->user_data);
+  g_slice_free(InfIoWatchUnix, unixwatch);
 }
 
-GType
-inf_io_get_type(void)
+static void
+inf_io_default_init(InfIoInterface* iface)
 {
-  static GType io_type = 0;
-
-  if(!io_type)
-  {
-    static const GTypeInfo io_info = {
-      sizeof(InfIoIface),    /* class_size */
-      NULL,                  /* base_init */
-      NULL,                  /* base_finalize */
-      NULL,                  /* class_init */
-      NULL,                  /* class_finalize */
-      NULL,                  /* class_data */
-      0,                     /* instance_size */
-      0,                     /* n_preallocs */
-      NULL,                  /* instance_init */
-      NULL                   /* value_table */
-    };
-
-    io_type = g_type_register_static(
-      G_TYPE_INTERFACE,
-      "InfIo",
-      &io_info,
-      0
-    );
-
-    g_type_interface_add_prerequisite(io_type, G_TYPE_OBJECT);
-  }
-
-  return io_type;
 }
 
 /**
@@ -124,7 +101,8 @@ inf_io_get_type(void)
  * Monitors the given socket for activity and calls @func if one of the
  * events specified in @events occurs.
  *
- * Returns: A #InfIoWatch that can be used to update or remove the watch.
+ * Returns: (transfer none): A #InfIoWatch that can be used to update or
+ * remove the watch.
  **/
 InfIoWatch*
 inf_io_add_watch(InfIo* io,
@@ -134,7 +112,7 @@ inf_io_add_watch(InfIo* io,
                  gpointer user_data,
                  GDestroyNotify notify)
 {
-  InfIoIface* iface;
+  InfIoInterface* iface;
 
   g_return_val_if_fail(INF_IS_IO(io), NULL);
   g_return_val_if_fail(socket != NULL, NULL);
@@ -146,6 +124,53 @@ inf_io_add_watch(InfIo* io,
   return iface->add_watch(io, socket, events, func, user_data, notify);
 }
 
+#ifdef G_OS_UNIX
+/**
+ * inf_io_add_watch_from_fd:
+ * @io: A #InfIo.
+ * @fd: The file descriptor to watch.
+ * @events: Events to watch for.
+ * @func: Function to be called when one of the events occurs.
+ * @user_data: Extra data to pass to @func.
+ * @notify: A #GDestroyNotify that is called when @user_data is no longer
+ * needed, or %NULL.
+ *
+ * Monitors the given file descriptor for activity and calls @func if one of
+ * the events specified in @events occurs. This is equivalent to
+ * inf_io_add_watch() for a socket that represents the given file descriptor,
+ * but takes care of the memory management of the file descriptor. This
+ * function is especially intended for language bindings. It is only available
+ * on unix-like operating systems.
+ *
+ * Returns: (transfer none): A #InfIoWatch that can be used to update or
+ * remove the watch.
+ **/
+InfIoWatch*
+inf_io_add_watch_from_fd(InfIo* io,
+                         int fd,
+                         InfIoEvent events,
+                         InfIoWatchFunc func,
+                         gpointer user_data,
+                         GDestroyNotify notify)
+{
+  InfIoWatchUnix* watch;
+
+  watch = g_slice_new(InfIoWatchUnix);
+  watch->socket = fd;
+  watch->user_data = user_data;
+  watch->notify = notify;
+
+  return inf_io_add_watch(
+    io,
+    &watch->socket,
+    events,
+    func,
+    watch,
+    inf_io_watch_unix_free
+  );
+}
+#endif
+
 /**
  * inf_io_update_watch:
  * @io: A #InfIo.
@@ -161,7 +186,7 @@ inf_io_update_watch(InfIo* io,
                     InfIoWatch* watch,
                     InfIoEvent events)
 {
-  InfIoIface* iface;
+  InfIoInterface* iface;
 
   g_return_if_fail(INF_IS_IO(io));
   g_return_if_fail(watch != NULL);
@@ -184,7 +209,7 @@ void
 inf_io_remove_watch(InfIo* io,
                     InfIoWatch* watch)
 {
-  InfIoIface* iface;
+  InfIoInterface* iface;
 
   g_return_if_fail(INF_IS_IO(io));
   g_return_if_fail(watch != NULL);
@@ -207,7 +232,8 @@ inf_io_remove_watch(InfIo* io,
  * Calls @func after at least @msecs milliseconds have elapsed. The timeout
  * is removed after it has elapsed.
  *
- * Returns: A timeout handle that can be used to remove the timeout.
+ * Returns: (transfer none): A timeout handle that can be used to remove
+ * the timeout.
  **/
 InfIoTimeout*
 inf_io_add_timeout(InfIo* io,
@@ -216,7 +242,7 @@ inf_io_add_timeout(InfIo* io,
                    gpointer user_data,
                    GDestroyNotify notify)
 {
-  InfIoIface* iface;
+  InfIoInterface* iface;
 
   g_return_val_if_fail(INF_IS_IO(io), NULL);
   g_return_val_if_fail(func != NULL, NULL);
@@ -238,7 +264,7 @@ void
 inf_io_remove_timeout(InfIo* io,
                       InfIoTimeout* timeout)
 {
-  InfIoIface* iface;
+  InfIoInterface* iface;
 
   g_return_if_fail(INF_IS_IO(io));
   g_return_if_fail(timeout != NULL);
@@ -260,8 +286,8 @@ inf_io_remove_timeout(InfIo* io,
  * Schedules @func to be called by the thread @io runs in. This function can
  * be used from a different thread to communicate to @io's thread.
  *
- * Returns: A dispatch handle that can be used to stop the dispatched function
- * from being called as long as it has not yet been called.
+ * Returns: (transfer none): A dispatch handle that can be used to stop the
+ * dispatched function from being called as long as it has not yet been called.
  **/
 InfIoDispatch*
 inf_io_add_dispatch(InfIo* io,
@@ -269,7 +295,7 @@ inf_io_add_dispatch(InfIo* io,
                     gpointer user_data,
                     GDestroyNotify notify)
 {
-  InfIoIface* iface;
+  InfIoInterface* iface;
 
   g_return_val_if_fail(INF_IS_IO(io), NULL);
   g_return_val_if_fail(func != NULL, NULL);
@@ -291,7 +317,7 @@ void
 inf_io_remove_dispatch(InfIo* io,
                        InfIoDispatch* dispatch)
 {
-  InfIoIface* iface;
+  InfIoInterface* iface;
 
   g_return_if_fail(INF_IS_IO(io));
   g_return_if_fail(dispatch != NULL);
