@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -49,6 +49,8 @@
 
 #include <string.h>
 
+G_DEFINE_BOXED_TYPE(InfSaslContext, inf_sasl_context, inf_sasl_context_ref, inf_sasl_context_unref)
+
 typedef enum _InfSaslContextMessageType {
   /* main -> session */
   INF_SASL_CONTEXT_MESSAGE_TERMINATE,
@@ -131,10 +133,11 @@ struct _InfSaslContext {
 
   InfSaslContextCallbackFunc callback;
   gpointer callback_user_data;
+  GDestroyNotify callback_notify;
 
   /* protects the session list, the callback function and access to the
    * Gsasl object. */
-  GMutex* mutex;
+  GMutex mutex;
 };
 
 /*
@@ -279,13 +282,13 @@ inf_sasl_context_session_message_func(gpointer user_data)
     break;
   case INF_SASL_CONTEXT_MESSAGE_QUERY:
     /* main thread */
-    g_mutex_lock(message->session->context->mutex);
+    g_mutex_lock(&message->session->context->mutex);
     g_assert(message->session->dispatch != NULL);
     message->session->dispatch = NULL;
 
     func = message->session->context->callback;
     func_user_data = message->session->context->callback_user_data;
-    g_mutex_unlock(message->session->context->mutex);
+    g_mutex_unlock(&message->session->context->mutex);
 
     func(
       message->session,
@@ -297,10 +300,10 @@ inf_sasl_context_session_message_func(gpointer user_data)
     break;
   case INF_SASL_CONTEXT_MESSAGE_STEPPED:
     /* main thread */
-    g_mutex_lock(message->session->context->mutex);
+    g_mutex_lock(&message->session->context->mutex);
     g_assert(message->session->dispatch != NULL);
     message->session->dispatch = NULL;
-    g_mutex_unlock(message->session->context->mutex);
+    g_mutex_unlock(&message->session->context->mutex);
 
     switch(message->shared.stepped.retval)
     {
@@ -364,7 +367,7 @@ inf_sasl_context_gsasl_callback(Gsasl* gsasl,
     inf_sasl_context_message_free
   );
 
-  g_mutex_unlock(session->context->mutex);
+  g_mutex_unlock(&session->context->mutex);
 
   session->retval = G_MAXINT;
   while(session->status == INF_SASL_CONTEXT_SESSION_INNER &&
@@ -375,7 +378,7 @@ inf_sasl_context_gsasl_callback(Gsasl* gsasl,
     inf_sasl_context_message_free(message);
   }
 
-  g_mutex_lock(session->context->mutex);
+  g_mutex_lock(&session->context->mutex);
 
   /* return on terminate */
   if(session->status == INF_SASL_CONTEXT_SESSION_TERMINATE)
@@ -409,7 +412,7 @@ inf_sasl_context_thread_func(gpointer data)
       inf_sasl_context_message_free(message);
       break;
     case INF_SASL_CONTEXT_SESSION_INNER:
-      g_mutex_lock(session->context->mutex);
+      g_mutex_lock(&session->context->mutex);
 
       g_assert(session->dispatch == NULL);
 
@@ -421,7 +424,7 @@ inf_sasl_context_thread_func(gpointer data)
         &output
       );
 
-      g_mutex_unlock(session->context->mutex);
+      g_mutex_unlock(&session->context->mutex);
 
       g_free(session->step64);
       session->step64 = NULL;
@@ -439,7 +442,7 @@ inf_sasl_context_thread_func(gpointer data)
 
         session->status = INF_SASL_CONTEXT_SESSION_OUTER;
 
-        g_mutex_lock(session->context->mutex);
+        g_mutex_lock(&session->context->mutex);
 
         g_assert(session->dispatch == NULL);
 
@@ -456,7 +459,7 @@ inf_sasl_context_thread_func(gpointer data)
           inf_sasl_context_message_free
         );
 
-        g_mutex_unlock(session->context->mutex);
+        g_mutex_unlock(&session->context->mutex);
       }
       else
       {
@@ -509,10 +512,10 @@ inf_sasl_context_start_session(InfSaslContext* context,
   context->sessions = g_slist_prepend(context->sessions, session);
   gsasl_session_hook_set(gsasl_session, session);
 
-  session->thread = g_thread_create(
+  session->thread = g_thread_try_new(
+    "InfSaslContext",
     inf_sasl_context_thread_func,
     session,
-    TRUE,
     error
   );
 
@@ -532,32 +535,15 @@ inf_sasl_context_start_session(InfSaslContext* context,
  * Public API
  */
 
-GType
-inf_sasl_context_get_type(void)
-{
-  static GType sasl_context_type = 0;
-
-  if(!sasl_context_type)
-  {
-    sasl_context_type = g_boxed_type_register_static(
-      "InfSaslContext",
-      (GBoxedCopyFunc)inf_sasl_context_ref,
-      (GBoxedFreeFunc)inf_sasl_context_unref
-    );
-  }
-
-  return sasl_context_type;
-}
-
 /**
- * inf_sasl_context_new:
+ * inf_sasl_context_new: (constructor)
  * @error: Location to store error information, if any.
  *
  * Creates a new #InfSaslContext with a reference count of 1. If the function
  * fails it returns %NULL and @error is set.
  *
- * Returns: A new #InfSaslContext, or %NULL on error. Free with
- * inf_sasl_context_unref() when no longer needed.
+ * Returns: (transfer full): A new #InfSaslContext, or %NULL on error. Free
+ * with inf_sasl_context_unref() when no longer needed.
  */
 InfSaslContext*
 inf_sasl_context_new(GError** error)
@@ -583,11 +569,12 @@ inf_sasl_context_new(GError** error)
 
   sasl->callback = NULL;
   sasl->callback_user_data = NULL;
+  sasl->callback_notify = NULL;
 
   gsasl_callback_set(gsasl, inf_sasl_context_gsasl_callback);
   gsasl_callback_hook_set(gsasl, sasl);
-  
-  sasl->mutex = g_mutex_new();
+
+  g_mutex_init(&sasl->mutex);
 
   return sasl;
 }
@@ -640,7 +627,10 @@ inf_sasl_context_unref(InfSaslContext* context)
     /* Again we don't need to lock the mutex for this since all session
      * threads have been stopped at this point. */
     gsasl_done(context->gsasl);
-    g_mutex_free(context->mutex);
+    g_mutex_clear(&context->mutex);
+
+    if(context->callback_notify != NULL)
+      context->callback_notify(context->callback_user_data);
 
     g_slice_free(InfSaslContext, context);
   }
@@ -651,6 +641,8 @@ inf_sasl_context_unref(InfSaslContext* context)
  * @context: A #InfSaslContext.
  * @callback: A function to call to query properties for authentication.
  * @user_data: Additional context to pass to @callback.
+ * @notify: Function called to destroy @user_data once it is no longer needed,
+ * or %NULL.
  *
  * Sets the callback to call when, during authentication, a certain properties
  * needs to be provided, such as a username or a password. The callback
@@ -667,14 +659,18 @@ inf_sasl_context_unref(InfSaslContext* context)
 void
 inf_sasl_context_set_callback(InfSaslContext* context,
                               InfSaslContextCallbackFunc callback,
-                              gpointer user_data)
+                              gpointer user_data,
+                              GDestroyNotify notify)
 {
   g_return_if_fail(context != NULL);
 
-  g_mutex_lock(context->mutex);
+  g_mutex_lock(&context->mutex);
+  if(context->callback_notify != NULL)
+    context->callback_notify(context->callback_user_data);
   context->callback = callback;
   context->callback_user_data = user_data;
-  g_mutex_unlock(context->mutex);
+  context->callback_notify = notify;
+  g_mutex_unlock(&context->mutex);
 }
 
 /**
@@ -692,7 +688,7 @@ inf_sasl_context_set_callback(InfSaslContext* context,
  *
  * The callback function will be called in the thread that @io runs in.
  *
- * Returns: A #InfSaslContextSession.
+ * Returns: (transfer none): A #InfSaslContextSession.
  */
 InfSaslContextSession*
 inf_sasl_context_client_start_session(InfSaslContext* context,
@@ -710,12 +706,12 @@ inf_sasl_context_client_start_session(InfSaslContext* context,
   g_return_val_if_fail(mech != NULL, NULL);
   g_return_val_if_fail(error == NULL || *error == NULL, NULL);
 
-  g_mutex_lock(context->mutex);
+  g_mutex_lock(&context->mutex);
 
   status = gsasl_client_start(context->gsasl, mech, &gsasl_session);
   if(status != GSASL_OK)
   {
-    g_mutex_unlock(context->mutex);
+    g_mutex_unlock(&context->mutex);
     inf_gsasl_set_error(error, status);
     return NULL;
   }
@@ -731,11 +727,11 @@ inf_sasl_context_client_start_session(InfSaslContext* context,
   if(session == NULL)
   {
     gsasl_finish(gsasl_session);
-    g_mutex_unlock(context->mutex);
+    g_mutex_unlock(&context->mutex);
     return NULL;
   }
 
-  g_mutex_unlock(context->mutex);
+  g_mutex_unlock(&context->mutex);
   return session;
 }
 
@@ -747,22 +743,23 @@ inf_sasl_context_client_start_session(InfSaslContext* context,
  * Returns a newly allocated space-separated string containing SASL mechanisms
  * that @context supports for client sessions.
  *
- * Returns: A newly allocated string. Free with gsasl_free() when no longer
- * in use.
+ * Returns: (transfer full): A newly allocated string. Free with g_free()
+ * when no longer in use.
  */
-char*
+gchar*
 inf_sasl_context_client_list_mechanisms(InfSaslContext* context,
                                         GError** error)
 {
   int ret;
   char* out;
+  gchar* v;
 
   g_return_val_if_fail(context != NULL, NULL);
   g_return_val_if_fail(error == NULL || *error == NULL, NULL);
 
-  g_mutex_lock(context->mutex);
+  g_mutex_lock(&context->mutex);
   ret = gsasl_client_mechlist(context->gsasl, &out);
-  g_mutex_unlock(context->mutex);
+  g_mutex_unlock(&context->mutex);
 
   if(ret != GSASL_OK)
   {
@@ -771,7 +768,10 @@ inf_sasl_context_client_list_mechanisms(InfSaslContext* context,
     return NULL;
   }
 
-  return out;
+  v = g_strdup(out);
+  gsasl_free(out);
+
+  return v;
 }
 
 /**
@@ -793,9 +793,9 @@ inf_sasl_context_client_supports_mechanism(InfSaslContext* context,
   g_return_val_if_fail(context != NULL, FALSE);
   g_return_val_if_fail(mech != NULL, FALSE);
 
-  g_mutex_lock(context->mutex);
+  g_mutex_lock(&context->mutex);
   result = gsasl_client_support_p(context->gsasl, mech);
-  g_mutex_unlock(context->mutex);
+  g_mutex_unlock(&context->mutex);
 
   return result != 0;
 }
@@ -808,7 +808,7 @@ inf_sasl_context_client_supports_mechanism(InfSaslContext* context,
  * Given a list of SASL mechanisms this function suggests the which is the
  * "best" one to be used.
  *
- * Returns: The name of the suggested mechanism.
+ * Returns: (transfer none): The name of the suggested mechanism.
  */
 const char*
 inf_sasl_context_client_suggest_mechanism(InfSaslContext* context,
@@ -818,9 +818,9 @@ inf_sasl_context_client_suggest_mechanism(InfSaslContext* context,
   g_return_val_if_fail(context != NULL, NULL);
   g_return_val_if_fail(mechanisms != NULL, NULL);
 
-  g_mutex_lock(context->mutex);
+  g_mutex_lock(&context->mutex);
   suggestion = gsasl_client_suggest_mechanism(context->gsasl, mechanisms);
-  g_mutex_unlock(context->mutex);
+  g_mutex_unlock(&context->mutex);
 
   return suggestion;
 }
@@ -840,7 +840,7 @@ inf_sasl_context_client_suggest_mechanism(InfSaslContext* context,
  *
  * The callback function will be called in the thread that @io runs in.
  *
- * Returns: A #InfSaslContextSession.
+ * Returns: (transfer none): A #InfSaslContextSession.
  */
 InfSaslContextSession*
 inf_sasl_context_server_start_session(InfSaslContext* context,
@@ -858,12 +858,12 @@ inf_sasl_context_server_start_session(InfSaslContext* context,
   g_return_val_if_fail(mech != NULL, NULL);
   g_return_val_if_fail(error == NULL || *error == NULL, NULL);
 
-  g_mutex_lock(context->mutex);
+  g_mutex_lock(&context->mutex);
 
   status = gsasl_server_start(context->gsasl, mech, &gsasl_session);
   if(status != GSASL_OK)
   {
-    g_mutex_unlock(context->mutex);
+    g_mutex_unlock(&context->mutex);
     inf_gsasl_set_error(error, status);
     return NULL;
   }
@@ -879,11 +879,11 @@ inf_sasl_context_server_start_session(InfSaslContext* context,
   if(session == NULL)
   {
     gsasl_finish(gsasl_session);
-    g_mutex_unlock(context->mutex);
+    g_mutex_unlock(&context->mutex);
     return NULL;
   }
 
-  g_mutex_unlock(context->mutex);
+  g_mutex_unlock(&context->mutex);
   return session;
 }
 
@@ -895,8 +895,8 @@ inf_sasl_context_server_start_session(InfSaslContext* context,
  * Returns a newly allocated space-separated string containing SASL mechanisms
  * that @context supports for server sessions.
  *
- * Returns: A newly allocated string. Free with gsasl_free() when no longer
- * in use.
+ * Returns: (transfer full): A newly allocated string. Free with g_free()
+ * when no longer in use.
  */
 char*
 inf_sasl_context_server_list_mechanisms(InfSaslContext* context,
@@ -904,13 +904,14 @@ inf_sasl_context_server_list_mechanisms(InfSaslContext* context,
 {
   int ret;
   char* out;
+  gchar* v;
 
   g_return_val_if_fail(context != NULL, NULL);
   g_return_val_if_fail(error == NULL || *error == NULL, NULL);
 
-  g_mutex_lock(context->mutex);
+  g_mutex_lock(&context->mutex);
   ret = gsasl_server_mechlist(context->gsasl, &out);
-  g_mutex_unlock(context->mutex);
+  g_mutex_unlock(&context->mutex);
 
   if(ret != GSASL_OK)
   {
@@ -919,7 +920,10 @@ inf_sasl_context_server_list_mechanisms(InfSaslContext* context,
     return NULL;
   }
 
-  return out;
+  v = g_strdup(out);
+  gsasl_free(out);
+
+  return v;
 }
 
 /**
@@ -941,9 +945,9 @@ inf_sasl_context_server_supports_mechanism(InfSaslContext* context,
   g_return_val_if_fail(context != NULL, FALSE);
   g_return_val_if_fail(mech != NULL, FALSE);
 
-  g_mutex_lock(context->mutex);
+  g_mutex_lock(&context->mutex);
   ret = gsasl_server_support_p(context->gsasl, mech);
-  g_mutex_unlock(context->mutex);
+  g_mutex_unlock(&context->mutex);
 
   return ret != 0;
 }
@@ -966,10 +970,10 @@ inf_sasl_context_stop_session(InfSaslContext* context,
   g_return_if_fail(context != NULL);
   g_return_if_fail(session != NULL);
 
-  g_mutex_lock(context->mutex);
+  g_mutex_lock(&context->mutex);
   g_return_if_fail(g_slist_find(context->sessions, session) != NULL);
   g_return_if_fail(session->context == context);
-  g_mutex_unlock(context->mutex);
+  g_mutex_unlock(&context->mutex);
 
   /* Tell client thread to terminate */
   g_async_queue_push(
@@ -979,13 +983,13 @@ inf_sasl_context_stop_session(InfSaslContext* context,
 
   g_thread_join(session->thread);
 
-  g_mutex_lock(context->mutex);
+  g_mutex_lock(&context->mutex);
   if(session->dispatch != NULL)
     inf_io_remove_dispatch(session->main_io, session->dispatch);
 
   context->sessions = g_slist_remove(context->sessions, session);
   gsasl_finish(session->session);
-  g_mutex_unlock(context->mutex);
+  g_mutex_unlock(&context->mutex);
 
   /* Note that this assertion should hold because us pushing the terminate
    * message into the end of the queue, and all other queued messages will
@@ -1007,8 +1011,8 @@ inf_sasl_context_stop_session(InfSaslContext* context,
  * yet exist then this function returns %NULL. It does not invoke the
  * #InfSaslContextCallbackFunc to query it.
  *
- * Returns: The value of the property, or %NULL. The value is owned by the
- * session and must not be freed.
+ * Returns: (allow-none): The value of the property, or %NULL. The value is
+ * owned by the session and must not be freed.
  */
 const char*
 inf_sasl_context_session_get_property(InfSaslContextSession* session,
@@ -1020,9 +1024,9 @@ inf_sasl_context_session_get_property(InfSaslContextSession* session,
 
   /* TODO: We should g_strdup the return value for thread safety reasons */
 
-  g_mutex_lock(session->context->mutex);
+  g_mutex_lock(&session->context->mutex);
   property = gsasl_property_fast(session->session, prop);
-  g_mutex_unlock(session->context->mutex);
+  g_mutex_unlock(&session->context->mutex);
 
   return property;
 }
@@ -1031,7 +1035,7 @@ inf_sasl_context_session_get_property(InfSaslContextSession* session,
  * inf_sasl_context_session_set_property:
  * @session: A #InfSaslContextSession.
  * @prop: A SASL property.
- * @value: The value to set the property to.
+ * @value: (allow-none): The value to set the property to.
  *
  * Sets the property @prop in @session to @value.
  */
@@ -1042,9 +1046,9 @@ inf_sasl_context_session_set_property(InfSaslContextSession* session,
 {
   g_return_if_fail(session != NULL);
 
-  g_mutex_lock(session->context->mutex);
+  g_mutex_lock(&session->context->mutex);
   gsasl_property_set(session->session, prop, value);
-  g_mutex_unlock(session->context->mutex);
+  g_mutex_unlock(&session->context->mutex);
 }
 
 /**
@@ -1076,7 +1080,8 @@ inf_sasl_context_session_continue(InfSaslContextSession* session,
  * inf_sasl_context_session_feed:
  * @session: A #InfSaslContextSession.
  * @data: The data to feed to the SASL session.
- * @func: The function to call when the data has been processed.
+ * @func: (scope async): The function to call when the data has been
+ * processed.
  * @user_data: Additional user data to pass to @func.
  *
  * This function feeds data from the session's remote counterpart to @session.
