@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -43,15 +43,15 @@
  * To be notified about changes other users make to a session, you need to
  * subscribe to the session (on client side), or wait for incoming
  * subscriptions (on server side). This is normally done by
- * infc_browser_iter_subscribe_session(). The first action that is performed
+ * inf_browser_subscribe(). The first action that is performed
  * upon subscription is a synchronization as described above. When the
  * synchronization is complete, the #InfSession::synchronization-complete signal
  * is emitted.
  *
  * After subscription, one can observe modifications other users make, but it is
  * not possible to make own modifications. Before doing so, a #InfUser needs to
- * be joined. This is done by client/server specific API such as
- * infc_session_proxy_join_user() or infd_session_proxy_add_user(). The
+ * be joined. This is done by client/server specific API from
+ * #InfSessionProxy, in particular inf_session_proxy_join_user(). The
  * required parameters still depend on the actual note type, which is why most
  * note implementations offer their own API to join a user.
  **/
@@ -61,9 +61,9 @@
 #include <libinfinity/common/inf-xml-util.h>
 #include <libinfinity/common/inf-error.h>
 #include <libinfinity/communication/inf-communication-object.h>
-#include <libinfinity/inf-marshal.h>
 #include <libinfinity/inf-i18n.h>
 #include <libinfinity/inf-signals.h>
+#include <libinfinity/inf-define-enum.h>
 
 #include <string.h>
 
@@ -71,6 +71,30 @@
 /* TODO: Cache requests received by other group members
  * during synchronization and process them afterwards */
 
+static const GEnumValue inf_session_status_values[] = {
+  {
+    INF_SESSION_PRESYNC,
+    "INF_SESSION_PRESYNC",
+    "presync"
+  }, {
+    INF_SESSION_SYNCHRONIZING,
+    "INF_SESSION_SYNCHRONIZING",
+    "synchronizing"
+  }, {
+    INF_SESSION_RUNNING,
+    "INF_SESSION_RUNNING",
+    "running"
+  }, {
+    INF_SESSION_CLOSED,
+    "INF_SESSION_CLOSED",
+    "closed"
+  }, {
+    0,
+    NULL,
+    NULL
+  }
+};
+
 typedef struct _InfSessionSync InfSessionSync;
 struct _InfSessionSync {
   InfCommunicationGroup* group;
@@ -140,6 +164,8 @@ enum {
 
 enum {
   CLOSE,
+  ERROR,
+
   SYNCHRONIZATION_BEGIN,
   SYNCHRONIZATION_PROGRESS,
   SYNCHRONIZATION_COMPLETE,
@@ -150,10 +176,15 @@ enum {
 
 #define INF_SESSION_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_TYPE_SESSION, InfSessionPrivate))
 
-static GObjectClass* parent_class;
 static guint session_signals[LAST_SIGNAL];
 static GQuark inf_session_sync_error_quark;
 
+static void inf_session_communication_object_iface_init(InfCommunicationObjectInterface* iface);
+INF_DEFINE_ENUM_TYPE(InfSessionStatus, inf_session_status, inf_session_status_values)
+G_DEFINE_TYPE_WITH_CODE(InfSession, inf_session, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfSession)
+  G_IMPLEMENT_INTERFACE(INF_COMMUNICATION_TYPE_OBJECT, inf_session_communication_object_iface_init))
+
 /*
  * Utility functions.
  */
@@ -355,11 +386,10 @@ inf_session_connection_notify_status_cb(InfXmlConnection* connection,
   if(status == INF_XML_CONNECTION_CLOSED ||
      status == INF_XML_CONNECTION_CLOSING)
   {
-    g_set_error(
+    g_set_error_literal(
       &error,
       inf_session_sync_error_quark,
       INF_SESSION_SYNC_ERROR_CONNECTION_CLOSED,
-      "%s",
       inf_session_sync_strerror(INF_SESSION_SYNC_ERROR_CONNECTION_CLOSED)
     );
 
@@ -418,34 +448,11 @@ inf_session_connection_notify_status_cb(InfXmlConnection* connection,
  */
 
 static void
-inf_session_register_sync(InfSession* session)
+inf_session_init(InfSession* session)
 {
-  /* TODO: Use _constructor */
   InfSessionPrivate* priv;
   priv = INF_SESSION_PRIVATE(session);
 
-  /* Register CommunicationObject when all requirements for initial
-   * synchronization are met. */
-  if(priv->status == INF_SESSION_SYNCHRONIZING &&
-     priv->manager != NULL &&
-     priv->shared.sync.conn != NULL &&
-     priv->shared.sync.group != NULL)
-  {
-    g_object_ref(priv->shared.sync.group);
-
-  }
-}
-
-static void
-inf_session_init(GTypeInstance* instance,
-                 gpointer g_class)
-{
-  InfSession* session;
-  InfSessionPrivate* priv;
-
-  session = INF_SESSION(instance);
-  priv = INF_SESSION_PRIVATE(session);
-
   priv->manager = NULL;
   priv->buffer = NULL;
   priv->user_table = NULL;
@@ -454,20 +461,13 @@ inf_session_init(GTypeInstance* instance,
   priv->shared.run.syncs = NULL;
 }
 
-static GObject*
-inf_session_constructor(GType type,
-                        guint n_construct_properties,
-                        GObjectConstructParam* construct_properties)
+static void
+inf_session_constructed(GObject* object)
 {
-  GObject* object;
   InfSessionPrivate* priv;
   InfXmlConnection* sync_conn;
 
-  object = G_OBJECT_CLASS(parent_class)->constructor(
-    type,
-    n_construct_properties,
-    construct_properties
-  );
+  G_OBJECT_CLASS(inf_session_parent_class)->constructed(object);
 
   priv = INF_SESSION_PRIVATE(object);
 
@@ -505,8 +505,6 @@ inf_session_constructor(GType type,
       object
     );
   }
-
-  return object;
 }
 
 static void
@@ -534,7 +532,7 @@ inf_session_dispose(GObject* object)
   g_object_unref(G_OBJECT(priv->manager));
   priv->manager = NULL;
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_session_parent_class)->dispose(object);
 }
 
 static void
@@ -546,7 +544,7 @@ inf_session_finalize(GObject* object)
   session = INF_SESSION(object);
   priv = INF_SESSION_PRIVATE(session);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(inf_session_parent_class)->finalize(object);
 }
 
 static void
@@ -566,7 +564,6 @@ inf_session_set_property(GObject* object,
   case PROP_COMMUNICATION_MANAGER:
     g_assert(priv->manager == NULL); /* construct only */
     priv->manager = INF_COMMUNICATION_MANAGER(g_value_dup_object(value));
-    inf_session_register_sync(session);
     break;
   case PROP_BUFFER:
     g_assert(priv->buffer == NULL); /* construct only */
@@ -778,11 +775,10 @@ inf_session_handle_user_status_change(InfSession* session,
   if(inf_user_get_status(user) == INF_USER_UNAVAILABLE ||
      inf_user_get_connection(user) != connection)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       inf_user_error_quark(),
       INF_USER_ERROR_NOT_JOINED,
-      "%s",
       _("User did not join from this connection")
     );
 
@@ -891,8 +887,7 @@ inf_session_process_xml_sync_impl(InfSession* session,
     user = inf_session_add_user(
       session,
       (GParameter*)user_props->data,
-      user_props->len,
-      error
+      user_props->len
     );
 
     for(i = 0; i < user_props->len; ++ i)
@@ -1101,11 +1096,10 @@ inf_session_validate_user_props_impl(InfSession* session,
   parameter = inf_session_lookup_user_property(params, n_params, "id");
   if(parameter == NULL)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       inf_session_sync_error_quark,
       INF_SESSION_SYNC_ERROR_ID_NOT_PRESENT,
-      "%s",
       inf_session_sync_strerror(INF_SESSION_SYNC_ERROR_ID_NOT_PRESENT)
     );
 
@@ -1117,11 +1111,10 @@ inf_session_validate_user_props_impl(InfSession* session,
 
   if(user != NULL && user != exclude)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       inf_session_sync_error_quark,
       INF_SESSION_SYNC_ERROR_ID_IN_USE,
-      "%s",
       inf_session_sync_strerror(INF_SESSION_SYNC_ERROR_ID_IN_USE)
     );
 
@@ -1131,11 +1124,10 @@ inf_session_validate_user_props_impl(InfSession* session,
   parameter = inf_session_lookup_user_property(params, n_params, "name");
   if(parameter == NULL)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       inf_session_sync_error_quark,
       INF_SESSION_SYNC_ERROR_NAME_NOT_PRESENT,
-      "%s",
       inf_session_sync_strerror(INF_SESSION_SYNC_ERROR_NAME_NOT_PRESENT)
     );
 
@@ -1147,11 +1139,10 @@ inf_session_validate_user_props_impl(InfSession* session,
 
   if(user != NULL && user != exclude)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       inf_session_sync_error_quark,
       INF_SESSION_SYNC_ERROR_NAME_IN_USE,
-      "%s",
       inf_session_sync_strerror(INF_SESSION_SYNC_ERROR_NAME_IN_USE)
     );
 
@@ -1187,11 +1178,10 @@ inf_session_handle_received_sync_message(InfSession* session,
   {
     local_error = NULL;
 
-    g_set_error(
+    g_set_error_literal(
       &local_error,
       inf_session_sync_error_quark,
       INF_SESSION_SYNC_ERROR_SENDER_CANCELLED,
-      "%s",
       inf_session_sync_strerror(INF_SESSION_SYNC_ERROR_SENDER_CANCELLED)
     );
 
@@ -1224,11 +1214,10 @@ inf_session_handle_received_sync_message(InfSession* session,
   {
     if(priv->shared.sync.messages_total > 0)
     {
-      g_set_error(
+      g_set_error_literal(
         error,
         inf_session_sync_error_quark,
         INF_SESSION_SYNC_ERROR_UNEXPECTED_BEGIN_OF_SYNC,
-        "%s",
         inf_session_sync_strerror(
           INF_SESSION_SYNC_ERROR_UNEXPECTED_BEGIN_OF_SYNC
         )
@@ -1241,11 +1230,10 @@ inf_session_handle_received_sync_message(InfSession* session,
       num_messages = xmlGetProp(node, (const xmlChar*)"num-messages");
       if(num_messages == NULL)
       {
-        g_set_error(
+        g_set_error_literal(
           error,
           inf_session_sync_error_quark,
           INF_SESSION_SYNC_ERROR_NUM_MESSAGES_MISSING,
-          "%s",
           inf_session_sync_strerror(
             INF_SESSION_SYNC_ERROR_NUM_MESSAGES_MISSING
           )
@@ -1286,11 +1274,10 @@ inf_session_handle_received_sync_message(InfSession* session,
     if(priv->shared.sync.messages_received !=
        priv->shared.sync.messages_total)
     {
-      g_set_error(
+      g_set_error_literal(
         error,
         inf_session_sync_error_quark,
         INF_SESSION_SYNC_ERROR_UNEXPECTED_END_OF_SYNC,
-        "%s",
         inf_session_sync_strerror(
           INF_SESSION_SYNC_ERROR_UNEXPECTED_END_OF_SYNC
         )
@@ -1325,11 +1312,10 @@ inf_session_handle_received_sync_message(InfSession* session,
   {
     if(priv->shared.sync.messages_received == 0)
     {
-      g_set_error(
+      g_set_error_literal(
         error,
         inf_session_sync_error_quark,
         INF_SESSION_SYNC_ERROR_EXPECTED_BEGIN_OF_SYNC,
-        "%s",
         inf_session_sync_strerror(
           INF_SESSION_SYNC_ERROR_EXPECTED_BEGIN_OF_SYNC
         )
@@ -1340,11 +1326,10 @@ inf_session_handle_received_sync_message(InfSession* session,
     else if(priv->shared.sync.messages_received ==
             priv->shared.sync.messages_total - 1)
     {
-      g_set_error(
+      g_set_error_literal(
         error,
         inf_session_sync_error_quark,
         INF_SESSION_SYNC_ERROR_EXPECTED_END_OF_SYNC,
-        "%s",
         inf_session_sync_strerror(INF_SESSION_SYNC_ERROR_EXPECTED_END_OF_SYNC)
       );
 
@@ -1451,14 +1436,14 @@ inf_session_communication_object_enqueued(InfCommunicationObject* comm_object,
 static InfCommunicationScope
 inf_session_communication_object_received(InfCommunicationObject* comm_object,
                                           InfXmlConnection* connection,
-                                          const xmlNodePtr node,
-                                          GError** error)
+                                          const xmlNodePtr node)
 {
   InfSessionClass* session_class;
   InfSession* session;
   InfSessionPrivate* priv;
   InfSessionSync* sync;
   gboolean result;
+  InfCommunicationScope scope;
   GError* local_error;
   const gchar* local_message;
 
@@ -1473,14 +1458,25 @@ inf_session_communication_object_received(InfCommunicationObject* comm_object,
     /* We do not expect any messages in presync. The whole idea of presync is
      * that one can keep a session around that is going to be synchronized
      * later, ie. when telling the remote site about it. */
+    local_error = NULL;
+
     g_set_error(
-      error,
+      &local_error,
       inf_session_sync_error_quark,
       INF_SESSION_SYNC_ERROR_GOT_MESSAGE_IN_PRESYNC,
       _("Unexpectedly received XML message \"%s\" in presync"),
       (const gchar*)node->name
     );
 
+     g_signal_emit(
+      G_OBJECT(session),
+      session_signals[ERROR],
+      0,
+      connection,
+      node,
+      local_error
+    );
+
     /* Don't forward unexpected message */
     return INF_COMMUNICATION_SCOPE_PTP;
   case INF_SESSION_SYNCHRONIZING:
@@ -1539,11 +1535,10 @@ inf_session_communication_object_received(InfCommunicationObject* comm_object,
         }
         else
         {
-          g_set_error(
+          g_set_error_literal(
             &local_error,
             inf_session_sync_error_quark,
             INF_SESSION_SYNC_ERROR_FAILED,
-            "%s",
             inf_session_sync_strerror(INF_SESSION_SYNC_ERROR_FAILED)
           );
         }
@@ -1572,13 +1567,21 @@ inf_session_communication_object_received(InfCommunicationObject* comm_object,
       }
       else
       {
+        local_error = NULL;
+
         g_set_error(
-          error,
+          &local_error,
           inf_session_sync_error_quark,
           INF_SESSION_SYNC_ERROR_UNEXPECTED_NODE,
           _("Received unexpected XML message \"%s\" while synchronizing"),
           (const gchar*)node->name
         );
+
+        /* We got sent something by the other session, even though we do not
+         * expect anything really. We consider it an error, and cancel the
+         * synchronization here. */
+
+        g_error_free(local_error);
       }
 
       /* Synchronization is always ptp only, don't forward */
@@ -1589,14 +1592,39 @@ inf_session_communication_object_received(InfCommunicationObject* comm_object,
       session_class = INF_SESSION_GET_CLASS(session);
       g_assert(session_class->process_xml_run != NULL);
 
-      return session_class->process_xml_run(session, connection, node, error);
-    }
+      local_error = NULL;
+      scope = session_class->process_xml_run(
+        session,
+        connection,
+        node,
+        &local_error
+      );
 
-    break;
+      if(local_error != NULL)
+      {
+        /* At this point we don't really know what's wrong with the session,
+         * or whether the error is fatal in a sense or not. We emit the
+         * "error" signal here, so that anyone who is interested can handle
+         * it, for example by closing the session, or logging it into a file
+         * for human inspection. */
+        g_signal_emit(
+          G_OBJECT(session),
+          session_signals[ERROR],
+          0,
+          connection,
+          node,
+          local_error
+        );
+
+        g_error_free(local_error);
+      }
+
+      return scope;
+    }
   case INF_SESSION_CLOSED:
   default:
     g_assert_not_reached();
-    break;
+    return INF_COMMUNICATION_SCOPE_PTP;
   }
 }
 
@@ -1609,22 +1637,9 @@ inf_session_close_handler(InfSession* session)
 {
   InfSessionPrivate* priv;
   InfSessionSync* sync;
-  xmlNodePtr xml;
-  GError* error;
-  InfXmlConnectionStatus status;
 
   priv = INF_SESSION_PRIVATE(session);
 
-  error = NULL;
-
-  g_set_error(
-    &error,
-    inf_session_sync_error_quark,
-    INF_SESSION_SYNC_ERROR_RECEIVER_CANCELLED,
-    "%s",
-    inf_session_sync_strerror(INF_SESSION_SYNC_ERROR_RECEIVER_CANCELLED)
-  );
-
   g_object_freeze_notify(G_OBJECT(session));
 
   switch(priv->status)
@@ -1636,13 +1651,7 @@ inf_session_close_handler(InfSession* session)
        * does not emit the close signal again: */
       priv->shared.presync.closing = TRUE;
 
-      g_signal_emit(
-        G_OBJECT(session),
-        session_signals[SYNCHRONIZATION_FAILED],
-        0,
-        priv->shared.presync.conn,
-        error
-      );
+      inf_session_cancel_synchronization(session, priv->shared.presync.conn);
     }
 
     inf_session_release_connection(session, priv->shared.presync.conn);
@@ -1654,23 +1663,7 @@ inf_session_close_handler(InfSession* session)
        * does not emit the close signal again: */
       priv->shared.sync.closing = TRUE;
 
-      /* If the connection was closed, another signal handler could close()
-       * the session explicitely (for example by disposing it), in which case
-       * we cannot send anything anymore through the connection even though
-       * sync.closing was not set already. */
-      g_object_get(G_OBJECT(priv->shared.sync.conn), "status", &status, NULL);
-      if(status == INF_XML_CONNECTION_OPEN)
-      {
-        inf_session_send_sync_error(session, error);
-      }
-
-      g_signal_emit(
-        G_OBJECT(session),
-        session_signals[SYNCHRONIZATION_FAILED],
-        0,
-        priv->shared.sync.conn,
-        error
-      );
+      inf_session_cancel_synchronization(session, priv->shared.sync.conn);
     }
 
     inf_session_release_connection(session, priv->shared.sync.conn);
@@ -1684,31 +1677,7 @@ inf_session_close_handler(InfSession* session)
     while(priv->shared.run.syncs != NULL)
     {
       sync = (InfSessionSync*)priv->shared.run.syncs->data;
-
-      /* If the sync-end message has already been enqueued within the
-       * connection manager, we cannot cancel it anymore, so the remote
-       * site will receive the full sync nevertheless, so we do not need
-       * to cancel anything. */
-      if(sync->status == INF_SESSION_SYNC_IN_PROGRESS)
-      {
-        inf_communication_group_cancel_messages(sync->group, sync->conn);
-
-        xml = xmlNewNode(NULL, (const xmlChar*)"sync-cancel");
-        inf_communication_group_send_message(sync->group, sync->conn, xml);
-      }
-
-      /* We had to cancel the synchronization, so the synchronization
-       * actually failed. */
-      g_signal_emit(
-        G_OBJECT(session),
-        session_signals[SYNCHRONIZATION_FAILED],
-        0,
-        sync->conn,
-        error
-      );
-
-      /* Note that the synchronization_failed default handler actualy
-       * removes the sync. */
+      inf_session_cancel_synchronization(session, sync->conn);
     }
 
     break;
@@ -1726,8 +1695,6 @@ inf_session_close_handler(InfSession* session)
     g_object_notify(G_OBJECT(session), "subscription-group");
   }
 
-  g_error_free(error);
-
   priv->status = INF_SESSION_CLOSED;
   g_object_notify(G_OBJECT(session), "status");
   g_object_thaw_notify(G_OBJECT(session));
@@ -1901,19 +1868,12 @@ inf_session_synchronization_failed_handler(InfSession* session,
  */
 
 static void
-inf_session_class_init(gpointer g_class,
-                       gpointer class_data)
+inf_session_class_init(InfSessionClass* session_class)
 {
   GObjectClass* object_class;
-  InfSessionClass* session_class;
+  object_class = G_OBJECT_CLASS(session_class);
 
-  object_class = G_OBJECT_CLASS(g_class);
-  session_class = INF_SESSION_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfSessionPrivate));
-
-  object_class->constructor = inf_session_constructor;
+  object_class->constructed = inf_session_constructed;
   object_class->dispose = inf_session_dispose;
   object_class->finalize = inf_session_finalize;
   object_class->set_property = inf_session_set_property;
@@ -1930,6 +1890,7 @@ inf_session_class_init(gpointer g_class,
   session_class->user_new = NULL;
 
   session_class->close = inf_session_close_handler;
+  session_class->error = NULL;
   session_class->synchronization_begin =
     inf_session_synchronization_begin_handler;
   session_class->synchronization_progress = NULL;
@@ -2042,11 +2003,39 @@ inf_session_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfSessionClass, close),
     NULL, NULL,
-    inf_marshal_VOID__VOID,
+    g_cclosure_marshal_VOID__VOID,
     G_TYPE_NONE,
     0
   );
 
+  /**
+   * InfSession::error:
+   * @session: The #InfSession having an error.
+   * @connection: The #InfXmlConnection which sent the erroneous request.
+   * @xml: The XML request that produced the error.
+   * @error: A #GError providing information about the error.
+   *
+   * This signal is emitted when the session detects an error. The session
+   * itself does not know much about the nature of the error. It might mean
+   * the session is in an inconsistent state, or it might be recoverable.
+   * This signal can be used to handle the error or to write error
+   * information into a log file or bring to a user's attention in another
+   * manner.
+   */
+  session_signals[ERROR] = g_signal_new(
+    "error",
+    G_OBJECT_CLASS_TYPE(object_class),
+    G_SIGNAL_RUN_LAST,
+    G_STRUCT_OFFSET(InfSessionClass, error),
+    NULL, NULL,
+    NULL,
+    G_TYPE_NONE,
+    3,
+    INF_TYPE_XML_CONNECTION,
+    G_TYPE_POINTER, /* actually a xmlNodePtr */
+    G_TYPE_ERROR
+  );
+
   /**
    * InfSession::synchronization-begin:
    * @session: The #InfSession that is synchronizing.
@@ -2061,14 +2050,14 @@ inf_session_class_init(gpointer g_class,
    * is emitted when @session is being synchronized itself, because that can
    * happen at construction time only when no one had a chance to connect
    * signal handlers anyway.
-   **/
+   */
   session_signals[SYNCHRONIZATION_BEGIN] = g_signal_new(
     "synchronization-begin",
     G_OBJECT_CLASS_TYPE(object_class),
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfSessionClass, synchronization_begin),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT_OBJECT,
+    NULL,
     G_TYPE_NONE,
     2,
     INF_COMMUNICATION_TYPE_GROUP,
@@ -2096,7 +2085,7 @@ inf_session_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfSessionClass, synchronization_progress),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT_DOUBLE,
+    NULL,
     G_TYPE_NONE,
     2,
     INF_TYPE_XML_CONNECTION,
@@ -2123,7 +2112,7 @@ inf_session_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfSessionClass, synchronization_complete),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT,
+    g_cclosure_marshal_VOID__OBJECT,
     G_TYPE_NONE,
     1,
     INF_TYPE_XML_CONNECTION
@@ -2147,116 +2136,31 @@ inf_session_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfSessionClass, synchronization_failed),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT_POINTER,
+    NULL,
     G_TYPE_NONE,
     2,
     INF_TYPE_XML_CONNECTION,
-    G_TYPE_POINTER /* actually a GError* */
+    G_TYPE_ERROR
   );
 }
 
 static void
-inf_session_communication_object_init(gpointer g_iface,
-                                      gpointer iface_data)
+inf_session_communication_object_iface_init(
+  InfCommunicationObjectInterface* iface)
 {
-  InfCommunicationObjectIface* iface;
-  iface = (InfCommunicationObjectIface*)g_iface;
-
   iface->sent = inf_session_communication_object_sent;
   iface->enqueued = inf_session_communication_object_enqueued;
   iface->received = inf_session_communication_object_received;
 }
 
-GType
-inf_session_status_get_type(void)
-{
-  static GType session_status_type = 0;
-
-  if(!session_status_type)
-  {
-    static const GEnumValue session_status_type_values[] = {
-      {
-        INF_SESSION_PRESYNC,
-        "INF_SESSION_PRESYNC",
-        "presync"
-      }, {
-        INF_SESSION_SYNCHRONIZING,
-        "INF_SESSION_SYNCHRONIZING",
-        "synchronizing"
-      }, {
-        INF_SESSION_RUNNING,
-        "INF_SESSION_RUNNING",
-        "running"
-      }, {
-        INF_SESSION_CLOSED,
-        "INF_SESSION_CLOSED",
-        "closed"
-      }, {
-        0,
-        NULL,
-        NULL
-      }
-    };
-
-    session_status_type = g_enum_register_static(
-      "InfSessionStatus",
-      session_status_type_values
-    );
-  }
-
-  return session_status_type;
-}
-
-GType
-inf_session_get_type(void)
-{
-  static GType session_type = 0;
-
-  if(!session_type)
-  {
-    static const GTypeInfo session_type_info = {
-      sizeof(InfSessionClass),  /* class_size */
-      NULL,                     /* base_init */
-      NULL,                     /* base_finalize */
-      inf_session_class_init,   /* class_init */
-      NULL,                     /* class_finalize */
-      NULL,                     /* class_data */
-      sizeof(InfSession),       /* instance_size */
-      0,                        /* n_preallocs */
-      inf_session_init,         /* instance_init */
-      NULL                      /* value_table */
-    };
-
-    static const GInterfaceInfo communication_object_info = {
-      inf_session_communication_object_init,
-      NULL,
-      NULL
-    };
-
-    session_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfSession",
-      &session_type_info,
-      0
-    );
-
-    g_type_add_interface_static(
-      session_type,
-      INF_COMMUNICATION_TYPE_OBJECT,
-      &communication_object_info
-    );
-  }
-
-  return session_type;
-}
-
 /*
  * Public API.
  */
 
 /**
  * inf_session_lookup_user_property:
- * @params: A pointer to an array of containing #GParameter values.
+ * @params: (array length=n_params): A pointer to an array of containing
+ * #GParameter values.
  * @n_params: The number of elements in the aforementioned array
  * @name: Name to look up.
  *
@@ -2283,13 +2187,13 @@ inf_session_lookup_user_property(const GParameter* params,
 
 /**
  * inf_session_get_user_property:
- * @array: A #GArray containing #GParameter values.
+ * @array: (element-type GParameter): A #GArray containing #GParameter values.
  * @name: Name to look up.
  *
  * Looks up the paremeter with the given name in @array. If there is no such
  * parameter, a new one will be created.
  *
- * Return Value: A #GParameter.
+ * Return Value: (transfer none): A #GParameter.
  **/
 GParameter*
 inf_session_get_user_property(GArray* array,
@@ -2389,7 +2293,7 @@ inf_session_close(InfSession* session)
  *
  * Returns the communication manager for @session.
  *
- * Return Value: A #InfCommunicationManager.
+ * Return Value: (transfer none): A #InfCommunicationManager.
  **/
 InfCommunicationManager*
 inf_session_get_communication_manager(InfSession* session)
@@ -2404,7 +2308,7 @@ inf_session_get_communication_manager(InfSession* session)
  *
  * Returns the buffer used by @session.
  *
- * Return Value: A #InfBuffer.
+ * Return Value: (transfer none): A #InfBuffer.
  **/
 InfBuffer*
 inf_session_get_buffer(InfSession* session)
@@ -2419,7 +2323,7 @@ inf_session_get_buffer(InfSession* session)
  *
  * Returns the user table used by @session.
  *
- * Return Value: A #InfUserTable.
+ * Return Value: (transfer none): A #InfUserTable.
  **/
 InfUserTable*
 inf_session_get_user_table(InfSession* session)
@@ -2446,9 +2350,9 @@ inf_session_get_status(InfSession* session)
 /**
  * inf_session_add_user:
  * @session: A #InfSession.
- * @params: Construction parameters for the #InfUser (or derived) object.
+ * @params: (array length=n_params): Construction parameters for the #InfUser
+ * (or derived) object.
  * @n_params: Number of parameters.
- * @error: Location to store error information.
  *
  * Adds a user to @session. The user object is constructed via the
  * user_new vfunc of #InfSessionClass. This will create a new #InfUser
@@ -2461,14 +2365,15 @@ inf_session_get_status(InfSession* session)
  * by #InfdSessionProxy or #InfcSessionProxy, respectively.
  *
  * You should not call this function unless you know what you are doing.
+ * If @params comes from an untrusted source, they should be checked first
+ * with the validate_user_props virtual function.
  *
- * Return Value: The new #InfUser, or %NULL in case of an error.
+ * Returns: (transfer none): The new #InfUser.
  **/
 InfUser*
 inf_session_add_user(InfSession* session,
                      const GParameter* params,
-                     guint n_params,
-                     GError** error)
+                     guint n_params)
 {
   InfSessionPrivate* priv;
   InfSessionClass* session_class;
@@ -2483,27 +2388,22 @@ inf_session_add_user(InfSession* session,
 
   priv = INF_SESSION_PRIVATE(session);
 
-  result = session_class->validate_user_props(
-    session,
-    params,
-    n_params,
-    NULL,
-    error
+  g_return_val_if_fail(
+    session_class->validate_user_props(session, params, n_params, NULL, NULL),
+    NULL
   );
 
-  if(result == TRUE)
-  {
-    /* No idea why g_object_newv wants unconst GParameter list */
-    user = session_class->user_new(session,
-                                   *(GParameter**)(gpointer)&params,
-                                   n_params);
-    inf_user_table_add_user(priv->user_table, user);
-    g_object_unref(user); /* We rely on the usertable holding a reference */
+  /* No idea why g_object_newv wants unconst GParameter list */
+  user = session_class->user_new(
+    session,
+    *(GParameter**)(gpointer)&params,
+    n_params
+  );
 
-    return user;
-  }
+  inf_user_table_add_user(priv->user_table, user);
+  g_object_unref(user); /* We rely on the usertable holding a reference */
 
-  return NULL;
+  return user;
 }
 
 /**
@@ -2515,8 +2415,7 @@ inf_session_add_user(InfSession* session,
  * Changes the status of the given @user which needs to have the
  * %INF_USER_LOCAL flag set for this function to be called. If the status
  * is changed to %INF_USER_UNAVAILABLE, then the user leaves the session. To
- * rejoin use infc_session_proxy_join_user() or infd_session_proxy_add_user(),
- * respectively for a proxy proxying @session.
+ * rejoin use inf_session_proxy_join_user() for a proxy proxying @session.
  **/
 void
 inf_session_set_user_status(InfSession* session,
@@ -2598,12 +2497,12 @@ inf_session_synchronize_from(InfSession* session)
  * inf_session_synchronize_to:
  * @session: A #InfSession in status %INF_SESSION_RUNNING.
  * @group: A #InfCommunicationGroup.
- * @connection: A #InfConnection.
+ * @connection: A #InfXmlConnection.
  *
  * Initiates a synchronization to @connection. On the other end of
  * @connection, a new session with the sync-connection and sync-group
  * construction properties set should have been created. @group is used
- * as a group in the connection manager. It is allowed for @group to have
+ * as a group in the communication manager. It is allowed for @group to have
  * another #InfCommunicationObject than @session, however, you should forward
  * the #InfCommunicationObject messages your object receives to @session then.
  * Also, @connection must already be present in @group, and should not be
@@ -2635,6 +2534,109 @@ inf_session_synchronize_to(InfSession* session,
   );
 }
 
+/**
+ * inf_session_cancel_synchronization:
+ * @session: A #InfSession.
+ * @connection: The #InfXmlConnection with which to cancel synchronization.
+ *
+ * Cancells an ongaing synchronization to or from @connection. If @session
+ * is in state %INF_SESSION_PRESYNC or %INF_SESSION_SYNCHRONIZING,
+ * @connection must match the connection that @session is synchronizing with.
+ * If @session is in state %INF_SESSION_RUNNING, @connection can be any
+ * connection that the session is currently being synchronized to.
+ *
+ * In any case, the #InfSession::synchronization-failed signal will be
+ * emitted for the cancelled synchronization. If the session is in state
+ * %INF_SESSION_PRESYNC or %INF_SESSION_SYNCHRONIZING, the session will also
+ * be closed, with the #InfSession::close signal being emited.
+ */
+void
+inf_session_cancel_synchronization(InfSession* session,
+                                   InfXmlConnection* connection)
+{
+  InfSessionPrivate* priv;
+  InfXmlConnectionStatus status;
+  InfSessionSync* sync;
+  xmlNodePtr xml;
+  GError* error;
+
+  g_return_if_fail(INF_IS_SESSION(session));
+  g_return_if_fail(INF_IS_XML_CONNECTION(connection));
+
+  priv = INF_SESSION_PRIVATE(session);
+  error = NULL;
+
+  switch(priv->status)
+  {
+  case INF_SESSION_PRESYNC:
+    g_return_if_fail(connection == priv->shared.presync.conn);
+
+    g_set_error_literal(
+      &error,
+      inf_session_sync_error_quark,
+      INF_SESSION_SYNC_ERROR_RECEIVER_CANCELLED,
+      inf_session_sync_strerror(INF_SESSION_SYNC_ERROR_RECEIVER_CANCELLED)
+    );
+
+    break;
+  case INF_SESSION_SYNCHRONIZING:
+    g_return_if_fail(connection == priv->shared.sync.conn);
+
+    g_set_error_literal(
+      &error,
+      inf_session_sync_error_quark,
+      INF_SESSION_SYNC_ERROR_RECEIVER_CANCELLED,
+      inf_session_sync_strerror(INF_SESSION_SYNC_ERROR_RECEIVER_CANCELLED)
+    );
+
+    g_object_get(G_OBJECT(connection), "status", &status, NULL);
+    if(status == INF_XML_CONNECTION_OPEN)
+      inf_session_send_sync_error(session, error);
+
+    break;
+  case INF_SESSION_RUNNING:
+    sync = inf_session_find_sync_by_connection(session, connection);
+    g_return_if_fail(sync != NULL);
+
+    /* If the sync-end message has already been enqueued within the
+     * connection manager, we cannot cancel it anymore, so the remote
+     * site will receive the full sync nevertheless, so we do not need
+     * to cancel anything. */
+    if(sync->status == INF_SESSION_SYNC_IN_PROGRESS)
+    {
+      inf_communication_group_cancel_messages(sync->group, sync->conn);
+
+      xml = xmlNewNode(NULL, (const xmlChar*)"sync-cancel");
+      inf_communication_group_send_message(sync->group, sync->conn, xml);
+    }
+
+    g_set_error_literal(
+      &error,
+      inf_session_sync_error_quark,
+      INF_SESSION_SYNC_ERROR_SENDER_CANCELLED,
+      inf_session_sync_strerror(INF_SESSION_SYNC_ERROR_SENDER_CANCELLED)
+    );
+
+    break;
+  case INF_SESSION_CLOSED:
+    g_return_if_reached();
+    break;
+  default:
+    g_assert_not_reached();
+    break;
+  }
+
+  g_signal_emit(
+    G_OBJECT(session),
+    session_signals[SYNCHRONIZATION_FAILED],
+    0,
+    connection,
+    error
+  );
+
+  g_error_free(error);
+}
+
 /**
  * inf_session_get_synchronization_status:
  * @session: A #InfSession.
@@ -2805,7 +2807,7 @@ inf_session_has_synchronizations(InfSession* session)
  *
  * Returns the subscription group for @session, if any.
  *
- * Return Value: A #InfCommunicationGroup, or %NULL.
+ * Return Value: (transfer none): A #InfCommunicationGroup, or %NULL.
  **/
 InfCommunicationGroup*
 inf_session_get_subscription_group(InfSession* session)
@@ -2854,7 +2856,7 @@ inf_session_set_subscription_group(InfSession* session,
 /**
  * inf_session_send_to_subscriptions:
  * @session: A #InfSession.
- * @xml: The message to send.
+ * @xml: (transfer full): The message to send.
  *
  * Sends a XML message to the all members of @session's subscription group.
  * This function can only be called if the subscription group is non-%NULL. It
