@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,10 +17,6 @@
  * MA 02110-1301, USA.
  */
 
-#include <libinfinity/common/inf-simulated-connection.h>
-#include <libinfinity/common/inf-xml-connection.h>
-#include <libinfinity/inf-marshal.h>
-
 /**
  * SECTION:inf-simulated-connection
  * @title: InfSimulatedConnection
@@ -34,6 +30,30 @@
  * data sent through one is received by the other.
  */
 
+#include <libinfinity/common/inf-simulated-connection.h>
+#include <libinfinity/common/inf-xml-connection.h>
+#include <libinfinity/inf-define-enum.h>
+
+static const GEnumValue inf_simulated_connection_mode_values[] = {
+  {
+    INF_SIMULATED_CONNECTION_IMMEDIATE,
+    "INF_SIMULATED_CONNECTION_IMMEDIATE",
+    "immediate"
+  }, {
+    INF_SIMULATED_CONNECTION_DELAYED,
+    "INF_SIMULATED_CONNECTION_DELAYED",
+    "delayed"
+  }, {
+    INF_SIMULATED_CONNECTION_IO_CONTROLLED,
+    "INF_SIMULATED_CONNECTION_IO_CONTROLLED",
+    "io-controlled"
+  }, {
+    0,
+    NULL,
+    NULL
+  }
+};
+
 typedef struct _InfSimulatedConnectionPrivate InfSimulatedConnectionPrivate;
 struct _InfSimulatedConnectionPrivate {
   InfIo* io;
@@ -58,12 +78,18 @@ enum {
   PROP_STATUS,
   PROP_NETWORK,
   PROP_LOCAL_ID,
-  PROP_REMOTE_ID
+  PROP_REMOTE_ID,
+  PROP_LOCAL_CERTIFICATE,
+  PROP_REMOTE_CERTIFICATE
 };
 
 #define INF_SIMULATED_CONNECTION_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_TYPE_SIMULATED_CONNECTION, InfSimulatedConnectionPrivate))
 
-static GObjectClass* parent_class;
+static void inf_simulated_connection_xml_connection_iface_init(InfXmlConnectionInterface* iface);
+INF_DEFINE_ENUM_TYPE(InfSimulatedConnectionMode, inf_simulated_connection_mode, inf_simulated_connection_mode_values)
+G_DEFINE_TYPE_WITH_CODE(InfSimulatedConnection, inf_simulated_connection, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfSimulatedConnection)
+  G_IMPLEMENT_INTERFACE(INF_TYPE_XML_CONNECTION, inf_simulated_connection_xml_connection_iface_init))
 
 static void
 inf_simulated_connection_clear_queue(InfSimulatedConnection* connection)
@@ -152,13 +178,9 @@ inf_simulated_connection_set_target(InfSimulatedConnection* connection,
  */
 
 static void
-inf_simulated_connection_init(GTypeInstance* instance,
-                              gpointer g_class)
+inf_simulated_connection_init(InfSimulatedConnection* connection)
 {
-  InfSimulatedConnection* connection;
   InfSimulatedConnectionPrivate* priv;
-
-  connection = INF_SIMULATED_CONNECTION(instance);
   priv = INF_SIMULATED_CONNECTION_PRIVATE(connection);
 
   priv->io = NULL;
@@ -185,7 +207,7 @@ inf_simulated_connection_dispose(GObject* object)
     priv->io = NULL;
   }
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_simulated_connection_parent_class)->dispose(object);
 }
 
 static void
@@ -205,7 +227,7 @@ inf_simulated_connection_set_property(GObject* object,
   case PROP_IO:
     g_assert(priv->io == NULL); /* construct only */
     priv->io = INF_IO(g_value_get_object(value));
-    g_object_ref(priv->io);
+    if(priv->io) g_object_ref(priv->io);
     break;
   case PROP_TARGET:
     inf_simulated_connection_set_target(
@@ -266,6 +288,12 @@ inf_simulated_connection_get_property(GObject* object,
     id = g_strdup_printf("simulated-%p", (void*)priv->target);
     g_value_take_string(value, id);
     break;
+  case PROP_LOCAL_CERTIFICATE:
+    g_value_set_pointer(value, NULL);
+    break;
+  case PROP_REMOTE_CERTIFICATE:
+    g_value_set_boxed(value, NULL);
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
     break;
@@ -356,14 +384,11 @@ inf_simulated_connection_xml_connection_send(InfXmlConnection* connection,
  */
 
 static void
-inf_simulated_connection_class_init(gpointer g_class,
-                                    gpointer class_data)
+inf_simulated_connection_class_init(
+  InfSimulatedConnectionClass* connection_class)
 {
   GObjectClass* object_class;
-  object_class = G_OBJECT_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfSimulatedConnectionPrivate));
+  object_class = G_OBJECT_CLASS(connection_class);
 
   object_class->dispose = inf_simulated_connection_dispose;
   object_class->set_property = inf_simulated_connection_set_property;
@@ -410,110 +435,40 @@ inf_simulated_connection_class_init(gpointer g_class,
   g_object_class_override_property(object_class, PROP_NETWORK, "network");
   g_object_class_override_property(object_class, PROP_LOCAL_ID, "local-id");
   g_object_class_override_property(object_class, PROP_REMOTE_ID, "remote-id");
+
+  g_object_class_override_property(
+    object_class,
+    PROP_LOCAL_CERTIFICATE,
+    "local-certificate"
+  );
+
+  g_object_class_override_property(
+    object_class,
+    PROP_REMOTE_CERTIFICATE,
+    "remote-certificate"
+  );
 }
 
 static void
-inf_simulated_connection_xml_connection_init(gpointer g_iface,
-                                             gpointer iface_data)
+inf_simulated_connection_xml_connection_iface_init(
+  InfXmlConnectionInterface* iface)
 {
-  InfXmlConnectionIface* iface;
-  iface = (InfXmlConnectionIface*)g_iface;
-
   iface->close = inf_simulated_connection_xml_connection_close;
   iface->send = inf_simulated_connection_xml_connection_send;
 }
 
-GType
-inf_simulated_connection_mode_get_type(void)
-{
-  static GType mode_type = 0;
-
-  if(!mode_type)
-  {
-    static const GEnumValue mode_type_values[] = {
-      {
-        INF_SIMULATED_CONNECTION_IMMEDIATE,
-        "INF_SIMULATED_CONNECTION_IMMEDIATE",
-        "immediate"
-      }, {
-        INF_SIMULATED_CONNECTION_DELAYED,
-        "INF_SIMULATED_CONNECTION_DELAYED",
-        "delayed"
-      }, {
-        INF_SIMULATED_CONNECTION_IO_CONTROLLED,
-        "INF_SIMULATED_CONNECTION_IO_CONTROLLED",
-        "io-controlled"
-      }, {
-        0,
-        NULL,
-        NULL
-      }
-    };
-
-    mode_type = g_enum_register_static(
-      "InfSimulatedConnectionMode",
-      mode_type_values
-    );
-  }
-
-  return mode_type;
-}
-
-GType
-inf_simulated_connection_get_type(void)
-{
-  static GType simulated_connection_type = 0;
-
-  if(!simulated_connection_type)
-  {
-    static const GTypeInfo simulated_connection_type_info = {
-      sizeof(InfSimulatedConnectionClass),   /* class_size */
-      NULL,                                  /* base_init */
-      NULL,                                  /* base_finalize */
-      inf_simulated_connection_class_init,   /* class_init */
-      NULL,                                  /* class_finalize */
-      NULL,                                  /* class_data */
-      sizeof(InfSimulatedConnection),        /* instance_size */
-      0,                                     /* n_preallocs */
-      inf_simulated_connection_init,         /* instance_init */
-      NULL                                   /* value_table */
-    };
-
-    static const GInterfaceInfo xml_connection_info = {
-      inf_simulated_connection_xml_connection_init,
-      NULL,
-      NULL
-    };
-
-    simulated_connection_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfSimulatedConnection",
-      &simulated_connection_type_info,
-      0
-    );
-
-    g_type_add_interface_static(
-      simulated_connection_type,
-      INF_TYPE_XML_CONNECTION,
-      &xml_connection_info
-    );
-  }
-
-  return simulated_connection_type;
-}
-
 /*
  * Public API
  */
 
 /**
- * inf_simulated_connection_new:
+ * inf_simulated_connection_new: (constructor)
  *
  * Creates a new #InfSimulatedConnection. A connection created this way cannot
  * be switched to %INF_SIMULATED_CONNECTION_IO_CONTROLLED mode. Use
  * inf_simulated_connection_new_with_io() instead if you intend to do that.
  *
- * Returns: A new #InfSimulatedConnection.
+ * Returns: (transfer full): A new #InfSimulatedConnection.
  **/
 InfSimulatedConnection*
 inf_simulated_connection_new(void)
@@ -524,7 +479,7 @@ inf_simulated_connection_new(void)
 }
 
 /**
- * inf_simulated_connection_new_with_io:
+ * inf_simulated_connection_new_with_io: (constructor)
  * @io: The main loop to be used for %INF_SIMULATED_CONNECTION_IO_CONTROLLED
  * mode.
  *
@@ -534,7 +489,7 @@ inf_simulated_connection_new(void)
  * inf_simulated_connection_new() is also good enough since the #InfIo object
  * is not required in that case.
  *
- * Returns: A new #InfSimulatedConnection.
+ * Returns: (transfer full): A new #InfSimulatedConnection.
  */
 InfSimulatedConnection*
 inf_simulated_connection_new_with_io(InfIo* io)
