@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -109,7 +109,7 @@ struct _InfIoDispatch {
 typedef struct _InfStandaloneIoPrivate InfStandaloneIoPrivate;
 struct _InfStandaloneIoPrivate {
   InfStandaloneIoNativeEvent* events;
-  GMutex* mutex;
+  GMutex mutex;
 
   guint fd_size;
   guint fd_alloc;
@@ -150,7 +150,10 @@ static const InfStandaloneIoEventTableEntry inf_standalone_io_event_table[] =
 
 #define INF_STANDALONE_IO_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_TYPE_STANDALONE_IO, InfStandaloneIoPrivate))
 
-static GObjectClass* parent_class;
+static void inf_standalone_io_io_iface_init(InfIoInterface* iface);
+G_DEFINE_TYPE_WITH_CODE(InfStandaloneIo, inf_standalone_io, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfStandaloneIo)
+  G_IMPLEMENT_INTERFACE(INF_TYPE_IO, inf_standalone_io_io_iface_init))
 
 static guint
 inf_standalone_io_timeval_diff(GTimeVal* first,
@@ -228,11 +231,11 @@ inf_standalone_io_iteration_impl(InfStandaloneIo* io,
   }
 
   priv->polling = TRUE;
-  g_mutex_unlock(priv->mutex);
+  g_mutex_unlock(&priv->mutex);
 
   result = inf_standalone_io_poll(priv->events, priv->fd_size, timeout);
 
-  g_mutex_lock(priv->mutex);
+  g_mutex_lock(&priv->mutex);
   priv->polling = FALSE;
 
 #ifdef G_OS_WIN32
@@ -269,14 +272,14 @@ inf_standalone_io_iteration_impl(InfStandaloneIo* io,
       if(elapsed >= cur_timeout->msecs)
       {
         priv->timeouts = g_list_delete_link(priv->timeouts, item);
-        g_mutex_unlock(priv->mutex);
+        g_mutex_unlock(&priv->mutex);
 
         cur_timeout->func(cur_timeout->user_data);
         if(cur_timeout->notify)
           cur_timeout->notify(cur_timeout->user_data);
         g_slice_free(InfIoTimeout, cur_timeout);
 
-        g_mutex_lock(priv->mutex);
+        g_mutex_lock(&priv->mutex);
         return;
       }
     }
@@ -321,18 +324,18 @@ inf_standalone_io_iteration_impl(InfStandaloneIo* io,
       /* protect from removing the watch object via
        * inf_io_remove_watch() when running the callback. */
       watch->executing = TRUE;
-      g_mutex_unlock(priv->mutex);
+      g_mutex_unlock(&priv->mutex);
 
       watch->func(watch->socket, events, watch->user_data);
 
-      g_mutex_lock(priv->mutex);
+      g_mutex_lock(&priv->mutex);
       watch->executing = FALSE;
       if(watch->disposed == TRUE)
       {
-        g_mutex_unlock(priv->mutex);
+        g_mutex_unlock(&priv->mutex);
         if(watch->notify) watch->notify(watch->user_data);
         g_slice_free(InfIoWatch, watch);
-        g_mutex_lock(priv->mutex);
+        g_mutex_lock(&priv->mutex);
       }
 
       return;
@@ -405,18 +408,18 @@ inf_standalone_io_iteration_impl(InfStandaloneIo* io,
             /* protect from removing the watch object via
              * inf_io_remove_watch() when running the callback. */
             watch->executing = TRUE;
-            g_mutex_unlock(priv->mutex);
+            g_mutex_unlock(&priv->mutex);
 
             watch->func(watch->socket, events, watch->user_data);
 
-            g_mutex_lock(priv->mutex);
+            g_mutex_lock(&priv->mutex);
             watch->executing = FALSE;
             if(watch->disposed == TRUE)
             {
-              g_mutex_unlock(priv->mutex);
+              g_mutex_unlock(&priv->mutex);
               if(watch->notify) watch->notify(watch->user_data);
               g_slice_free(InfIoWatch, watch);
-              g_mutex_lock(priv->mutex);
+              g_mutex_lock(&priv->mutex);
             }
 
             return;
@@ -432,32 +435,29 @@ inf_standalone_io_iteration_impl(InfStandaloneIo* io,
   {
     dispatch = (InfIoDispatch*)priv->dispatchs->data;
     priv->dispatchs = g_list_delete_link(priv->dispatchs, priv->dispatchs);
-    g_mutex_unlock(priv->mutex);
+    g_mutex_unlock(&priv->mutex);
 
     dispatch->func(dispatch->user_data);
     if(dispatch->notify)
       dispatch->notify(dispatch->user_data);
     g_slice_free(InfIoDispatch, dispatch);
 
-    g_mutex_lock(priv->mutex);
+    g_mutex_lock(&priv->mutex);
   }
 }
 
 static void
-inf_standalone_io_init(GTypeInstance* instance,
-                       gpointer g_class)
+inf_standalone_io_init(InfStandaloneIo* io)
 {
-  InfStandaloneIo* io;
   InfStandaloneIoPrivate* priv;
 
 #ifdef G_OS_WIN32
   gchar* error_message;
 #endif
 
-  io = INF_STANDALONE_IO(instance);
   priv = INF_STANDALONE_IO_PRIVATE(io);
 
-  priv->mutex = g_mutex_new();
+  g_mutex_init(&priv->mutex);
 
   priv->fd_size = 0;
   priv->fd_alloc = 4;
@@ -516,7 +516,7 @@ inf_standalone_io_finalize(GObject* object)
   io = INF_STANDALONE_IO(object);
   priv = INF_STANDALONE_IO_PRIVATE(io);
 
-  g_mutex_lock(priv->mutex);
+  g_mutex_lock(&priv->mutex);
 
   for(i = 1; i < priv->fd_size; ++i)
   {
@@ -592,10 +592,10 @@ inf_standalone_io_finalize(GObject* object)
   }
 #endif
 
-  g_mutex_unlock(priv->mutex);
-  g_mutex_free(priv->mutex);
+  g_mutex_unlock(&priv->mutex);
+  g_mutex_clear(&priv->mutex);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(inf_standalone_io_parent_class)->finalize(object);
 }
 
 static InfIoWatch**
@@ -724,14 +724,14 @@ inf_standalone_io_io_add_watch(InfIo* io,
     pevents |= (POLLERR | POLLHUP | POLLNVAL | POLLPRI);
 #endif
 
-  g_mutex_lock(priv->mutex);
+  g_mutex_lock(&priv->mutex);
 
   /* Watching the same socket for different events at least won't work on
    * Windows since WSAEventSelect cancels the effect of previous
    * WSAEventSelect calls for the same socket. */
   if(inf_standalone_io_find_watch_by_socket(INF_STANDALONE_IO(io), socket))
   {
-    g_mutex_unlock(priv->mutex);
+    g_mutex_unlock(&priv->mutex);
     return NULL;
   }
 
@@ -767,7 +767,7 @@ inf_standalone_io_io_add_watch(InfIo* io,
     g_warning("WSACreateEvent() failed: %s", error_message);
     g_free(error_message);
 
-    g_mutex_unlock(priv->mutex);
+    g_mutex_unlock(&priv->mutex);
     return NULL;
   }
 
@@ -779,7 +779,7 @@ inf_standalone_io_io_add_watch(InfIo* io,
     g_free(error_message);
 
     WSACloseEvent(priv->events[priv->fd_size]);
-    g_mutex_unlock(priv->mutex);
+    g_mutex_unlock(&priv->mutex);
     return NULL;
   }
 #else
@@ -801,7 +801,7 @@ inf_standalone_io_io_add_watch(InfIo* io,
   ++priv->fd_size;
 
   inf_standalone_io_wakeup(INF_STANDALONE_IO(io));
-  g_mutex_unlock(priv->mutex);
+  g_mutex_unlock(&priv->mutex);
 
   return watch;
 }
@@ -837,7 +837,7 @@ inf_standalone_io_io_update_watch(InfIo* io,
     pevents |= (POLLERR | POLLHUP | POLLNVAL | POLLPRI);
 #endif
 
-  g_mutex_lock(priv->mutex);
+  g_mutex_lock(&priv->mutex);
 
   watch_iter = inf_standalone_io_find_watch(INF_STANDALONE_IO(io), watch);
   if(watch_iter != NULL)
@@ -860,7 +860,7 @@ inf_standalone_io_io_update_watch(InfIo* io,
     inf_standalone_io_wakeup(INF_STANDALONE_IO(io));
   }
 
-  g_mutex_unlock(priv->mutex);
+  g_mutex_unlock(&priv->mutex);
 }
 
 static void
@@ -877,7 +877,7 @@ inf_standalone_io_io_remove_watch(InfIo* io,
 
   priv = INF_STANDALONE_IO_PRIVATE(io);
 
-  g_mutex_lock(priv->mutex);
+  g_mutex_lock(&priv->mutex);
 
   watch_iter = inf_standalone_io_find_watch(INF_STANDALONE_IO(io), watch);
   if(watch_iter != NULL)
@@ -940,7 +940,7 @@ inf_standalone_io_io_remove_watch(InfIo* io,
     inf_standalone_io_wakeup(INF_STANDALONE_IO(io));
   }
 
-  g_mutex_unlock(priv->mutex);
+  g_mutex_unlock(&priv->mutex);
 }
 
 static InfIoTimeout*
@@ -962,10 +962,10 @@ inf_standalone_io_io_add_timeout(InfIo* io,
   timeout->user_data = user_data;
   timeout->notify = notify;
 
-  g_mutex_lock(priv->mutex);
+  g_mutex_lock(&priv->mutex);
   priv->timeouts = g_list_prepend(priv->timeouts, timeout);
   inf_standalone_io_wakeup(INF_STANDALONE_IO(io));
-  g_mutex_unlock(priv->mutex);
+  g_mutex_unlock(&priv->mutex);
 
   return timeout;
 }
@@ -979,13 +979,13 @@ inf_standalone_io_io_remove_timeout(InfIo* io,
 
   priv = INF_STANDALONE_IO_PRIVATE(io);
 
-  g_mutex_lock(priv->mutex);
+  g_mutex_lock(&priv->mutex);
 
   item = g_list_find(priv->timeouts, timeout);
   if(item != NULL)
   {
     priv->timeouts = g_list_delete_link(priv->timeouts, item);
-    g_mutex_unlock(priv->mutex);
+    g_mutex_unlock(&priv->mutex);
 
     if(timeout->notify)
       timeout->notify(timeout->user_data);
@@ -997,7 +997,7 @@ inf_standalone_io_io_remove_timeout(InfIo* io,
   }
   else
   {
-    g_mutex_unlock(priv->mutex);
+    g_mutex_unlock(&priv->mutex);
   }
 }
 
@@ -1017,10 +1017,10 @@ inf_standalone_io_io_add_dispatch(InfIo* io,
   dispatch->user_data = user_data;
   dispatch->notify = notify;
 
-  g_mutex_lock(priv->mutex);
+  g_mutex_lock(&priv->mutex);
   priv->dispatchs = g_list_prepend(priv->dispatchs, dispatch);
   inf_standalone_io_wakeup(INF_STANDALONE_IO(io));
-  g_mutex_unlock(priv->mutex);
+  g_mutex_unlock(&priv->mutex);
 
   return dispatch;
 }
@@ -1034,13 +1034,13 @@ inf_standalone_io_io_remove_dispatch(InfIo* io,
 
   priv = INF_STANDALONE_IO_PRIVATE(io);
 
-  g_mutex_lock(priv->mutex);
+  g_mutex_lock(&priv->mutex);
 
   item = g_list_find(priv->dispatchs, dispatch);
   if(item != NULL)
   {
     priv->dispatchs = g_list_delete_link(priv->dispatchs, item);
-    g_mutex_unlock(priv->mutex);
+    g_mutex_unlock(&priv->mutex);
 
     if(dispatch->notify)
       dispatch->notify(dispatch->user_data);
@@ -1052,30 +1052,22 @@ inf_standalone_io_io_remove_dispatch(InfIo* io,
   }
   else
   {
-    g_mutex_unlock(priv->mutex);
+    g_mutex_unlock(&priv->mutex);
   }
 }
 
 static void
-inf_standalone_io_class_init(gpointer g_class,
-                             gpointer class_data)
+inf_standalone_io_class_init(InfStandaloneIoClass* io_class)
 {
   GObjectClass* object_class;
-  object_class = G_OBJECT_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfStandaloneIoPrivate));
+  object_class = G_OBJECT_CLASS(io_class);
 
   object_class->finalize = inf_standalone_io_finalize;
 }
 
 static void
-inf_standalone_io_io_init(gpointer g_iface,
-                          gpointer iface_data)
+inf_standalone_io_io_iface_init(InfIoInterface* iface)
 {
-  InfIoIface* iface;
-  iface = (InfIoIface*)g_iface;
-
   iface->add_watch = inf_standalone_io_io_add_watch;
   iface->update_watch = inf_standalone_io_io_update_watch;
   iface->remove_watch = inf_standalone_io_io_remove_watch;
@@ -1085,56 +1077,13 @@ inf_standalone_io_io_init(gpointer g_iface,
   iface->remove_dispatch = inf_standalone_io_io_remove_dispatch;
 }
 
-GType
-inf_standalone_io_get_type(void)
-{
-  static GType standalone_io_type = 0;
-
-  if(!standalone_io_type)
-  {
-    static const GTypeInfo standalone_io_type_info = {
-      sizeof(InfStandaloneIoClass),   /* class_size */
-      NULL,                           /* base_init */
-      NULL,                           /* base_finalize */
-      inf_standalone_io_class_init,   /* class_init */
-      NULL,                           /* class_finalize */
-      NULL,                           /* class_data */
-      sizeof(InfStandaloneIo),        /* instance_size */
-      0,                              /* n_preallocs */
-      inf_standalone_io_init,         /* instance_init */
-      NULL                            /* value_table */
-    };
-
-    static const GInterfaceInfo io_info = {
-      inf_standalone_io_io_init,
-      NULL,
-      NULL
-    };
-
-    standalone_io_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfStandaloneIo",
-      &standalone_io_type_info,
-      0
-    );
-
-    g_type_add_interface_static(
-      standalone_io_type,
-      INF_TYPE_IO,
-      &io_info
-    );
-  }
-
-  return standalone_io_type;
-}
-
 /**
- * inf_standalone_io_new:
+ * inf_standalone_io_new: (constructor)
  *
  * Creates a new #InfStandaloneIo.
  *
- * Returns: A new #InfStandaloneIo. Free with g_object_unref() when no longer
- * needed.
+ * Returns: (transfer full): A new #InfStandaloneIo. Free with
+ * g_object_unref() when no longer needed.
  **/
 InfStandaloneIo*
 inf_standalone_io_new(void)
@@ -1160,13 +1109,13 @@ inf_standalone_io_iteration(InfStandaloneIo* io)
   priv = INF_STANDALONE_IO_PRIVATE(io);
 
   g_object_ref(io);
-  g_mutex_lock(priv->mutex);
+  g_mutex_lock(&priv->mutex);
 
-  g_return_val_if_fail(priv->polling == FALSE, g_mutex_unlock(priv->mutex));
+  g_return_val_if_fail(priv->polling == FALSE, g_mutex_unlock(&priv->mutex));
 
   inf_standalone_io_iteration_impl(io, INF_STANDALONE_IO_POLL_INFINITE);
 
-  g_mutex_unlock(priv->mutex);
+  g_mutex_unlock(&priv->mutex);
   g_object_unref(io);
 }
 
@@ -1188,13 +1137,13 @@ inf_standalone_io_iteration_timeout(InfStandaloneIo* io,
   priv = INF_STANDALONE_IO_PRIVATE(io);
 
   g_object_ref(io);
-  g_mutex_lock(priv->mutex);
+  g_mutex_lock(&priv->mutex);
 
-  g_return_val_if_fail(priv->polling == FALSE, g_mutex_unlock(priv->mutex));
+  g_return_val_if_fail(priv->polling == FALSE, g_mutex_unlock(&priv->mutex));
 
   inf_standalone_io_iteration_impl(io, (int)timeout);
 
-  g_mutex_unlock(priv->mutex);
+  g_mutex_unlock(&priv->mutex);
   g_object_unref(io);
 }
 
@@ -1214,16 +1163,16 @@ inf_standalone_io_loop(InfStandaloneIo* io)
   priv = INF_STANDALONE_IO_PRIVATE(io);
 
   g_object_ref(io);
-  g_mutex_lock(priv->mutex);
+  g_mutex_lock(&priv->mutex);
 
   g_return_val_if_fail(
     priv->loop_running == FALSE,
-    g_mutex_unlock(priv->mutex)
+    g_mutex_unlock(&priv->mutex)
   );
 
   g_return_val_if_fail(
     priv->polling == FALSE,
-    g_mutex_unlock(priv->mutex)
+    g_mutex_unlock(&priv->mutex)
   );
 
   /* TODO: Actually we need to make sure that a previous loop() call in
@@ -1234,7 +1183,7 @@ inf_standalone_io_loop(InfStandaloneIo* io)
   while(priv->loop_running == TRUE)
     inf_standalone_io_iteration_impl(io, -1);
 
-  g_mutex_unlock(priv->mutex);
+  g_mutex_unlock(&priv->mutex);
   g_object_unref(io);
 }
 
@@ -1253,13 +1202,13 @@ inf_standalone_io_loop_quit(InfStandaloneIo* io)
   g_return_if_fail(INF_IS_STANDALONE_IO(io));
   priv = INF_STANDALONE_IO_PRIVATE(io);
 
-  g_mutex_lock(priv->mutex);
+  g_mutex_lock(&priv->mutex);
 
   g_return_if_fail(priv->loop_running == TRUE);
   priv->loop_running = FALSE;
 
   inf_standalone_io_wakeup(io);
-  g_mutex_unlock(priv->mutex);
+  g_mutex_unlock(&priv->mutex);
 }
 
 /**
@@ -1280,9 +1229,9 @@ inf_standalone_io_loop_running(InfStandaloneIo* io)
   g_return_val_if_fail(INF_IS_STANDALONE_IO(io), FALSE);
   priv = INF_STANDALONE_IO_PRIVATE(io);
 
-  g_mutex_lock(priv->mutex);
+  g_mutex_lock(&priv->mutex);
   running = priv->loop_running;
-  g_mutex_unlock(priv->mutex);
+  g_mutex_unlock(&priv->mutex);
 
   return running;
 }
