@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,6 +21,7 @@
 #define __INF_TCP_CONNECTION_PRIVATE_H__
 
 #include <libinfinity/common/inf-io.h>
+#include <libinfinity/common/inf-keepalive.h>
 
 #include <glib-object.h>
 
@@ -31,6 +32,7 @@ _inf_tcp_connection_accepted(InfIo* io,
                              InfNativeSocket socket,
                              InfIpAddress* address,
                              guint port,
+                             const InfKeepalive* keepalive,
                              GError** error);
 
 G_END_DECLS
