@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -47,8 +47,8 @@
 #include <libinfinity/common/inf-io.h>
 #include <libinfinity/common/inf-native-socket.h>
 #include <libinfinity/inf-signals.h>
-#include <libinfinity/inf-marshal.h>
 #include <libinfinity/inf-i18n.h>
+#include <libinfinity/inf-define-enum.h>
 
 #include <unistd.h> /* For ssize_t */
 
@@ -69,6 +69,26 @@
 # include <ws2tcpip.h>
 #endif
 
+static const GEnumValue inf_tcp_connection_status_values[] = {
+  {
+    INF_TCP_CONNECTION_CONNECTING,
+    "INF_TCP_CONNECTION_CONNECTING",
+    "connecting"
+  }, {
+    INF_TCP_CONNECTION_CONNECTED,
+    "INF_TCP_CONNECTION_CONNECTED",
+    "connected"
+  }, {
+    INF_TCP_CONNECTION_CLOSED,
+    "INF_TCP_CONNECTION_CLOSED",
+    "closed"
+  }, {
+    0,
+    NULL,
+    NULL
+  }
+};
+
 typedef struct _InfTcpConnectionPrivate InfTcpConnectionPrivate;
 struct _InfTcpConnectionPrivate {
   InfIo* io;
@@ -80,6 +100,7 @@ struct _InfTcpConnectionPrivate {
 
   InfTcpConnectionStatus status;
   InfNativeSocket socket;
+  InfKeepalive keepalive;
 
   InfIpAddress* remote_address;
   guint remote_port;
@@ -98,6 +119,7 @@ enum {
   PROP_RESOLVER,
 
   PROP_STATUS,
+  PROP_KEEPALIVE,
 
   PROP_REMOTE_ADDRESS,
   PROP_REMOTE_PORT,
@@ -118,9 +140,12 @@ enum {
 
 #define INF_TCP_CONNECTION_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_TYPE_TCP_CONNECTION, InfTcpConnectionPrivate))
 
-static GObjectClass* parent_class;
 static guint tcp_connection_signals[LAST_SIGNAL];
 
+INF_DEFINE_ENUM_TYPE(InfTcpConnectionStatus, inf_tcp_connection_status, inf_tcp_connection_status_values)
+G_DEFINE_TYPE_WITH_CODE(InfTcpConnection, inf_tcp_connection, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfTcpConnection))
+
 static gboolean
 inf_tcp_connection_addr_info(InfNativeSocket socket,
                              gboolean local,
@@ -173,6 +198,58 @@ inf_tcp_connection_addr_info(InfNativeSocket socket,
   return TRUE;
 }
 
+static gboolean
+inf_tcp_connection_configure_socket(InfNativeSocket socket,
+                                    const InfKeepalive* keepalive,
+                                    GError** error)
+{
+
+#ifdef G_OS_WIN32
+  u_long argp;
+#else
+  int result;
+#endif
+  int errcode;
+  GError* local_error;
+
+  /* Configure the connection's underlying socket, by setting keepalive and
+   * and nonblocking. */
+#ifndef G_OS_WIN32
+  result = fcntl(socket, F_GETFL);
+  if(result == INVALID_SOCKET)
+  {
+    errcode = INF_NATIVE_SOCKET_LAST_ERROR;
+    inf_native_socket_make_error(errcode, error);
+    return FALSE;
+  }
+
+  if(fcntl(socket, F_SETFL, result | O_NONBLOCK) == -1)
+  {
+    errcode = INF_NATIVE_SOCKET_LAST_ERROR;
+    inf_native_socket_make_error(errcode, error);
+    return FALSE;
+  }
+#else
+  argp = 1;
+  if(ioctlsocket(socket, FIONBIO, &argp) != 0)
+  {
+    errcode = INF_NATIVE_SOCKET_LAST_ERROR;
+    inf_native_socket_make_error(errcode, error);
+    return FALSE;
+  }
+#endif
+
+  /* Error setting keepalives is not fatal */
+  local_error = NULL;
+  if(inf_keepalive_apply(keepalive, &socket, 0, &local_error) == FALSE)
+  {
+    g_warning("Failed to set keepalive on socket: %s", local_error->message);
+    g_error_free(local_error);
+  }
+
+  return TRUE;
+}
+
 static void
 inf_tcp_connection_system_error(InfTcpConnection* connection,
                                 int code)
@@ -285,7 +362,8 @@ inf_tcp_connection_connection_error(InfTcpConnection* connection,
   if(priv->resolver != NULL)
   {
     /* Try next address, if there is one */
-    if(priv->resolver_index < inf_name_resolver_get_n_addresses(priv->resolver))
+    if(priv->resolver_index <
+       inf_name_resolver_get_n_addresses(priv->resolver))
     {
       ++priv->resolver_index;
       if(inf_tcp_connection_open_with_resolver(connection, NULL) == TRUE)
@@ -314,10 +392,6 @@ inf_tcp_connection_open_real(InfTcpConnection* connection,
 {
   InfTcpConnectionPrivate* priv;
 
-#ifdef G_OS_WIN32
-  u_long argp;
-#endif
-
   union {
     struct sockaddr_in in;
     struct sockaddr_in6 in6;
@@ -327,6 +401,7 @@ inf_tcp_connection_open_real(InfTcpConnection* connection,
   socklen_t addrlen;
   int result;
   int errcode;
+  const InfKeepalive* keepalive;
   GError* local_error;
 
   priv = INF_TCP_CONNECTION_PRIVATE(connection);
@@ -383,41 +458,15 @@ inf_tcp_connection_open_real(InfTcpConnection* connection,
     return FALSE;
   }
 
-  /* Set socket non-blocking */
-#ifndef G_OS_WIN32
-  result = fcntl(priv->socket, F_GETFL);
-  if(result == INVALID_SOCKET)
+  /* Set socket non-blocking and keepalive */
+  keepalive = &priv->keepalive;
+  if(!inf_tcp_connection_configure_socket(priv->socket, keepalive, error))
   {
-    errcode = INF_NATIVE_SOCKET_LAST_ERROR;
-    inf_native_socket_make_error(errcode, error);
-
     closesocket(priv->socket);
     priv->socket = INVALID_SOCKET;
     return FALSE;
   }
 
-  if(fcntl(priv->socket, F_SETFL, result | O_NONBLOCK) == -1)
-  {
-    errcode = INF_NATIVE_SOCKET_LAST_ERROR;
-    inf_native_socket_make_error(errcode, error);
-
-    closesocket(priv->socket);
-    priv->socket = INVALID_SOCKET;
-    return FALSE;
-  }
-#else
-  argp = 1;
-  if(ioctlsocket(priv->socket, FIONBIO, &argp) != 0)
-  {
-    errcode = INF_NATIVE_SOCKET_LAST_ERROR;
-    inf_native_socket_make_error(errcode, error);
-
-    closesocket(priv->socket);
-    priv->socket = INVALID_SOCKET;
-    return FALSE;
-  }
-#endif
-
   /* Connect */
   do
   {
@@ -786,6 +835,9 @@ inf_tcp_connection_resolved_cb(InfNameResolver* resolver,
   connection = INF_TCP_CONNECTION(user_data);
   priv = INF_TCP_CONNECTION_PRIVATE(connection);
 
+  /* Note that the connection could even be closed here, namely if
+   * tcp_connection_close() was called while we are still resolving. */
+
   if(priv->status == INF_TCP_CONNECTION_CONNECTING)
   {
     if(error != NULL)
@@ -840,13 +892,9 @@ inf_tcp_connection_set_resolver(InfTcpConnection* connection,
 }
 
 static void
-inf_tcp_connection_init(GTypeInstance* instance,
-                        gpointer g_class)
+inf_tcp_connection_init(InfTcpConnection* connection)
 {
-  InfTcpConnection* connection;
   InfTcpConnectionPrivate* priv;
-
-  connection = INF_TCP_CONNECTION(instance);
   priv = INF_TCP_CONNECTION_PRIVATE(connection);
 
   priv->io = NULL;
@@ -856,6 +904,7 @@ inf_tcp_connection_init(GTypeInstance* instance,
   priv->resolver_index = 0;
   priv->status = INF_TCP_CONNECTION_CLOSED;
   priv->socket = INVALID_SOCKET;
+  priv->keepalive.mask = 0;
 
   priv->remote_address = NULL;
   priv->remote_port = 0;
@@ -887,7 +936,7 @@ inf_tcp_connection_dispose(GObject* object)
     priv->io = NULL;
   }
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_tcp_connection_parent_class)->dispose(object);
 }
 
 static void
@@ -907,7 +956,7 @@ inf_tcp_connection_finalize(GObject* object)
 
   g_free(priv->queue);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(inf_tcp_connection_parent_class)->finalize(object);
 }
 
 static void
@@ -922,6 +971,7 @@ inf_tcp_connection_set_property(GObject* object,
   const gchar* device_string;
   unsigned int new_index;
 #endif
+  GError* error;
 
   connection = INF_TCP_CONNECTION(object);
   priv = INF_TCP_CONNECTION_PRIVATE(connection);
@@ -941,6 +991,22 @@ inf_tcp_connection_set_property(GObject* object,
       INF_NAME_RESOLVER(g_value_get_object(value))
     );
 
+    break;
+  case PROP_KEEPALIVE:
+    error = NULL;
+
+    inf_tcp_connection_set_keepalive(
+      connection,
+      (InfKeepalive*)g_value_get_boxed(value),
+      &error
+    );
+    
+    if(error != NULL)
+    {
+      g_warning("Failed to set keepalive settings: %s\n", error->message);
+      g_error_free(error);
+    }
+
     break;
   case PROP_REMOTE_ADDRESS:
     g_assert(priv->status == INF_TCP_CONNECTION_CLOSED);
@@ -1011,6 +1077,9 @@ inf_tcp_connection_get_property(GObject* object,
   case PROP_RESOLVER:
     g_value_set_object(value, G_OBJECT(priv->resolver));
     break;
+  case PROP_KEEPALIVE:
+    g_value_set_boxed(value, &priv->keepalive);
+    break;
   case PROP_STATUS:
     g_value_set_enum(value, priv->status);
     break;
@@ -1121,17 +1190,10 @@ inf_tcp_connection_error(InfTcpConnection* connection,
 }
 
 static void
-inf_tcp_connection_class_init(gpointer g_class,
-                              gpointer class_data)
+inf_tcp_connection_class_init(InfTcpConnectionClass* tcp_connection_class)
 {
   GObjectClass* object_class;
-  InfTcpConnectionClass* tcp_connection_class;
-
-  object_class = G_OBJECT_CLASS(g_class);
-  tcp_connection_class = INF_TCP_CONNECTION_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfTcpConnectionPrivate));
+  object_class = G_OBJECT_CLASS(tcp_connection_class);
 
   object_class->dispose = inf_tcp_connection_dispose;
   object_class->finalize = inf_tcp_connection_finalize;
@@ -1179,6 +1241,18 @@ inf_tcp_connection_class_init(gpointer g_class,
     )
   );
 
+  g_object_class_install_property(
+    object_class,
+    PROP_KEEPALIVE,
+    g_param_spec_boxed(
+      "keepalive",
+      "Keepalive",
+      "The keepalive settings for the connection",
+      INF_TYPE_KEEPALIVE,
+      G_PARAM_READWRITE
+    )
+  );
+
   g_object_class_install_property(
     object_class,
     PROP_REMOTE_ADDRESS,
@@ -1271,7 +1345,7 @@ inf_tcp_connection_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfTcpConnectionClass, sent),
     NULL, NULL,
-    inf_marshal_VOID__POINTER_UINT,
+    NULL,
     G_TYPE_NONE,
     2,
     G_TYPE_POINTER,
@@ -1294,7 +1368,7 @@ inf_tcp_connection_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfTcpConnectionClass, received),
     NULL, NULL,
-    inf_marshal_VOID__POINTER_UINT,
+    NULL,
     G_TYPE_NONE,
     2,
     G_TYPE_POINTER,
@@ -1316,82 +1390,15 @@ inf_tcp_connection_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfTcpConnectionClass, error),
     NULL, NULL,
-    inf_marshal_VOID__POINTER,
+    g_cclosure_marshal_VOID__POINTER,
     G_TYPE_NONE,
     1,
-    G_TYPE_POINTER /* actually a GError* */
+    G_TYPE_ERROR
   );
 }
 
-GType
-inf_tcp_connection_status_get_type(void)
-{
-  static GType tcp_connection_status_type = 0;
-
-  if(!tcp_connection_status_type)
-  {
-    static const GEnumValue tcp_connection_status_values[] = {
-      {
-        INF_TCP_CONNECTION_CONNECTING,
-        "INF_TCP_CONNECTION_CONNECTING",
-        "connecting"
-      }, {
-        INF_TCP_CONNECTION_CONNECTED,
-        "INF_TCP_CONNECTION_CONNECTED",
-        "connected"
-      }, {
-        INF_TCP_CONNECTION_CLOSED,
-        "INF_TCP_CONNECTION_CLOSED",
-        "closed"
-      }, {
-        0,
-        NULL,
-        NULL
-      }
-    };
-
-    tcp_connection_status_type = g_enum_register_static(
-      "InfTcpConnectionStatus",
-      tcp_connection_status_values
-    );
-  }
-
-  return tcp_connection_status_type;
-}
-
-GType
-inf_tcp_connection_get_type(void)
-{
-  static GType tcp_connection_type = 0;
-
-  if(!tcp_connection_type)
-  {
-    static const GTypeInfo tcp_connection_type_info = {
-      sizeof(InfTcpConnectionClass),  /* class_size */
-      NULL,                           /* base_init */
-      NULL,                           /* base_finalize */
-      inf_tcp_connection_class_init,  /* class_init */
-      NULL,                           /* class_finalize */
-      NULL,                           /* class_data */
-      sizeof(InfTcpConnection),       /* instance_size */
-      0,                              /* n_preallocs */
-      inf_tcp_connection_init,        /* instance_init */
-      NULL                            /* value_table */
-    };
-
-    tcp_connection_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfTcpConnection",
-      &tcp_connection_type_info,
-      0
-    );
-  }
-
-  return tcp_connection_type;
-}
-
 /**
- * inf_tcp_connection_new:
+ * inf_tcp_connection_new: (constructor)
  * @io: A #InfIo object used to watch for activity.
  * @remote_addr: The address to eventually connect to.
  * @remote_port: The port to eventually connect to.
@@ -1400,7 +1407,8 @@ inf_tcp_connection_get_type(void)
  * an eventual inf_tcp_connection_open() call, this function itself does not
  * establish a connection.
  *
- * Returns: A new #InfTcpConnection. Free with g_object_unref().
+ * Returns: (transfer full): A new #InfTcpConnection. Free with
+ * g_object_unref().
  **/
 InfTcpConnection*
 inf_tcp_connection_new(InfIo* io,
@@ -1427,7 +1435,7 @@ inf_tcp_connection_new(InfIo* io,
 }
 
 /**
- * inf_tcp_connection_new_and_open:
+ * inf_tcp_connection_new_and_open: (constructor)
  * @io: A #InfIo object used to watch for activity.
  * @remote_addr: The address to connect to.
  * @remote_port: The port to connect to.
@@ -1436,8 +1444,8 @@ inf_tcp_connection_new(InfIo* io,
  * Creates a new #InfTcpConnection and connects it to the given TCP endpoint.
  * Like inf_tcp_connection_new(), but calls inf_tcp_connection_open().
  *
- * Returns: A new #InfTcpConnection, or %NULL on error. Free with
- * g_object_unref().
+ * Returns: (transfer full): A new #InfTcpConnection, or %NULL on error.
+ * Free with g_object_unref().
  **/
 InfTcpConnection*
 inf_tcp_connection_new_and_open(InfIo* io,
@@ -1464,7 +1472,7 @@ inf_tcp_connection_new_and_open(InfIo* io,
 }
 
 /**
- * inf_tcp_connection_new_resolve:
+ * inf_tcp_connection_new_resolve: (constructor)
  * @io: A #InfIo object used to watch for activity.
  * @resolver: The hostname resolver object used to look up the remote
  * hostname.
@@ -1478,7 +1486,8 @@ inf_tcp_connection_new_and_open(InfIo* io,
  * inf_tcp_connection_open() call, this function itself does not
  * establish a connection.
  *
- * Returns: A new #InfTcpConnection. Free with g_object_unref().
+ * Returns: (transfer full): A new #InfTcpConnection. Free with
+ * g_object_unref().
  */
 InfTcpConnection*
 inf_tcp_connection_new_resolve(InfIo* io,
@@ -1573,9 +1582,11 @@ inf_tcp_connection_close(InfTcpConnection* connection)
 
   priv->events = 0;
 
-  g_assert(priv->watch != NULL);
-  inf_io_remove_watch(priv->io, priv->watch);
-  priv->watch = NULL;
+  if(priv->watch != NULL)
+  {
+    inf_io_remove_watch(priv->io, priv->watch);
+    priv->watch = NULL;
+  }
 
   priv->front_pos = 0;
   priv->back_pos = 0;
@@ -1587,7 +1598,7 @@ inf_tcp_connection_close(InfTcpConnection* connection)
 /**
  * inf_tcp_connection_send:
  * @connection: A #InfTcpConnection with status %INF_TCP_CONNECTION_CONNECTED.
- * @data: The data to send.
+ * @data: (type guint8*) (array length=len): The data to send.
  * @len: Number of bytes to send.
  *
  * Sends data through the TCP connection. The data is not sent immediately,
@@ -1703,9 +1714,9 @@ inf_tcp_connection_send(InfTcpConnection* connection,
  *
  * Returns the IP address of the remote site.
  *
- * Return Value: A #InfIpAddress owned by @connection. You do not need to
- * free it, but need to make your own copy if you want to keep it longer than
- * @connection's lifetime.
+ * Returns: (transfer none): A #InfIpAddress owned by @connection. You do not
+ * need to free it, but need to make your own copy if you want to keep it
+ * longer than @connection's lifetime.
  **/
 InfIpAddress*
 inf_tcp_connection_get_remote_address(InfTcpConnection* connection)
@@ -1721,7 +1732,7 @@ inf_tcp_connection_get_remote_address(InfTcpConnection* connection)
  * Returns the port of the remote site to which @connection is (or was)
  * connected or connecting.
  *
- * Return Value: The port of the remote site.
+ * Returns: The port of the remote site.
  **/
 guint
 inf_tcp_connection_get_remote_port(InfTcpConnection* connection)
@@ -1730,6 +1741,62 @@ inf_tcp_connection_get_remote_port(InfTcpConnection* connection)
   return INF_TCP_CONNECTION_PRIVATE(connection)->remote_port;
 }
 
+/**
+ * inf_tcp_connection_set_keepalive:
+ * @connection: A #InfTcpConnection.
+ * @keepalive: New keepalive settings for the connection.
+ * @error: Location to store error information, if any, or %NULL.
+ *
+ * Sets the keepalive settings for @connection. When this function is not
+ * called, the system defaults are used. If the connection is closed, then
+ * the function always succeeds and stores the keepalive values internally.
+ * The values are actually set on the underlying socket when the connection
+ * is opened. If the connection is already open, the function might fail if
+ * the system call fails.
+ *
+ * Returns: %TRUE if the new keeplalive values were set, or %FALSE on error.
+ */
+gboolean
+inf_tcp_connection_set_keepalive(InfTcpConnection* connection,
+                                 const InfKeepalive* keepalive,
+                                 GError** error)
+{
+  InfTcpConnectionPrivate* priv;
+  InfKeepaliveMask mask;
+
+  g_return_val_if_fail(INF_IS_TCP_CONNECTION(connection), FALSE);
+  g_return_val_if_fail(keepalive != NULL, FALSE);
+  g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+  priv = INF_TCP_CONNECTION_PRIVATE(connection);
+
+  if(priv->socket != INVALID_SOCKET)
+  {
+    mask = priv->keepalive.mask;
+    if(inf_keepalive_apply(keepalive, &priv->socket, mask, error) != TRUE)
+      return FALSE;
+  }
+
+  priv->keepalive = *keepalive;
+  return TRUE;
+}
+
+/**
+ * inf_tcp_connection_get_keepalive:
+ * @connection: A #InfTcpConnection.
+ *
+ * Returns the current keepalive settings for @connection.
+ *
+ * Returns: The current keepalive configuration for @connection, owned by
+ * @connection.
+ */
+const InfKeepalive*
+inf_tcp_connection_get_keepalive(InfTcpConnection* connection)
+{
+  g_return_val_if_fail(INF_IS_TCP_CONNECTION(connection), NULL);
+  return &INF_TCP_CONNECTION_PRIVATE(connection)->keepalive;
+}
+
 /* Creates a new TCP connection from an accepted socket. This is only used
  * by InfdTcpServer and should not be considered regular API. Do not call
  * this function. Language bindings should not wrap it. */
@@ -1738,45 +1805,20 @@ _inf_tcp_connection_accepted(InfIo* io,
                              InfNativeSocket socket,
                              InfIpAddress* address,
                              guint port,
+                             const InfKeepalive* keepalive,
                              GError** error)
 {
   InfTcpConnection* connection;
   InfTcpConnectionPrivate* priv;
   int errcode;
 
-#ifdef G_OS_WIN32
-  u_long argp;
-#else
-  int result;
-#endif
-
   g_return_val_if_fail(INF_IS_IO(io), NULL);
   g_return_val_if_fail(socket != INVALID_SOCKET, NULL);
+  g_return_val_if_fail(address != NULL, NULL);
+  g_return_val_if_fail(keepalive != NULL, NULL);
 
-#ifndef G_OS_WIN32
-  result = fcntl(socket, F_GETFL);
-  if(result == -1)
-  {
-    errcode = INF_NATIVE_SOCKET_LAST_ERROR;
-    inf_native_socket_make_error(errcode, error);
+  if(inf_tcp_connection_configure_socket(socket, keepalive, error) != TRUE)
     return NULL;
-  }
-
-  if(fcntl(socket, F_SETFL, result | O_NONBLOCK) == -1)
-  {
-    errcode = INF_NATIVE_SOCKET_LAST_ERROR;
-    inf_native_socket_make_error(errcode, error);
-    return NULL;
-  }
-#else
-  argp = 1;
-  if(ioctlsocket(socket, FIONBIO, &argp) != 0)
-  {
-    errcode = INF_NATIVE_SOCKET_LAST_ERROR;
-    inf_native_socket_make_error(errcode, error);
-    return NULL;
-  }
-#endif
 
   g_return_val_if_fail(address != NULL, NULL);
   g_return_val_if_fail(port != 0, NULL);
@@ -1787,6 +1829,7 @@ _inf_tcp_connection_accepted(InfIo* io,
 
   priv = INF_TCP_CONNECTION_PRIVATE(connection);
   priv->socket = socket;
+  priv->keepalive = *keepalive;
 
   inf_tcp_connection_connected(connection);
   return connection;
