@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,6 @@
  */
 
 #include <libinfinity/common/inf-user-table.h>
-#include <libinfinity/inf-marshal.h>
 #include <libinfinity/inf-signals.h>
 
 #include <string.h>
@@ -65,9 +64,11 @@ enum {
 
 #define INF_USER_TABLE_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_TYPE_USER_TABLE, InfUserTablePrivate))
 
-static GObjectClass* parent_class;
 static guint user_table_signals[LAST_SIGNAL];
 
+G_DEFINE_TYPE_WITH_CODE(InfUserTable, inf_user_table, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfUserTable))
+
 static gboolean
 inf_user_table_is_local(InfUser* user)
 {
@@ -190,13 +191,9 @@ inf_user_table_foreach_user_func(gpointer key,
 }
 
 static void
-inf_user_table_init(GTypeInstance* instance,
-                    gpointer g_class)
+inf_user_table_init(InfUserTable* user_table)
 {
-  InfUserTable* user_table;
   InfUserTablePrivate* priv;
-
-  user_table = INF_USER_TABLE(instance);
   priv = INF_USER_TABLE_PRIVATE(user_table);
 
   priv->table = g_hash_table_new_full(NULL, NULL, NULL, NULL);
@@ -226,7 +223,7 @@ inf_user_table_dispose(GObject* object)
   );
 
   g_hash_table_remove_all(priv->table);
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(inf_user_table_parent_class)->dispose(object);
 }
 
 static void
@@ -240,7 +237,7 @@ inf_user_table_finalize(GObject* object)
 
   g_hash_table_destroy(priv->table);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(inf_user_table_parent_class)->finalize(object);
 }
 
 static void
@@ -367,17 +364,10 @@ inf_user_table_remove_local_user(InfUserTable* user_table,
 }
 
 static void
-inf_user_table_class_init(gpointer g_class,
-                          gpointer class_data)
+inf_user_table_class_init(InfUserTableClass* user_table_class)
 {
   GObjectClass* object_class;
-  InfUserTableClass* user_table_class;
-
-  object_class = G_OBJECT_CLASS(g_class);
-  user_table_class = INF_USER_TABLE_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfUserTablePrivate));
+  object_class = G_OBJECT_CLASS(user_table_class);
 
   object_class->dispose = inf_user_table_dispose;
   object_class->finalize = inf_user_table_finalize;
@@ -408,7 +398,7 @@ inf_user_table_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfUserTableClass, add_user),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT,
+    g_cclosure_marshal_VOID__OBJECT,
     G_TYPE_NONE,
     1,
     INF_TYPE_USER
@@ -432,7 +422,7 @@ inf_user_table_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfUserTableClass, remove_user),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT,
+    g_cclosure_marshal_VOID__OBJECT,
     G_TYPE_NONE,
     1,
     INF_TYPE_USER
@@ -454,7 +444,7 @@ inf_user_table_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfUserTableClass, add_available_user),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT,
+    g_cclosure_marshal_VOID__OBJECT,
     G_TYPE_NONE,
     1,
     INF_TYPE_USER
@@ -477,7 +467,7 @@ inf_user_table_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfUserTableClass, remove_available_user),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT,
+    g_cclosure_marshal_VOID__OBJECT,
     G_TYPE_NONE,
     1,
     INF_TYPE_USER
@@ -504,7 +494,7 @@ inf_user_table_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfUserTableClass, add_local_user),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT,
+    g_cclosure_marshal_VOID__OBJECT,
     G_TYPE_NONE,
     1,
     INF_TYPE_USER
@@ -530,50 +520,19 @@ inf_user_table_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfUserTableClass, remove_local_user),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT,
+    g_cclosure_marshal_VOID__OBJECT,
     G_TYPE_NONE,
     1,
     INF_TYPE_USER
   );
 }
 
-GType
-inf_user_table_get_type(void)
-{
-  static GType user_table_type = 0;
-
-  if(!user_table_type)
-  {
-    static const GTypeInfo user_table_type_info = {
-      sizeof(InfUserTableClass),   /* class_size */
-      NULL,                        /* base_init */
-      NULL,                        /* base_finalize */
-      inf_user_table_class_init,   /* class_init */
-      NULL,                        /* class_finalize */
-      NULL,                        /* class_data */
-      sizeof(InfUserTable),        /* instance_size */
-      0,                           /* n_preallocs */
-      inf_user_table_init,         /* instance_init */
-      NULL                         /* value_table */
-    };
-
-    user_table_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfUserTable",
-      &user_table_type_info,
-      0
-    );
-  }
-
-  return user_table_type;
-}
-
 /**
- * inf_user_table_new:
+ * inf_user_table_new: (constructor)
  *
  * Creates a new, empty user table.
  *
- * Return Value: A #InfUserTable.
+ * Returns: (transfer full): A #InfUserTable.
  **/
 InfUserTable*
 inf_user_table_new(void)
@@ -627,7 +586,7 @@ inf_user_table_remove_user(InfUserTable* user_table,
  *
  * Returns the #InfUser with the given User ID in @user_table.
  *
- * Return Value: A #InfUser, or %NULL.
+ * Returns: (transfer none) (allow-none): A #InfUser, or %NULL.
  **/
 InfUser*
 inf_user_table_lookup_user_by_id(InfUserTable* user_table,
@@ -649,7 +608,7 @@ inf_user_table_lookup_user_by_id(InfUserTable* user_table,
  *
  * Returns an #InfUser with the given name if there is one.
  *
- * Return Value: A #InfUser, or %NULL.
+ * Returns: (transfer none) (allow-none): A #InfUser, or %NULL.
  **/
 InfUser*
 inf_user_table_lookup_user_by_name(InfUserTable* user_table,
@@ -675,7 +634,7 @@ inf_user_table_lookup_user_by_name(InfUserTable* user_table,
 /**
  * inf_user_table_foreach_user:
  * @user_table: A #InfUserTable.
- * @func: The function to call for each user.
+ * @func: (scope call): The function to call for each user.
  * @user_data: User data to pass to the function.
  *
  * Calls the given function for each user in the user_table. You should not
@@ -707,7 +666,7 @@ inf_user_table_foreach_user(InfUserTable* user_table,
 /**
  * inf_user_table_foreach_local_user:
  * @user_table: A #InfUserTable.
- * @func: The function to call for each user.
+ * @func: (scope call): The function to call for each user.
  * @user_data: User data to pass to the function.
  *
  * Calls the given function for each local user in the user_table. A local
