@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -41,7 +41,7 @@
  * #InfdDirectory takes advantage of it. The only required operation is to be
  * able to look up an account name by its ID, and the reverse lookup, i.e.
  * find all accounts with a given name. The
- * #InfdAccountStorageIface.get_support() function returns a bitmask of the
+ * #InfdAccountStorageInterface.get_support() function returns a bitmask of the
  * supported operations.
  *
  * Implementations of this interface can couple the available accounts to
@@ -53,9 +53,52 @@
 
 #include <libinfinity/server/infd-account-storage.h>
 #include <libinfinity/common/inf-error.h>
-#include <libinfinity/inf-marshal.h>
+#include <libinfinity/inf-define-enum.h>
 #include <libinfinity/inf-i18n.h>
 
+static const GFlagsValue infd_account_storage_support_values[] = {
+  {
+    INFD_ACCOUNT_STORAGE_SUPPORT_NOTIFICATION,
+    "INFD_ACCOUNT_STORAGE_SUPPORT_NOTIFICATION",
+    "notification"
+  }, {
+    INFD_ACCOUNT_STORAGE_SUPPORT_LIST_ACCOUNTS,
+    "INFD_ACCOUNT_STORAGE_SUPPORT_LIST_ACCOUNTS",
+    "list-accounts"
+  }, {
+    INFD_ACCOUNT_STORAGE_SUPPORT_ADD_ACCOUNT,
+    "INFD_ACCOUNT_STORAGE_SUPPORT_ADD_ACCOUNT",
+    "add-account"
+  }, {
+    INFD_ACCOUNT_STORAGE_SUPPORT_REMOVE_ACCOUNT,
+    "INFD_ACCOUNT_STORAGE_SUPPORT_REMOVE_ACCOUNT",
+    "remove-account"
+  }, {
+    INFD_ACCOUNT_STORAGE_SUPPORT_CERTIFICATE_LOGIN,
+    "INFD_ACCOUNT_STORAGE_SUPPORT_CERTIFICATE_LOGIN",
+    "certificate-login"
+  }, {
+    INFD_ACCOUNT_STORAGE_SUPPORT_PASSWORD_LOGIN,
+    "INFD_ACCOUNT_STORAGE_SUPPORT_PASSWORD_LOGIN",
+    "password-login"
+  }, {
+    INFD_ACCOUNT_STORAGE_SUPPORT_CERTIFICATE_CHANGE,
+    "INFD_ACCOUNT_STORAGE_SUPPORT_CERTIFICATE_CHANGE",
+    "certificate-change"
+  }, {
+    INFD_ACCOUNT_STORAGE_SUPPORT_PASSWORD_CHANGE,
+    "INFD_ACCOUNT_STORAGE_SUPPORT_PASSWORD_CHANGE",
+    "password-change"
+  }, {
+    0,
+    NULL,
+    NULL
+  }
+};
+
+INF_DEFINE_FLAGS_TYPE(InfdAccountStorageSupport, infd_account_storage_support, infd_account_storage_support_values)
+G_DEFINE_INTERFACE(InfdAccountStorage, infd_account_storage, G_TYPE_OBJECT)
+
 enum {
   ACCOUNT_ADDED,
   ACCOUNT_REMOVED,
@@ -66,149 +109,53 @@ enum {
 static guint account_storage_signals[LAST_SIGNAL];
 
 static void
-infd_account_storage_base_init(gpointer g_class)
+infd_account_storage_default_init(InfdAccountStorageInterface* iface)
 {
-  static gboolean initialized = FALSE;
+  /**
+   * InfdAccountStorage::account-added:
+   * @storage: The #InfdAccountStorage emitting the signal.
+   * @info: The #InfAclAccount containing the account ID and account name
+   * of the added account.
+   *
+   * This signal is emitted whenever an account has been added to the
+   * account storage. However, the signal is only emitted if the storage
+   * implementations supports the %INFD_ACCOUNT_STORAGE_SUPPORT_NOTIFICATION
+   * support flag.
+   */
+  account_storage_signals[ACCOUNT_ADDED] = g_signal_new(
+    "account-added",
+    INFD_TYPE_ACCOUNT_STORAGE,
+    G_SIGNAL_RUN_LAST,
+    G_STRUCT_OFFSET(InfdAccountStorageInterface, account_added),
+    NULL, NULL,
+    g_cclosure_marshal_VOID__BOXED,
+    G_TYPE_NONE,
+    1,
+    INF_TYPE_ACL_ACCOUNT | G_SIGNAL_TYPE_STATIC_SCOPE
+  );
 
-  if(!initialized)
-  {
-    /**
-     * InfdAccountStorage::account-added:
-     * @storage: The #InfdAccountStorage emitting the signal.
-     * @info: The #InfAclAccount containing the account ID and account name
-     * of the added account.
-     *
-     * This signal is emitted whenever an account has been added to the
-     * account storage. However, the signal is only emitted if the storage
-     * implementations supports the %INFD_ACCOUNT_STORAGE_SUPPORT_NOTIFICATION
-     * support flag.
-     */
-    account_storage_signals[ACCOUNT_ADDED] = g_signal_new(
-      "account-added",
-      INFD_TYPE_ACCOUNT_STORAGE,
-      G_SIGNAL_RUN_LAST,
-      G_STRUCT_OFFSET(InfdAccountStorageIface, account_added),
-      NULL, NULL,
-      inf_marshal_VOID__BOXED,
-      G_TYPE_NONE,
-      1,
-      INF_TYPE_ACL_ACCOUNT | G_SIGNAL_TYPE_STATIC_SCOPE
-    );
-
-    /**
-     * InfdAccountStorage::account-removed:
-     * @storage: The #InfdAccountStorage emitting the signal.
-     * @info: The #InfAclAccount containing the account ID and account name
-     * of the removed account.
-     * 
-     * This signal is emitted whenever an account has been permanently removed
-     * from the storage. However, the signal is only emitted if the storage
-     * implementations supports the %INFD_ACCOUNT_STORAGE_SUPPORT_NOTIFICATION
-     * support flag.
-     */
-    account_storage_signals[ACCOUNT_REMOVED] = g_signal_new(
-      "account-removed",
-      INFD_TYPE_ACCOUNT_STORAGE,
-      G_SIGNAL_RUN_LAST,
-      G_STRUCT_OFFSET(InfdAccountStorageIface, account_removed),
-      NULL, NULL,
-      inf_marshal_VOID__BOXED,
-      G_TYPE_NONE,
-      1,
-      INF_TYPE_ACL_ACCOUNT | G_SIGNAL_TYPE_STATIC_SCOPE
-    );
-
-    initialized = TRUE;
-  }
-}
-
-GType
-infd_account_storage_support_get_type(void)
-{
-  static GType account_storage_support_type = 0;
-
-  if(!account_storage_support_type)
-  {
-    static const GFlagsValue account_storage_support_values[] = {
-      {
-        INFD_ACCOUNT_STORAGE_SUPPORT_NOTIFICATION,
-        "INFD_ACCOUNT_STORAGE_SUPPORT_NOTIFICATION",
-        "notification"
-      }, {
-        INFD_ACCOUNT_STORAGE_SUPPORT_LIST_ACCOUNTS,
-        "INFD_ACCOUNT_STORAGE_SUPPORT_LIST_ACCOUNTS",
-        "list-accounts"
-      }, {
-        INFD_ACCOUNT_STORAGE_SUPPORT_ADD_ACCOUNT,
-        "INFD_ACCOUNT_STORAGE_SUPPORT_ADD_ACCOUNT",
-        "add-account"
-      }, {
-        INFD_ACCOUNT_STORAGE_SUPPORT_REMOVE_ACCOUNT,
-        "INFD_ACCOUNT_STORAGE_SUPPORT_REMOVE_ACCOUNT",
-        "remove-account"
-      }, {
-        INFD_ACCOUNT_STORAGE_SUPPORT_CERTIFICATE_LOGIN,
-        "INFD_ACCOUNT_STORAGE_SUPPORT_CERTIFICATE_LOGIN",
-        "certificate-login"
-      }, {
-        INFD_ACCOUNT_STORAGE_SUPPORT_PASSWORD_LOGIN,
-        "INFD_ACCOUNT_STORAGE_SUPPORT_PASSWORD_LOGIN",
-        "password-login"
-      }, {
-        INFD_ACCOUNT_STORAGE_SUPPORT_CERTIFICATE_CHANGE,
-        "INFD_ACCOUNT_STORAGE_SUPPORT_CERTIFICATE_CHANGE",
-        "certificate-change"
-      }, {
-        INFD_ACCOUNT_STORAGE_SUPPORT_PASSWORD_CHANGE,
-        "INFD_ACCOUNT_STORAGE_SUPPORT_PASSWORD_CHANGE",
-        "password-change"
-      }, {
-        0,
-        NULL,
-        NULL
-      }
-    };
-
-    account_storage_support_type = g_flags_register_static(
-      "InfdAccountStorageSupport",
-      account_storage_support_values
-    );
-  }
-
-  return account_storage_support_type;
-}
-
-GType
-infd_account_storage_get_type(void)
-{
-  static GType account_storage_type = 0;
-
-  if(!account_storage_type)
-  {
-    static const GTypeInfo account_storage_info = {
-      sizeof(InfdAccountStorageIface),  /* class_size */
-      infd_account_storage_base_init,   /* base_init */
-      NULL,                             /* base_finalize */
-      NULL,                             /* class_init */
-      NULL,                             /* class_finalize */
-      NULL,                             /* class_data */
-      0,                                /* instance_size */
-      0,                                /* n_preallocs */
-      NULL,                             /* instance_init */
-      NULL                              /* value_table */
-    };
-
-    account_storage_type = g_type_register_static(
-      G_TYPE_INTERFACE,
-      "InfdAccountStorage",
-      &account_storage_info,
-      0
-    );
-
-    g_type_interface_add_prerequisite(account_storage_type, G_TYPE_OBJECT);
-  }
-
-  return account_storage_type;
+  /**
+   * InfdAccountStorage::account-removed:
+   * @storage: The #InfdAccountStorage emitting the signal.
+   * @info: The #InfAclAccount containing the account ID and account name
+   * of the removed account.
+   * 
+   * This signal is emitted whenever an account has been permanently removed
+   * from the storage. However, the signal is only emitted if the storage
+   * implementations supports the %INFD_ACCOUNT_STORAGE_SUPPORT_NOTIFICATION
+   * support flag.
+   */
+  account_storage_signals[ACCOUNT_REMOVED] = g_signal_new(
+    "account-removed",
+    INFD_TYPE_ACCOUNT_STORAGE,
+    G_SIGNAL_RUN_LAST,
+    G_STRUCT_OFFSET(InfdAccountStorageInterface, account_removed),
+    NULL, NULL,
+    g_cclosure_marshal_VOID__BOXED,
+    G_TYPE_NONE,
+    1,
+    INF_TYPE_ACL_ACCOUNT | G_SIGNAL_TYPE_STATIC_SCOPE
+  );
 }
 
 /**
@@ -224,7 +171,7 @@ infd_account_storage_get_type(void)
 InfdAccountStorageSupport
 infd_account_storage_get_support(InfdAccountStorage* storage)
 {
-  InfdAccountStorageIface* iface;
+  InfdAccountStorageInterface* iface;
 
   g_return_val_if_fail(INFD_IS_ACCOUNT_STORAGE(storage), 0);
 
@@ -264,7 +211,8 @@ infd_account_storage_supports(InfdAccountStorage* storage,
 /**
  * infd_account_storage_lookup_accounts:
  * @storage: A #InfdAccountStorage.
- * @accounts: An array of #InfAclAccountId<!-- -->s to look up.
+ * @accounts: (array length=n_accounts): An array of #InfAclAccountId<!-- -->s
+ * to look up.
  * @n_accounts: The number of elements in @accounts.
  * @error: Location to store error information, if any, or %NULL.
  *
@@ -274,8 +222,9 @@ infd_account_storage_supports(InfdAccountStorage* storage,
  * the #InfAclAccount.id field set to 0, it means that the account with the
  * corresponding ID in the @accounts array does not exist in @storage.
  *
- * Returns: An array of #InfAclAccount structures with @n_accounts entries.
- * Free with inf_acl_account_array_free().
+ * Returns: (array length=n_accounts) (transfer full): An array of
+ * #InfAclAccount structures with @n_accounts entries. Free with
+ * inf_acl_account_array_free().
  */
 InfAclAccount*
 infd_account_storage_lookup_accounts(InfdAccountStorage* storage,
@@ -283,7 +232,7 @@ infd_account_storage_lookup_accounts(InfdAccountStorage* storage,
                                      guint n_accounts,
                                      GError** error)
 {
-  InfdAccountStorageIface* iface;
+  InfdAccountStorageInterface* iface;
 
   g_return_val_if_fail(INFD_IS_ACCOUNT_STORAGE(storage), NULL);
   g_return_val_if_fail(accounts != NULL, NULL);
@@ -300,7 +249,8 @@ infd_account_storage_lookup_accounts(InfdAccountStorage* storage,
  * infd_account_storage_lookup_accounts_by_name:
  * @storage: A #InfdAccountStorage.
  * @name: The name to look up.
- * @n_accounts: An output parameter holding the number of returned accounts.
+ * @n_accounts: (out): An output parameter holding the number of returned
+ * accounts.
  * @error: Location to store error information, if any, or %NULL.
  *
  * This function performs the &quot;reverse&quot; lookup: Given an account
@@ -315,8 +265,9 @@ infd_account_storage_lookup_accounts(InfdAccountStorage* storage,
  * lookup error occured or this is no account with the given name, a non-%NULL
  * error pointer should be passed and checked after the function call.
  *
- * Returns: An array of #InfAclAccount structures with length @n_accounts, or
- * %NULL on error or when @n_accounts is 0 or @error is set. Free with
+ * Returns: (array length=n_accounts) (transfer full): An array of
+ * #InfAclAccount structures with length @n_accounts, or %NULL on error
+ * or when @n_accounts is 0 or @error is set. Free with
  * inf_acl_account_array_free().
  */
 InfAclAccount*
@@ -325,7 +276,7 @@ infd_account_storage_lookup_accounts_by_name(InfdAccountStorage* storage,
                                              guint* n_accounts,
                                              GError** error)
 {
-  InfdAccountStorageIface* iface;
+  InfdAccountStorageInterface* iface;
 
   g_return_val_if_fail(INFD_IS_ACCOUNT_STORAGE(storage), NULL);
   g_return_val_if_fail(name != NULL, NULL);
@@ -341,7 +292,7 @@ infd_account_storage_lookup_accounts_by_name(InfdAccountStorage* storage,
 /**
  * infd_account_storage_list_accounts:
  * @storage: A #InfdAccountStorage.
- * @n_accounts: An output parameter holding the number of accounts
+ * @n_accounts: (out): An output parameter holding the number of accounts
  * in @storage.
  * @error: Location to store error information, if any, or %NULL.
  *
@@ -356,8 +307,9 @@ infd_account_storage_lookup_accounts_by_name(InfdAccountStorage* storage,
  * Note that this function might not be supported by the backend. See
  * infd_account_storage_get_support().
  *
- * Returns: An array of #InfAclAccount structures with length @n_accounts, or
- * %NULL if @n_accounts is 0 or @error is set. Free with
+ * Returns: (array length=n_accounts) (allow-none) (transfer full): An array
+ * of #InfAclAccount structures with length @n_accounts, or %NULL if
+ * @n_accounts is 0 or @error is set. Free with
  * inf_acl_account_array_free().
  */
 InfAclAccount*
@@ -365,7 +317,7 @@ infd_account_storage_list_accounts(InfdAccountStorage* storage,
                                    guint* n_accounts,
                                    GError** error)
 {
-  InfdAccountStorageIface* iface;
+  InfdAccountStorageInterface* iface;
 
   g_return_val_if_fail(INFD_IS_ACCOUNT_STORAGE(storage), NULL);
   g_return_val_if_fail(n_accounts != NULL, NULL);
@@ -374,11 +326,10 @@ infd_account_storage_list_accounts(InfdAccountStorage* storage,
   iface = INFD_ACCOUNT_STORAGE_GET_IFACE(storage);
   if(iface->list_accounts == NULL)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       inf_directory_error_quark(),
       INF_DIRECTORY_ERROR_OPERATION_UNSUPPORTED,
-      "%s",
       _("The account backend does not support acount listing")
     );
 
@@ -392,11 +343,11 @@ infd_account_storage_list_accounts(InfdAccountStorage* storage,
  * infd_account_storage_add_account:
  * @storage: A #InfdAccountStorage.
  * @name: The name of the new account.
- * @certs: An array of certificates that can be used to login to the new
- * account, or %NULL.
+ * @certs: (array length=n_certs) (allow-none): An array of certificates that
+ * can be used to login to the new account, or %NULL.
  * @n_certs: The length of the certificate array.
- * @password: A password that can be used to login to the new account,
- * or %NULL.
+ * @password: (allow-none): A password that can be used to login to the
+ * new account, or %NULL.
  * @error: Location to store error information, if any, or %NULL.
  *
  * Adds a new account to @storage. The account will have the given name. The
@@ -408,7 +359,8 @@ infd_account_storage_list_accounts(InfdAccountStorage* storage,
  * respectively. Note also that this function might not be supported at all
  * by the backend. See infd_account_storage_get_support().
  *
- * Returns: The #InfAclAccountId of the added account, or 0 on error.
+ * Returns: (transfer full): The #InfAclAccountId of the added account, or 0
+ * on error.
  */
 InfAclAccountId
 infd_account_storage_add_account(InfdAccountStorage* storage,
@@ -418,7 +370,7 @@ infd_account_storage_add_account(InfdAccountStorage* storage,
                                  const gchar* password,
                                  GError** error)
 {
-  InfdAccountStorageIface* iface;
+  InfdAccountStorageInterface* iface;
 
   g_return_val_if_fail(INFD_IS_ACCOUNT_STORAGE(storage), 0);
   g_return_val_if_fail(name != NULL, 0);
@@ -428,11 +380,10 @@ infd_account_storage_add_account(InfdAccountStorage* storage,
   iface = INFD_ACCOUNT_STORAGE_GET_IFACE(storage);
   if(iface->add_account == NULL)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       inf_directory_error_quark(),
       INF_DIRECTORY_ERROR_OPERATION_UNSUPPORTED,
-      "%s",
       _("The account backend does not support adding accounts")
     );
 
@@ -460,7 +411,7 @@ infd_account_storage_remove_account(InfdAccountStorage* storage,
                                     InfAclAccountId account,
                                     GError** error)
 {
-  InfdAccountStorageIface* iface;
+  InfdAccountStorageInterface* iface;
 
   g_return_val_if_fail(INFD_IS_ACCOUNT_STORAGE(storage), FALSE);
   g_return_val_if_fail(account != 0, FALSE);
@@ -469,11 +420,10 @@ infd_account_storage_remove_account(InfdAccountStorage* storage,
   iface = INFD_ACCOUNT_STORAGE_GET_IFACE(storage);
   if(iface->remove_account == NULL)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       inf_directory_error_quark(),
       INF_DIRECTORY_ERROR_OPERATION_UNSUPPORTED,
-      "%s",
       _("The account backend does not support removing accounts")
     );
 
@@ -497,15 +447,15 @@ infd_account_storage_remove_account(InfdAccountStorage* storage,
  * Note that this function might not be supported by the backend. See
  * infd_account_storage_get_support().
  *
- * Returns: The account ID of the account associated to @cert, or 0 on error
- * or if there is no such account.
+ * Returns: (transfer full): The account ID of the account associated to
+ * @cert, or 0 on error or if there is no such account.
  */
 InfAclAccountId
 infd_account_storage_login_by_certificate(InfdAccountStorage* storage,
                                           gnutls_x509_crt_t cert,
                                           GError** error)
 {
-  InfdAccountStorageIface* iface;
+  InfdAccountStorageInterface* iface;
 
   g_return_val_if_fail(INFD_IS_ACCOUNT_STORAGE(storage), 0);
   g_return_val_if_fail(cert != 0, 0);
@@ -514,11 +464,10 @@ infd_account_storage_login_by_certificate(InfdAccountStorage* storage,
   iface = INFD_ACCOUNT_STORAGE_GET_IFACE(storage);
   if(iface->login_by_certificate == NULL)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       inf_directory_error_quark(),
       INF_DIRECTORY_ERROR_OPERATION_UNSUPPORTED,
-      "%s",
       _("The account backend does not support logging in with a certificate")
     );
 
@@ -547,8 +496,8 @@ infd_account_storage_login_by_certificate(InfdAccountStorage* storage,
  * Note that this function might not be supported by the backend. See
  * infd_account_storage_get_support().
  *
- * Returns: The account ID of the account associated to @username if @password
- * is correct, or 0 otherwise.
+ * Returns: (transfer full): The account ID of the account associated to
+ * @username if @password is correct, or 0 otherwise.
  */
 InfAclAccountId
 infd_account_storage_login_by_password(InfdAccountStorage* storage,
@@ -556,7 +505,7 @@ infd_account_storage_login_by_password(InfdAccountStorage* storage,
                                        const gchar* password,
                                        GError** error)
 {
-  InfdAccountStorageIface* iface;
+  InfdAccountStorageInterface* iface;
 
   g_return_val_if_fail(INFD_IS_ACCOUNT_STORAGE(storage), 0);
   g_return_val_if_fail(username != 0, 0);
@@ -566,11 +515,10 @@ infd_account_storage_login_by_password(InfdAccountStorage* storage,
   iface = INFD_ACCOUNT_STORAGE_GET_IFACE(storage);
   if(iface->login_by_password == NULL)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       inf_directory_error_quark(),
       INF_DIRECTORY_ERROR_OPERATION_UNSUPPORTED,
-      "%s",
       _("The account backend does not support logging in with a password")
     );
 
@@ -584,7 +532,8 @@ infd_account_storage_login_by_password(InfdAccountStorage* storage,
  * infd_account_storage_set_certificate:
  * @storage: A #InfdAccountStorage.
  * @account: The ID of the account whose certificate to set.
- * @certs: An array of certificates, or %NULL if n_certs is 0.
+ * @certs: (array length=n_certs) (allow-none): An array of certificates,
+ * or %NULL if n_certs is 0.
  * @n_certs: The number of certificates in the certificate array.
  * @error: Location to store error information, if any.
  *
@@ -607,7 +556,7 @@ infd_account_storage_set_certificate(InfdAccountStorage* storage,
                                      guint n_certs,
                                      GError** error)
 {
-  InfdAccountStorageIface* iface;
+  InfdAccountStorageInterface* iface;
 
   g_return_val_if_fail(INFD_IS_ACCOUNT_STORAGE(storage), FALSE);
   g_return_val_if_fail(account != 0, FALSE);
@@ -617,11 +566,10 @@ infd_account_storage_set_certificate(InfdAccountStorage* storage,
   iface = INFD_ACCOUNT_STORAGE_GET_IFACE(storage);
   if(iface->set_certificate == NULL)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       inf_directory_error_quark(),
       INF_DIRECTORY_ERROR_OPERATION_UNSUPPORTED,
-      "%s",
       _("The account backend does not support changing the certificate")
     );
 
@@ -655,7 +603,7 @@ infd_account_storage_set_password(InfdAccountStorage* storage,
                                   const gchar* password,
                                   GError** error)
 {
-  InfdAccountStorageIface* iface;
+  InfdAccountStorageInterface* iface;
 
   g_return_val_if_fail(INFD_IS_ACCOUNT_STORAGE(storage), FALSE);
   g_return_val_if_fail(account != 0, FALSE);
@@ -664,11 +612,10 @@ infd_account_storage_set_password(InfdAccountStorage* storage,
   iface = INFD_ACCOUNT_STORAGE_GET_IFACE(storage);
   if(iface->set_password == NULL)
   {
-    g_set_error(
+    g_set_error_literal(
       error,
       inf_directory_error_quark(),
       INF_DIRECTORY_ERROR_OPERATION_UNSUPPORTED,
-      "%s",
       _("The account backend does not support changing the password")
     );
 
