@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,7 +21,8 @@
 #include <libinfinity/common/inf-tcp-connection-private.h>
 #include <libinfinity/common/inf-ip-address.h>
 #include <libinfinity/common/inf-io.h>
-#include <libinfinity/inf-marshal.h>
+#include <libinfinity/common/inf-native-socket.h>
+#include <libinfinity/inf-define-enum.h>
 #include <config.h>
 
 #ifndef G_OS_WIN32
@@ -38,17 +39,21 @@
 # include <ws2tcpip.h>
 #endif
 
-#ifdef G_OS_WIN32
-# define INFD_TCP_SERVER_LAST_ERROR WSAGetLastError()
-# define INFD_TCP_SERVER_EINTR      WSAEINTR
-# define INFD_TCP_SERVER_EAGAIN     WSAEWOULDBLOCK
-#else
-# define INFD_TCP_SERVER_LAST_ERROR errno
-# define INFD_TCP_SERVER_EINTR      EINTR
-# define INFD_TCP_SERVER_EAGAIN     EAGAIN
-# define closesocket(s) close(s)
-# define INVALID_SOCKET -1
-#endif
+static const GEnumValue infd_tcp_server_status_values[] = {
+  {
+    INFD_TCP_SERVER_CLOSED,
+    "INFD_TCP_SERVER_CLOSED",
+    "closed"
+  }, {
+    INFD_TCP_SERVER_OPEN,
+    "INFD_TCP_SERVER_OPEN",
+    "open"
+  }, {
+    0,
+    NULL,
+    NULL
+  }
+};
 
 typedef struct _InfdTcpServerPrivate InfdTcpServerPrivate;
 struct _InfdTcpServerPrivate {
@@ -60,6 +65,8 @@ struct _InfdTcpServerPrivate {
 
   InfIpAddress* local_address;
   guint local_port;
+
+  InfKeepalive keepalive;
 };
 
 enum {
@@ -70,7 +77,9 @@ enum {
   PROP_STATUS,
 
   PROP_LOCAL_ADDRESS,
-  PROP_LOCAL_PORT
+  PROP_LOCAL_PORT,
+
+  PROP_KEEPALIVE
 };
 
 enum {
@@ -82,10 +91,11 @@ enum {
 
 #define INFD_TCP_SERVER_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INFD_TYPE_TCP_SERVER, InfdTcpServerPrivate))
 
-static GObjectClass* parent_class;
 static guint tcp_server_signals[LAST_SIGNAL];
 
-static GQuark infd_tcp_server_error_quark;
+INF_DEFINE_ENUM_TYPE(InfdTcpServerStatus, infd_tcp_server_status, infd_tcp_server_status_values)
+G_DEFINE_TYPE_WITH_CODE(InfdTcpServer, infd_tcp_server, G_TYPE_OBJECT,
+  G_ADD_PRIVATE(InfdTcpServer))
 
 /* TODO: The following functions are merely copied from inf-tcp-connection.c.
  * Probably they should belong into some inf-net-util.c file in
@@ -131,34 +141,6 @@ infd_tcp_server_addr_info(InfNativeSocket socket,
   }
 }
 
-static void
-infd_tcp_server_make_system_error(int code,
-                                  GError** error)
-{
-#ifdef G_OS_WIN32
-  gchar* error_message;
-  error_message = g_win32_error_message(code);
-
-  g_set_error(
-    error,
-    infd_tcp_server_error_quark,
-    code,
-    "%s",
-    error_message
-  );
-
-  g_free(error_message);
-#else
-  g_set_error(
-    error,
-    infd_tcp_server_error_quark,
-    code,
-    "%s",
-    strerror(code)
-  );
-#endif
-}
-
 static void
 infd_tcp_server_system_error(InfdTcpServer* server,
                              int code)
@@ -166,7 +148,7 @@ infd_tcp_server_system_error(InfdTcpServer* server,
   GError* error;
   error = NULL;
 
-  infd_tcp_server_make_system_error(code, &error);
+  inf_native_socket_make_error(code, &error);
 
   g_signal_emit(G_OBJECT(server), tcp_server_signals[ERROR_], 0, error);
   g_error_free(error);
@@ -225,11 +207,11 @@ infd_tcp_server_io(InfNativeSocket* socket,
 #endif
       len = sizeof(native_addr);
       new_socket = accept(priv->socket, &native_addr.in_generic, &len);
-      errcode = INFD_TCP_SERVER_LAST_ERROR;
+      errcode = INF_NATIVE_SOCKET_LAST_ERROR;
 
       if(new_socket == INVALID_SOCKET &&
-         errcode != INFD_TCP_SERVER_EINTR &&
-         errcode != INFD_TCP_SERVER_EAGAIN)
+         errcode != INF_NATIVE_SOCKET_EINTR &&
+         errcode != INF_NATIVE_SOCKET_EAGAIN)
       {
         infd_tcp_server_system_error(server, errcode);
       }
@@ -256,6 +238,7 @@ infd_tcp_server_io(InfNativeSocket* socket,
           new_socket,
           address,
           port,
+          &priv->keepalive,
           &error
         );
 
@@ -287,7 +270,7 @@ infd_tcp_server_io(InfNativeSocket* socket,
       }
     } while( (new_socket != INVALID_SOCKET ||
               (new_socket == INVALID_SOCKET &&
-               errcode == INFD_TCP_SERVER_EINTR)) &&
+               errcode == INF_NATIVE_SOCKET_EINTR)) &&
              (priv->socket != INVALID_SOCKET));
   }
 
@@ -295,13 +278,9 @@ infd_tcp_server_io(InfNativeSocket* socket,
 }
 
 static void
-infd_tcp_server_init(GTypeInstance* instance,
-                     gpointer g_class)
+infd_tcp_server_init(InfdTcpServer* server)
 {
-  InfdTcpServer* server;
   InfdTcpServerPrivate* priv;
-
-  server = INFD_TCP_SERVER(instance);
   priv = INFD_TCP_SERVER_PRIVATE(server);
 
   priv->io = NULL;
@@ -311,6 +290,8 @@ infd_tcp_server_init(GTypeInstance* instance,
 
   priv->local_address = NULL;
   priv->local_port = 0;
+
+  priv->keepalive.mask = 0;
 }
 
 static void
@@ -331,7 +312,7 @@ infd_tcp_server_dispose(GObject* object)
     priv->io = NULL;
   }
 
-  G_OBJECT_CLASS(parent_class)->dispose(object);
+  G_OBJECT_CLASS(infd_tcp_server_parent_class)->dispose(object);
 }
 
 static void
@@ -346,7 +327,7 @@ infd_tcp_server_finalize(GObject* object)
   if(priv->local_address != NULL)
     inf_ip_address_free(priv->local_address);
 
-  G_OBJECT_CLASS(parent_class)->finalize(object);
+  G_OBJECT_CLASS(infd_tcp_server_parent_class)->finalize(object);
 }
 
 static void
@@ -378,6 +359,10 @@ infd_tcp_server_set_property(GObject* object,
     g_assert(priv->status == INFD_TCP_SERVER_CLOSED);
     priv->local_port = g_value_get_uint(value);
     break;
+  case PROP_KEEPALIVE:
+    g_assert(g_value_get_boxed(value) != NULL);
+    priv->keepalive = *(const InfKeepalive*)g_value_get_boxed(value);
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
     break;
@@ -410,6 +395,9 @@ infd_tcp_server_get_property(GObject* object,
   case PROP_LOCAL_PORT:
     g_value_set_uint(value, priv->local_port);
     break;
+  case PROP_KEEPALIVE:
+    g_value_set_boxed(value, &priv->keepalive);
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
     break;
@@ -444,17 +432,10 @@ infd_tcp_server_error(InfdTcpServer* server,
 }
 
 static void
-infd_tcp_server_class_init(gpointer g_class,
-                           gpointer class_data)
+infd_tcp_server_class_init(InfdTcpServerClass* tcp_server_class)
 {
   GObjectClass* object_class;
-  InfdTcpServerClass* tcp_server_class;
-
-  object_class = G_OBJECT_CLASS(g_class);
-  tcp_server_class = INFD_TCP_SERVER_CLASS(g_class);
-
-  parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(g_class));
-  g_type_class_add_private(g_class, sizeof(InfdTcpServerPrivate));
+  object_class = G_OBJECT_CLASS(tcp_server_class);
 
   object_class->dispose = infd_tcp_server_dispose;
   object_class->finalize = infd_tcp_server_finalize;
@@ -464,10 +445,6 @@ infd_tcp_server_class_init(gpointer g_class,
   tcp_server_class->new_connection = NULL;
   tcp_server_class->error = infd_tcp_server_error;
 
-  infd_tcp_server_error_quark = g_quark_from_static_string(
-    "INFD_TCP_SERVER_ERROR"
-  );
-
   g_object_class_install_property(
     object_class,
     PROP_IO,
@@ -519,13 +496,25 @@ infd_tcp_server_class_init(gpointer g_class,
     )
   );
 
+  g_object_class_install_property(
+    object_class,
+    PROP_KEEPALIVE,
+    g_param_spec_boxed(
+      "keepalive",
+      "Keepalive",
+      "Keepalive settings for accepted connections",
+      INF_TYPE_KEEPALIVE,
+      G_PARAM_READWRITE
+    )
+  );
+
   tcp_server_signals[NEW_CONNECTION] = g_signal_new(
     "new-connection",
     G_OBJECT_CLASS_TYPE(object_class),
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfdTcpServerClass, new_connection),
     NULL, NULL,
-    inf_marshal_VOID__OBJECT,
+    g_cclosure_marshal_VOID__OBJECT,
     G_TYPE_NONE,
     1,
     INF_TYPE_TCP_CONNECTION
@@ -537,76 +526,13 @@ infd_tcp_server_class_init(gpointer g_class,
     G_SIGNAL_RUN_LAST,
     G_STRUCT_OFFSET(InfdTcpServerClass, error),
     NULL, NULL,
-    inf_marshal_VOID__POINTER,
+    g_cclosure_marshal_VOID__BOXED,
     G_TYPE_NONE,
     1,
-    G_TYPE_POINTER /* actually a GError* */
+    G_TYPE_ERROR
   );
 }
 
-GType
-infd_tcp_server_status_get_type(void)
-{
-  static GType tcp_server_status_type = 0;
-
-  if(!tcp_server_status_type)
-  {
-    static const GEnumValue tcp_server_status_values[] = {
-      {
-        INFD_TCP_SERVER_CLOSED,
-        "INFD_TCP_SERVER_CLOSED",
-        "closed"
-      }, {
-        INFD_TCP_SERVER_OPEN,
-        "INFD_TCP_SERVER_OPEN",
-        "open"
-      }, {
-        0,
-        NULL,
-        NULL
-      }
-    };
-
-    tcp_server_status_type = g_enum_register_static(
-      "InfdTcpServerStatus",
-      tcp_server_status_values
-    );
-  }
-
-  return tcp_server_status_type;
-}
-
-GType
-infd_tcp_server_get_type(void)
-{
-  static GType tcp_server_type = 0;
-
-  if(!tcp_server_type)
-  {
-    static const GTypeInfo tcp_server_type_info = {
-      sizeof(InfdTcpServerClass),   /* class_size */
-      NULL,                         /* base_init */
-      NULL,                         /* base_finalize */
-      infd_tcp_server_class_init,   /* class_init */
-      NULL,                         /* class_finalize */
-      NULL,                         /* class_data */
-      sizeof(InfdTcpServer),        /* instance_size */
-      0,                            /* n_preallocs */
-      infd_tcp_server_init,         /* instance_init */
-      NULL                          /* value_table */
-    };
-
-    tcp_server_type = g_type_register_static(
-      G_TYPE_OBJECT,
-      "InfdTcpServer",
-      &tcp_server_type_info,
-      0
-    );
-  }
-
-  return tcp_server_type;
-}
-
 /**
  * infd_tcp_server_bind:
  * @server: A #InfdTcpServer.
@@ -698,7 +624,7 @@ infd_tcp_server_bind(InfdTcpServer* server,
 
   if(priv->socket == INVALID_SOCKET)
   {
-    infd_tcp_server_make_system_error(INFD_TCP_SERVER_LAST_ERROR, error);
+    inf_native_socket_make_error(INF_NATIVE_SOCKET_LAST_ERROR, error);
     return FALSE;
   }
 
@@ -709,7 +635,7 @@ infd_tcp_server_bind(InfdTcpServer* server,
   if(setsockopt(priv->socket, SOL_SOCKET, SO_REUSEADDR, &value,
       sizeof(int)) == -1)
   {
-    infd_tcp_server_make_system_error(INFD_TCP_SERVER_LAST_ERROR, error);
+    inf_native_socket_make_error(INF_NATIVE_SOCKET_LAST_ERROR, error);
 
     closesocket(priv->socket);
     priv->socket = INVALID_SOCKET;
@@ -719,7 +645,7 @@ infd_tcp_server_bind(InfdTcpServer* server,
 
   if(bind(priv->socket, addr, addrlen) == -1)
   {
-    infd_tcp_server_make_system_error(INFD_TCP_SERVER_LAST_ERROR, error);
+    inf_native_socket_make_error(INF_NATIVE_SOCKET_LAST_ERROR, error);
 
     closesocket(priv->socket);
     priv->socket = INVALID_SOCKET;
@@ -810,11 +736,13 @@ infd_tcp_server_open(InfdTcpServer* server,
     }
   }
 
+  /* TODO: Should move this code to InfNativeSocket, so that it can be
+   * shared with InfTcpConnection */
 #ifndef G_OS_WIN32
   result = fcntl(priv->socket, F_GETFL);
   if(result == -1)
   {
-    infd_tcp_server_make_system_error(INFD_TCP_SERVER_LAST_ERROR, error);
+    inf_native_socket_make_error(INF_NATIVE_SOCKET_LAST_ERROR, error);
     if(!was_bound)
       infd_tcp_server_close(server);
     g_object_thaw_notify(G_OBJECT(server));
@@ -823,7 +751,7 @@ infd_tcp_server_open(InfdTcpServer* server,
 
   if(fcntl(priv->socket, F_SETFL, result | O_NONBLOCK) == -1)
   {
-    infd_tcp_server_make_system_error(INFD_TCP_SERVER_LAST_ERROR, error);
+    inf_native_socket_make_error(INF_NATIVE_SOCKET_LAST_ERROR, error);
     if(!was_bound)
       infd_tcp_server_close(server);
     g_object_thaw_notify(G_OBJECT(server));
@@ -833,7 +761,7 @@ infd_tcp_server_open(InfdTcpServer* server,
   argp = 1;
   if(ioctlsocket(priv->socket, FIONBIO, &argp) != 0)
   {
-    infd_tcp_server_make_system_error(INFD_TCP_SERVER_LAST_ERROR, error);
+    inf_native_socket_make_error(INF_NATIVE_SOCKET_LAST_ERROR, error);
     if(!was_bound)
       infd_tcp_server_close(server);
     g_object_thaw_notify(G_OBJECT(server));
@@ -843,7 +771,7 @@ infd_tcp_server_open(InfdTcpServer* server,
 
   if(listen(priv->socket, 5) == -1)
   {
-    infd_tcp_server_make_system_error(INFD_TCP_SERVER_LAST_ERROR, error);
+    inf_native_socket_make_error(INF_NATIVE_SOCKET_LAST_ERROR, error);
     if(!was_bound)
       infd_tcp_server_close(server);
     g_object_thaw_notify(G_OBJECT(server));
@@ -897,4 +825,37 @@ infd_tcp_server_close(InfdTcpServer* server)
   g_object_notify(G_OBJECT(server), "status");
 }
 
+/**
+ * infd_tcp_server_set_keepalive:
+ * @server: A #InfdTcpServer.
+ * @keepalive: The keepalive settings for accepted connections.
+ *
+ * Sets the keepalive settings for new connections accepted by the server.
+ */
+void
+infd_tcp_server_set_keepalive(InfdTcpServer* server,
+                              const InfKeepalive* keepalive)
+{
+  g_return_if_fail(INFD_IS_TCP_SERVER(server));
+  g_return_if_fail(keepalive != NULL);
+
+  INFD_TCP_SERVER_PRIVATE(server)->keepalive = *keepalive;
+}
+
+/**
+ * infd_tcp_server_get_keepalive:
+ * @server: A #InfdTcpServer.
+ *
+ * Obtains the current keepalive settings for accepted connections.
+ *
+ * Returns: A #InfKeepalive representing the keepalive configuration for
+ * accepted connections, owned by @server.
+ */
+const InfKeepalive*
+infd_tcp_server_get_keepalive(InfdTcpServer* server)
+{
+  g_return_val_if_fail(INFD_IS_TCP_SERVER(server), NULL);
+  return &INFD_TCP_SERVER_PRIVATE(server)->keepalive;
+}
+
 /* vim:set et sw=2 ts=2: */
