@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,14 +17,6 @@
  * MA 02110-1301, USA.
  */
 
-#include <libinftext/inf-text-undo-grouping.h>
-#include <libinftext/inf-text-default-insert-operation.h>
-#include <libinftext/inf-text-default-delete-operation.h>
-#include <libinftext/inf-text-insert-operation.h>
-#include <libinftext/inf-text-delete-operation.h>
-#include <libinftext/inf-text-move-operation.h>
-#include <libinftext/inf-text-chunk.h>
-
 /**
  * SECTION:inf-text-undo-grouping
  * @title:InfTextUndoGrouping
@@ -42,9 +34,17 @@
  * #InfAdoptedUndoGrouping::group-requests to perform the grouping.
  */
 
+#include <libinftext/inf-text-undo-grouping.h>
+#include <libinftext/inf-text-default-insert-operation.h>
+#include <libinftext/inf-text-default-delete-operation.h>
+#include <libinftext/inf-text-insert-operation.h>
+#include <libinftext/inf-text-delete-operation.h>
+#include <libinftext/inf-text-move-operation.h>
+#include <libinftext/inf-text-chunk.h>
+
 #define INF_TEXT_UNDO_GROUPING_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), INF_TEXT_TYPE_UNDO_GROUPING, InfTextUndoGroupingPrivate))
 
-static InfAdoptedUndoGroupingClass* parent_class;
+G_DEFINE_TYPE(InfTextUndoGrouping, inf_text_undo_grouping, INF_ADOPTED_TYPE_UNDO_GROUPING)
 
 /* Returns the gunichar of the first character of a InfTextChunk */
 static gunichar
@@ -62,7 +62,7 @@ inf_text_undo_grouping_get_char_from_chunk(InfTextChunk* chunk)
   cd = g_iconv_open("UTF-8", inf_text_chunk_get_encoding(chunk));
   g_assert(cd != (GIConv)-1);
 
-  inf_text_chunk_iter_init(chunk, &iter);
+  inf_text_chunk_iter_init_begin(chunk, &iter);
   /* cast const away without warning */ /* more or less */
   *(gconstpointer*) &inbuf = inf_text_chunk_iter_get_text(&iter);
   inlen = inf_text_chunk_iter_get_bytes(&iter);
@@ -87,6 +87,9 @@ inf_text_undo_grouping_get_translated_position(InfAdoptedAlgorithm* algorithm,
   InfAdoptedOperation* move_op;
   InfAdoptedStateVector* move_vec;
   InfAdoptedRequest* move_req;
+
+  InfAdoptedRequest* moved_req;
+  InfAdoptedOperation* moved_op;
   guint move_pos;
 
   move_id = inf_adopted_request_get_user_id(from);
@@ -97,29 +100,32 @@ inf_text_undo_grouping_get_translated_position(InfAdoptedAlgorithm* algorithm,
   inf_adopted_state_vector_set(
     move_vec,
     move_id,
-    inf_adopted_state_vector_get(inf_adopted_request_get_vector(to), move_id)
+    inf_adopted_request_get_index(to)
+  );
+  move_req = inf_adopted_request_new_do(
+    move_vec,
+    move_id,
+    move_op,
+    inf_adopted_request_get_receive_time(from)
   );
-  move_req = inf_adopted_request_new_do(move_vec, move_id, move_op);
-  inf_adopted_state_vector_free(move_vec);
 
-  /* TODO: I think that it might be possible that this translatation is not
-   * always possible as we might have already removed some request log entries
-   * in other user's request logs to do the transformation.
-   *
-   * Instead we should: Do the transformation if all requests are available
-   * If not, check whether to follows from, in which case no transformation is
-   * necessary. If that is not the case, we can't decide,
-   * so don't group. */
-  inf_adopted_algorithm_translate_request(
+  inf_adopted_state_vector_free(move_vec);
+  g_object_unref(move_op);
+
+  /* This should always succeed because of the vdiff check in
+   * inf_text_undo_grouping_group_requests(). */
+  moved_req = inf_adopted_algorithm_translate_request(
     algorithm,
     move_req,
     inf_adopted_request_get_vector(to)
   );
 
   g_object_unref(move_req);
+  moved_op = inf_adopted_request_get_operation(moved_req);
   move_pos =
-    inf_text_move_operation_get_position(INF_TEXT_MOVE_OPERATION(move_op));
-  g_object_unref(move_op);
+    inf_text_move_operation_get_position(INF_TEXT_MOVE_OPERATION(moved_op));
+
+  g_object_unref(moved_req);
   return move_pos;
 }
 
@@ -130,6 +136,11 @@ inf_text_undo_grouping_group_requests(InfAdoptedUndoGrouping* grouping,
 {
   InfAdoptedOperation* first_op;
   InfAdoptedOperation* second_op;
+  InfAdoptedStateVector* first_vector;
+  InfAdoptedStateVector* second_vector;
+  InfAdoptedAlgorithm* algorithm;
+  guint max_total_log_size;
+  guint vdiff;
   guint first_length;
   guint second_length;
   guint first_pos;
@@ -142,6 +153,24 @@ inf_text_undo_grouping_group_requests(InfAdoptedUndoGrouping* grouping,
   g_assert(inf_adopted_request_get_request_type(second) ==
            INF_ADOPTED_REQUEST_DO);
 
+  /* We want to avoid doing excessive transformations so that if someone
+   * types a character, waits a long time with others writing a lot in the
+   * meantime, and then writes another character. This also avoids the
+   * problem  that we might not have all the requests from other users
+   * around anymore to do the transformation. */
+  first_vector = inf_adopted_request_get_vector(first);
+  second_vector = inf_adopted_request_get_vector(second);
+  vdiff = inf_adopted_state_vector_vdiff(first_vector, second_vector);
+  algorithm = inf_adopted_undo_grouping_get_algorithm(grouping);
+  g_object_get(
+    G_OBJECT(algorithm),
+    "max-total-log-size", &max_total_log_size,
+    NULL
+  );
+
+  if(vdiff > max_total_log_size)
+    return FALSE;
+
   first_op = inf_adopted_request_get_operation(first);
   second_op = inf_adopted_request_get_operation(second);
 
@@ -270,55 +299,21 @@ inf_text_undo_grouping_group_requests(InfAdoptedUndoGrouping* grouping,
   }
 }
 
-/*
- * GType registration.
- */
-
 static void
-inf_text_undo_grouping_class_init(gpointer g_class,
-                                  gpointer class_data)
+inf_text_undo_grouping_init(InfTextUndoGrouping* grouping)
 {
-  GObjectClass* object_class;
-  InfAdoptedUndoGroupingClass* undo_grouping_class;
-
-  object_class = G_OBJECT_CLASS(g_class);
-  undo_grouping_class = INF_ADOPTED_UNDO_GROUPING_CLASS(g_class);
-
-  parent_class =
-    INF_ADOPTED_UNDO_GROUPING_CLASS(g_type_class_peek_parent(g_class));
-
-  undo_grouping_class->group_requests = inf_text_undo_grouping_group_requests;
 }
 
-GType
-inf_text_undo_grouping_get_type(void)
+static void
+inf_text_undo_grouping_class_init(
+  InfTextUndoGroupingClass* text_undo_grouping_class)
 {
-  static GType undo_grouping_type = 0;
+  InfAdoptedUndoGroupingClass* undo_grouping_class;
 
-  if(!undo_grouping_type)
-  {
-    static const GTypeInfo undo_grouping_type_info = {
-      sizeof(InfTextUndoGroupingClass),   /* class_size */
-      NULL,                              /* base_init */
-      NULL,                              /* base_finalize */
-      inf_text_undo_grouping_class_init,  /* class_init */
-      NULL,                              /* class_finalize */
-      NULL,                              /* class_data */
-      sizeof(InfTextUndoGrouping),        /* instance_size */
-      0,                                 /* n_preallocs */
-      NULL,                              /* instance_init */
-      NULL                               /* value_table */
-    };
+  undo_grouping_class =
+    INF_ADOPTED_UNDO_GROUPING_CLASS(text_undo_grouping_class);
 
-    undo_grouping_type = g_type_register_static(
-      INF_ADOPTED_TYPE_UNDO_GROUPING,
-      "InfTextUndoGrouping",
-      &undo_grouping_type_info,
-      0
-    );
-  }
-
-  return undo_grouping_type;
+  undo_grouping_class->group_requests = inf_text_undo_grouping_group_requests;
 }
 
 /*
@@ -326,11 +321,12 @@ inf_text_undo_grouping_get_type(void)
  */
 
 /**
- * inf_text_undo_grouping_new:
+ * inf_text_undo_grouping_new: (constructor)
  *
  * Creates a new #InfTextUndoGrouping.
  *
- * Return Value: A new #InfTextUndoGrouping. To be freed with g_object_unref().
+ * Returns: (transfer full): A new #InfTextUndoGrouping. To be freed with
+ * g_object_unref().
  **/
 InfTextUndoGrouping*
 inf_text_undo_grouping_new(void)
