@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2014 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,6 +25,7 @@
 #include <libinfinity/common/inf-standalone-io.h>
 #include <libinfinity/common/inf-io.h>
 #include <libinfinity/common/inf-protocol.h>
+#include <libinfinity/common/inf-init.h>
 
 #include <string.h>
 
@@ -325,8 +326,12 @@ main(int argc, char* argv[])
   InfTcpConnection* tcp_conn;
   GError* error;
 
-  gnutls_global_init();
-  g_type_init();
+  error = NULL;
+  if(!inf_init(&error))
+  {
+    fprintf(stderr, "%s\n", error->message);
+    return 1;
+  }
 
   test.io = inf_standalone_io_new();
 #ifndef G_OS_WIN32
@@ -334,7 +339,6 @@ main(int argc, char* argv[])
 #endif
   address = inf_ip_address_new_loopback4();
 
-  error = NULL;
   tcp_conn =
     inf_tcp_connection_new_and_open(INF_IO(test.io), address, inf_protocol_get_default_port(), &error);
 
