@@ -1,5 +1,5 @@
 /* libinfinity - a GObject-based infinote implementation
- * Copyright (C) 2007-2011 Armin Burgmeier <armin@arbur.net>
+ * Copyright (C) 2007-2015 Armin Burgmeier <armin@arbur.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -29,10 +29,11 @@
 #include <libinfinity/common/inf-xmpp-manager.h>
 #include <libinfinity/common/inf-discovery-avahi.h>
 #include <libinfinity/common/inf-chat-session.h>
+#include <libinfinity/common/inf-request-result.h>
 #include <libinfinity/common/inf-error.h>
 #include <libinfinity/common/inf-protocol.h>
-#include <libinfinity/inf-signals.h>
 #include <libinfinity/common/inf-init.h>
+#include <libinfinity/inf-signals.h>
 
 #include <gtk/gtk.h>
 
@@ -45,9 +46,9 @@ struct _InfTestGtkBrowserWindow {
   InfTextGtkBuffer* buffer;
   InfTextGtkView* view;
   InfTextGtkViewport* viewport;
-  InfcSessionProxy* proxy;
+  InfSessionProxy* proxy;
   InfUser* user;
-  InfcUserRequest* request;
+  InfRequest* request;
 };
 
 typedef struct _InfTestGtkBrowserChatWindow InfTestGtkBrowserChatWindow;
@@ -56,17 +57,18 @@ struct _InfTestGtkBrowserChatWindow {
   GtkWidget* status;
 
   InfChatBuffer* buffer;
-  InfcSessionProxy* proxy;
+  InfSessionProxy* proxy;
   InfUser* user;
-  InfcUserRequest* request;
+  InfRequest* request;
 };
 
 static InfSession*
 inf_test_gtk_browser_session_new(InfIo* io,
                                  InfCommunicationManager* manager,
                                  InfSessionStatus status,
-                                 InfCommunicationJoinedGroup* sync_group,
+                                 InfCommunicationGroup* sync_group,
                                  InfXmlConnection* sync_connection,
+                                 const gchar* path,
                                  gpointer user_data)
 {
   InfTextGtkBuffer* buffer;
@@ -84,7 +86,7 @@ inf_test_gtk_browser_session_new(InfIo* io,
     io,
     user_table,
     status,
-    INF_COMMUNICATION_GROUP(sync_group),
+    sync_group,
     sync_connection
   );
 
@@ -123,11 +125,18 @@ on_undo_button_clicked(GtkButton* button,
                        gpointer user_data)
 {
   InfTestGtkBrowserWindow* test;
-  InfAdoptedSession* session;
+  InfSession* session;
 
   test = (InfTestGtkBrowserWindow*)user_data;
-  session = INF_ADOPTED_SESSION(infc_session_proxy_get_session(test->proxy));
-  inf_adopted_session_undo(session, INF_ADOPTED_USER(test->user), 1);
+  g_object_get(G_OBJECT(test->proxy), "session", &session, NULL);
+
+  inf_adopted_session_undo(
+    INF_ADOPTED_SESSION(session),
+    INF_ADOPTED_USER(test->user),
+    1
+  );
+
+  g_object_unref(session);
 }
 
 static void
@@ -135,11 +144,18 @@ on_redo_button_clicked(GtkButton* button,
                        gpointer user_data)
 {
   InfTestGtkBrowserWindow* test;
-  InfAdoptedSession* session;
+  InfSession* session;
 
   test = (InfTestGtkBrowserWindow*)user_data;
-  session = INF_ADOPTED_SESSION(infc_session_proxy_get_session(test->proxy));
-  inf_adopted_session_redo(session, INF_ADOPTED_USER(test->user), 1);
+  g_object_get(G_OBJECT(test->proxy), "session", &session, NULL);
+
+  inf_adopted_session_redo(
+    INF_ADOPTED_SESSION(session),
+    INF_ADOPTED_USER(test->user),
+    1
+  );
+
+  g_object_unref(session);
 }
 
 static void
@@ -166,100 +182,61 @@ on_can_redo_changed(InfAdoptedAlgorithm* algorithm,
   gtk_widget_set_sensitive(test->redo_button, can_redo);
 }
 
-static void
-on_chat_join_finished(InfcUserRequest* request,
-                      InfUser* user,
-                      gpointer user_data)
-{
-  InfTestGtkBrowserChatWindow* test;
-  gchar* text;
-
-  test = (InfTestGtkBrowserChatWindow*)user_data;
-  inf_gtk_chat_set_active_user(INF_GTK_CHAT(test->chat), user);
-
-  text = g_strdup_printf("Joined as %s", inf_user_get_name(user));
-  gtk_label_set_text(GTK_LABEL(test->status), text);
-  g_free(text);
-
-  test->user = user;
-  g_object_ref(user);
-
-  g_assert(test->request != NULL);
-  g_object_unref(test->request);
-  test->request = NULL;
-
-  /* Unfortunately, gtk_widget_grab_focus(test->chat) +
-   * gtk_container_set_focus_child() in inf_gtk_chat_set_active_user() does
-   * not do the job which is why I added this crappy API. */
-  gtk_widget_grab_focus(inf_gtk_chat_get_entry(INF_GTK_CHAT(test->chat)));
-}
-
-static void
-on_join_finished(InfcUserRequest* request,
-                 InfUser* user,
-                 gpointer user_data)
-{
-  InfTestGtkBrowserWindow* test;
-  InfAdoptedSession* session;
-  InfAdoptedAlgorithm* algorithm;
-  gboolean undo;
-  gboolean redo;
-
-  test = (InfTestGtkBrowserWindow*)user_data;
-
-  inf_text_gtk_buffer_set_active_user(test->buffer, INF_TEXT_USER(user));
-  inf_text_gtk_view_set_active_user(test->view, INF_TEXT_USER(user));
-  inf_text_gtk_viewport_set_active_user(test->viewport, INF_TEXT_USER(user));
-  gtk_text_view_set_editable(GTK_TEXT_VIEW(test->textview), TRUE);
-
-  test->user = user;
-  g_object_ref(user);
-
-  g_assert(test->request != NULL);
-  g_object_unref(test->request);
-  test->request = NULL;
-
-  session = INF_ADOPTED_SESSION(infc_session_proxy_get_session(test->proxy));
-  algorithm = inf_adopted_session_get_algorithm(session);
-
-  undo = inf_adopted_algorithm_can_undo(algorithm, INF_ADOPTED_USER(user));
-  redo = inf_adopted_algorithm_can_redo(algorithm, INF_ADOPTED_USER(user));
-
-  gtk_widget_set_sensitive(test->undo_button, undo);
-  gtk_widget_set_sensitive(test->redo_button, redo);
-}
-
 static void
 request_chat_join(InfTestGtkBrowserChatWindow* test,
                   const gchar* user_name);
 
 static void
-on_chat_join_failed(InfcRequest* request,
-                    const GError* error,
-                    gpointer user_data)
+on_chat_join_finished(InfRequest* request,
+                      const InfRequestResult* result,
+                      const GError* error,
+                      gpointer user_data)
 {
   InfTestGtkBrowserChatWindow* test;
+  InfUser* user;
   gchar* new_name;
   gchar* text;
 
   test = (InfTestGtkBrowserChatWindow*)user_data;
 
-  g_assert(test->request != NULL);
-  g_object_unref(test->request);
-  test->request = NULL;
-
-  if(error->domain == inf_user_error_quark() &&
-     error->code == INF_USER_ERROR_NAME_IN_USE)
+  if(test->request != NULL)
   {
-    new_name = g_strdup_printf("%s%d", g_get_user_name(), rand());
-    request_chat_join(test, new_name);
-    g_free(new_name);
+    g_object_unref(test->request);
+    test->request = NULL;
+  }
+
+  if(error == NULL)
+  {
+    inf_request_result_get_join_user(result, NULL, &user);
+    inf_gtk_chat_set_active_user(INF_GTK_CHAT(test->chat), user);
+
+    text = g_strdup_printf("Joined as %s", inf_user_get_name(user));
+    gtk_label_set_text(GTK_LABEL(test->status), text);
+    g_free(text);
+
+    test->user = user;
+    g_object_ref(user);
+
+    /* Unfortunately, gtk_widget_grab_focus(test->chat) +
+     * gtk_container_set_focus_child() in inf_gtk_chat_set_active_user() does
+     * not do the job which is why I added this crappy API. */
+    gtk_widget_grab_focus(inf_gtk_chat_get_entry(INF_GTK_CHAT(test->chat)));
   }
   else
   {
-    text = g_strdup_printf("User join failed: %s", error->message);
-    gtk_label_set_text(GTK_LABEL(test->status), text);
-    g_free(text);
+    if(error->domain == inf_user_error_quark() &&
+       error->code == INF_USER_ERROR_NAME_IN_USE)
+    {
+      new_name = g_strdup_printf("%s%d", g_get_user_name(), rand());
+      request_chat_join(test, new_name);
+      g_free(new_name);
+    }
+    else
+    {
+      text = g_strdup_printf("User join failed: %s", error->message);
+      gtk_label_set_text(GTK_LABEL(test->status), text);
+      g_free(text);
+    }
   }
 }
 
@@ -268,29 +245,65 @@ request_join(InfTestGtkBrowserWindow* test,
              const gchar* user_name);
 
 static void
-on_join_failed(InfcRequest* request,
-               const GError* error,
-               gpointer user_data)
+on_join_finished(InfRequest* request,
+                 const InfRequestResult* result,
+                 const GError* error,
+                 gpointer user_data)
 {
   InfTestGtkBrowserWindow* test;
+  InfUser* user;
+  InfSession* session;
+  InfAdoptedAlgorithm* algorithm;
+  gboolean undo;
+  gboolean redo;
   gchar* new_name;
 
   test = (InfTestGtkBrowserWindow*)user_data;
 
-  g_assert(test->request != NULL);
-  g_object_unref(test->request);
-  test->request = NULL;
-
-  if(error->domain == inf_user_error_quark() &&
-     error->code == INF_USER_ERROR_NAME_IN_USE)
+  if(test->request != NULL)
   {
-    new_name = g_strdup_printf("%s%d", g_get_user_name(), rand());
-    request_join(test, new_name);
-    g_free(new_name);
+    g_object_unref(test->request);
+    test->request = NULL;
+  }
+
+  if(error == NULL)
+  {
+    inf_request_result_get_join_user(result, NULL, &user);
+    inf_text_gtk_buffer_set_active_user(test->buffer, INF_TEXT_USER(user));
+    inf_text_gtk_view_set_active_user(test->view, INF_TEXT_USER(user));
+    inf_text_gtk_viewport_set_active_user(
+      test->viewport,
+      INF_TEXT_USER(user)
+    );
+    gtk_text_view_set_editable(GTK_TEXT_VIEW(test->textview), TRUE);
+
+    test->user = user;
+    g_object_ref(user);
+
+    g_object_get(G_OBJECT(test->proxy), "session", &session, NULL);
+    algorithm =
+      inf_adopted_session_get_algorithm(INF_ADOPTED_SESSION(session));
+    g_object_unref(session);
+
+    undo = inf_adopted_algorithm_can_undo(algorithm, INF_ADOPTED_USER(user));
+    redo = inf_adopted_algorithm_can_redo(algorithm, INF_ADOPTED_USER(user));
+
+    gtk_widget_set_sensitive(test->undo_button, undo);
+    gtk_widget_set_sensitive(test->redo_button, redo);
   }
   else
   {
-    set_error(test, "User join failed", error->message);
+    if(error->domain == inf_user_error_quark() &&
+       error->code == INF_USER_ERROR_NAME_IN_USE)
+    {
+      new_name = g_strdup_printf("%s%d", g_get_user_name(), rand());
+      request_join(test, new_name);
+      g_free(new_name);
+    }
+    else
+    {
+      set_error(test, "User join failed", error->message);
+    }
   }
 }
 
@@ -298,45 +311,29 @@ static void
 request_chat_join(InfTestGtkBrowserChatWindow* test,
                   const gchar* user_name)
 {
-  InfcUserRequest* request;
-  GError* error;
+  InfRequest* request;
   gchar* text;
 
   GParameter params[1] = { { "name", { 0 } } };
   g_value_init(&params[0].value, G_TYPE_STRING);
   g_value_set_static_string(&params[0].value, user_name);
 
-  error = NULL;
-  request = infc_session_proxy_join_user(test->proxy, params, 1, &error);
+  text = g_strdup_printf("Requesting user join for %s", user_name);
+  gtk_label_set_text(GTK_LABEL(test->status), text);
+  g_free(text);
+
+  request = inf_session_proxy_join_user(
+    test->proxy,
+    1,
+    params,
+    on_chat_join_finished,
+    test
+  );
+
   g_value_unset(&params[0].value);
 
-  if(request == NULL)
+  if(request != NULL)
   {
-    text = g_strdup_printf("User join failed: %s", error->message);
-    g_error_free(error);
-    gtk_label_set_text(GTK_LABEL(test->status), text);
-    g_free(text);
-  }
-  else
-  {
-    text = g_strdup_printf("Requesting user join for %s", user_name);
-    gtk_label_set_text(GTK_LABEL(test->status), text);
-    g_free(text);
-
-    g_signal_connect_after(
-      G_OBJECT(request),
-      "failed",
-      G_CALLBACK(on_chat_join_failed),
-      test
-    );
-
-    g_signal_connect_after(
-      G_OBJECT(request),
-      "finished",
-      G_CALLBACK(on_chat_join_finished),
-      test
-    );
-
     g_assert(test->request == NULL);
     test->request = request;
     g_object_ref(test->request);
@@ -347,9 +344,9 @@ static void
 request_join(InfTestGtkBrowserWindow* test,
              const gchar* user_name)
 {
-  InfcUserRequest* request;
+  InfRequest* request;
+  InfSession* session;
   InfAdoptedStateVector* v;
-  GError* error;
   GtkTextBuffer* buffer;
   GtkTextMark* mark;
   GtkTextIter iter;
@@ -365,12 +362,12 @@ request_join(InfTestGtkBrowserWindow* test,
 
   g_value_set_static_string(&params[0].value, user_name);
 
+  g_object_get(G_OBJECT(test->proxy), "session", &session, NULL);
+
   /* Use current state vector. Infinote should already do this. */
   v = inf_adopted_state_vector_copy(
     inf_adopted_algorithm_get_current(
-      inf_adopted_session_get_algorithm(
-        INF_ADOPTED_SESSION(infc_session_proxy_get_session(test->proxy))
-      )
+      inf_adopted_session_get_algorithm(INF_ADOPTED_SESSION(session))
     )
   );
 
@@ -381,31 +378,18 @@ request_join(InfTestGtkBrowserWindow* test,
   gtk_text_buffer_get_iter_at_mark(buffer, &iter, mark);
   g_value_set_uint(&params[2].value, gtk_text_iter_get_offset(&iter));
 
-  error = NULL;
-  request = infc_session_proxy_join_user(test->proxy, params, 3, &error);
+  request = inf_session_proxy_join_user(
+    test->proxy,
+    3,
+    params,
+    on_join_finished,
+    test
+  );
 
-  /* TODO: Free GValues? */
+  g_object_unref(session);
 
-  if(request == NULL)
+  if(request != NULL)
   {
-    set_error(test, "Failed to request user join", error->message);
-  }
-  else
-  {
-    g_signal_connect_after(
-      G_OBJECT(request),
-      "failed",
-      G_CALLBACK(on_join_failed),
-      test
-    );
-
-    g_signal_connect_after(
-      G_OBJECT(request),
-      "finished",
-      G_CALLBACK(on_join_finished),
-      test
-    );
-
     g_assert(test->request == NULL);
     test->request = request;
     g_object_ref(request);
@@ -459,7 +443,6 @@ on_synchronization_complete(InfSession* session,
   InfAdoptedAlgorithm* algorithm;
 
   test = (InfTestGtkBrowserWindow*)user_data;
-  session = infc_session_proxy_get_session(test->proxy);
   algorithm = inf_adopted_session_get_algorithm(INF_ADOPTED_SESSION(session));
 
   g_signal_connect(
@@ -487,7 +470,7 @@ on_chat_window_destroy(GtkWindow* window,
   InfSession* session;
 
   test = (InfTestGtkBrowserChatWindow*)user_data;
-  session = infc_session_proxy_get_session(test->proxy);
+  g_object_get(G_OBJECT(test->proxy), "session", &session, NULL);
 
   inf_signal_handlers_disconnect_by_func(
     session,
@@ -501,6 +484,8 @@ on_chat_window_destroy(GtkWindow* window,
     test
   );
 
+  g_object_unref(session);
+
   if(test->request != NULL)
   {
     inf_signal_handlers_disconnect_by_func(
@@ -509,12 +494,6 @@ on_chat_window_destroy(GtkWindow* window,
       test
     );
 
-    inf_signal_handlers_disconnect_by_func(
-      test->request,
-      G_CALLBACK(on_chat_join_failed),
-      test
-    );
-
     g_object_unref(test->request);
   }
 
@@ -532,7 +511,7 @@ on_text_window_destroy(GtkWindow* window,
   InfSession* session;
 
   test = (InfTestGtkBrowserWindow*)user_data;
-  session = infc_session_proxy_get_session(test->proxy);
+  g_object_get(G_OBJECT(test->proxy), "session", &session, NULL);
 
   inf_signal_handlers_disconnect_by_func(
     session,
@@ -546,6 +525,8 @@ on_text_window_destroy(GtkWindow* window,
     test
   );
 
+  g_object_unref(session);
+
   if(test->request != NULL)
   {
     inf_signal_handlers_disconnect_by_func(
@@ -554,12 +535,6 @@ on_text_window_destroy(GtkWindow* window,
       test
     );
 
-    inf_signal_handlers_disconnect_by_func(
-      test->request,
-      G_CALLBACK(on_join_failed),
-      test
-    );
-
     g_object_unref(test->request);
   }
 
@@ -584,7 +559,7 @@ on_subscribe_chat_session(InfcBrowser* browser,
   GtkWidget* window;
   InfTestGtkBrowserChatWindow* test;
 
-  session = infc_session_proxy_get_session(proxy);
+  g_object_get(G_OBJECT(proxy), "session", &session, NULL);
   buffer = INF_CHAT_BUFFER(inf_session_get_buffer(session));
 
   chat = inf_gtk_chat_new();
@@ -594,7 +569,7 @@ on_subscribe_chat_session(InfcBrowser* browser,
   status = gtk_label_new("Synchronizing chat...");
   gtk_widget_show(status);
 
-  vbox = gtk_vbox_new(FALSE, 6);
+  vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 6);
   gtk_box_pack_start(GTK_BOX(vbox), chat, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), status, FALSE, TRUE, 0);
   gtk_widget_show(vbox);
@@ -612,7 +587,7 @@ on_subscribe_chat_session(InfcBrowser* browser,
   test->chat = chat;
   test->status = status;
   test->buffer = buffer;
-  test->proxy = proxy;
+  test->proxy = INF_SESSION_PROXY(proxy);
   test->user = NULL;
   test->request = NULL;
   g_object_ref(test->proxy);
@@ -637,12 +612,15 @@ on_subscribe_chat_session(InfcBrowser* browser,
     G_CALLBACK(on_chat_window_destroy),
     test
   );
+
+  g_object_unref(session);
 }
 
 static void
 on_subscribe_session(InfcBrowser* browser,
-                     InfcBrowserIter* iter,
+                     const InfBrowserIter* iter,
                      InfcSessionProxy* proxy,
+                     InfRequest* request,
                      gpointer user_data)
 {
   GtkWidget* window;
@@ -667,7 +645,7 @@ on_subscribe_session(InfcBrowser* browser,
     return;
   }
 
-  session = infc_session_proxy_get_session(proxy);
+  g_object_get(G_OBJECT(proxy), "session", &session, NULL);
   io = inf_adopted_session_get_io(INF_ADOPTED_SESSION(session));
   user_table = inf_session_get_user_table(session);
   buffer = INF_TEXT_GTK_BUFFER(inf_session_get_buffer(session));
@@ -694,19 +672,19 @@ on_subscribe_session(InfcBrowser* browser,
   gtk_container_add(GTK_CONTAINER(scroll), textview);
   gtk_widget_show(scroll);
 
-  undo_button = gtk_button_new_from_stock(GTK_STOCK_UNDO);
-  redo_button = gtk_button_new_from_stock(GTK_STOCK_REDO);
+  undo_button = gtk_button_new_with_label("Undo");
+  redo_button = gtk_button_new_with_label("Redo");
   gtk_widget_set_sensitive(undo_button, FALSE);
   gtk_widget_set_sensitive(redo_button, FALSE);
   gtk_widget_show(undo_button);
   gtk_widget_show(redo_button);
 
-  hbox = gtk_hbutton_box_new();
+  hbox = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
   gtk_box_pack_start(GTK_BOX(hbox), undo_button, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), redo_button, FALSE, FALSE, 0);
   gtk_widget_show(hbox);
 
-  vbox = gtk_vbox_new(FALSE, 6);
+  vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 6);
   gtk_box_pack_start(GTK_BOX(vbox), scroll, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
   gtk_widget_show(vbox);
@@ -715,7 +693,7 @@ on_subscribe_session(InfcBrowser* browser,
 
   gtk_window_set_title(
     GTK_WINDOW(window),
-    infc_browser_iter_get_name(browser, iter)
+    inf_browser_get_node_name(INF_BROWSER(browser), iter)
   );
 
   gtk_window_set_default_size(GTK_WINDOW(window), 400, 400);
@@ -731,7 +709,7 @@ on_subscribe_session(InfcBrowser* browser,
   test->buffer = buffer;
   test->view = view;
   test->viewport = viewport;
-  test->proxy = proxy;
+  test->proxy = INF_SESSION_PROXY(proxy);
   test->user = NULL;
   test->request = NULL;
   g_object_ref(proxy);
@@ -770,6 +748,8 @@ on_subscribe_session(InfcBrowser* browser,
     G_CALLBACK(on_redo_button_clicked),
     test
   );
+
+  g_object_unref(session);
 }
 
 static void
@@ -777,8 +757,13 @@ on_activate(InfGtkBrowserView* view,
             GtkTreeIter* iter,
             gpointer user_data)
 {
-  InfcBrowser* browser;
-  InfcBrowserIter* browser_iter;
+  InfBrowser* browser;
+  InfBrowserIter* browser_iter;
+  const char* note_type;
+  const InfcNotePlugin* plugin;
+  InfRequest* request;
+
+  printf("activate\n");
 
   gtk_tree_model_get(
     GTK_TREE_MODEL(gtk_tree_view_get_model(GTK_TREE_VIEW(view))),
@@ -788,15 +773,23 @@ on_activate(InfGtkBrowserView* view,
     -1
   );
 
+  note_type = inf_browser_get_node_type(browser, browser_iter);
+  plugin = infc_browser_lookup_plugin(INFC_BROWSER(browser), note_type);
+
   /* Subscribe, if possible and not already */
-  if(!infc_browser_iter_get_session(browser, browser_iter) &&
-     !infc_browser_iter_get_subscribe_request(browser, browser_iter) &&
-     infc_browser_iter_get_plugin(browser, browser_iter) != NULL)
+  request = inf_browser_get_pending_request(
+    browser,
+    browser_iter,
+    "subscribe-session"
+  );
+
+  if(!inf_browser_get_session(browser, browser_iter) &&
+     !request && plugin != NULL)
   {
-    infc_browser_iter_subscribe_session(browser, browser_iter);
+    inf_browser_subscribe(browser, browser_iter, NULL, NULL);
   }
 
-  infc_browser_iter_free(browser_iter);
+  inf_browser_iter_free(browser_iter);
   g_object_unref(browser);
 }
 
@@ -806,22 +799,30 @@ on_browser_notify_status(GObject* object,
                          gpointer user_data)
 {
   InfcBrowser* browser;
+  InfBrowserStatus status;
   browser = INFC_BROWSER(object);
 
-  if(infc_browser_get_status(browser) == INFC_BROWSER_CONNECTED)
-    infc_browser_subscribe_chat(browser);
+  g_object_get(G_OBJECT(browser), "status", &status, NULL);
+  if(status == INF_BROWSER_OPEN)
+    infc_browser_subscribe_chat(browser, NULL, NULL);
 }
 
 static void
 on_set_browser(InfGtkBrowserModel* model,
                GtkTreePath* path,
                GtkTreeIter* iter,
-               InfcBrowser* browser,
+               InfBrowser* old_browser,
+               InfBrowser* browser,
                gpointer user_data)
 {
+  InfBrowserStatus status;
+
   if(browser != NULL)
   {
-    infc_browser_add_plugin(browser, &INF_TEST_GTK_BROWSER_TEXT_PLUGIN);
+    infc_browser_add_plugin(
+      INFC_BROWSER(browser),
+      &INF_TEST_GTK_BROWSER_TEXT_PLUGIN
+    );
 
     g_signal_connect_after(
       G_OBJECT(browser),
@@ -830,9 +831,10 @@ on_set_browser(InfGtkBrowserModel* model,
       NULL
     );
 
-    if(infc_browser_get_status(browser) == INFC_BROWSER_CONNECTED)
+    g_object_get(G_OBJECT(browser), "status", &status, NULL);
+    if(status == INF_BROWSER_OPEN)
     {
-      infc_browser_subscribe_chat(browser);
+      infc_browser_subscribe_chat(INFC_BROWSER(browser), NULL, NULL);
     }
     else
     {
